@@ -38,7 +38,7 @@
  *      only one thread is enabled, it equals 1.
  */
 
-/* thread info flags, for thread_info[].flags */
+/* thread info flags, for ha_thread_info[].flags */
 #define TI_FL_STUCK             0x00000001
 
 
@@ -67,7 +67,7 @@ extern struct thread_info {
 	/* pad to cache line (64B) */
 	char __pad[0];            /* unused except to check remaining room */
 	char __end[0] __attribute__((aligned(64)));
-} thread_info[MAX_THREADS];
+} ha_thread_info[MAX_THREADS];
 
 extern THREAD_LOCAL struct thread_info *ti; /* thread_info for the current thread */
 
@@ -77,12 +77,7 @@ extern THREAD_LOCAL struct thread_info *ti; /* thread_info for the current threa
 #define __decl_rwlock(lock)
 #define __decl_aligned_rwlock(lock)
 
-#define HA_ATOMIC_CAS(val, old, new)                                    \
-	({                                                              \
-		typeof(val) _v = (val);                                 \
-		typeof(old) _o = (old);                                 \
-		(*_v == *_o) ? ((*_v = (new)), 1) : ((*_o = *_v), 0);   \
-	})
+#define HA_ATOMIC_CAS(val, old, new) ({((*val) == (*old)) ? (*(val) = (new) , 1) : (*(old) = *(val), 0);})
 
 /* warning, n is a pointer to the double value for dwcas */
 #define HA_ATOMIC_DWCAS(val, o, n)				       \
@@ -113,23 +108,6 @@ extern THREAD_LOCAL struct thread_info *ti; /* thread_info for the current threa
 		*(val) = new;						\
 		__old_xchg;						\
 	})
-
-#define HA_ATOMIC_FETCH_AND(val, i)					\
-	({								\
-		typeof((val)) __p_val = (val);				\
-		typeof(*(val)) __old_val = *__p_val;			\
-		*__p_val &= (i);					\
-		__old_val;						\
-	})
-
-#define HA_ATOMIC_FETCH_OR(val, i)					\
-	({								\
-		typeof((val)) __p_val = (val);				\
-		typeof(*(val)) __old_val = *__p_val;			\
-		*__p_val |= (i);					\
-		__old_val;						\
-	})
-
 #define HA_ATOMIC_BTS(val, bit)						\
 	({								\
 		typeof((val)) __p_bts = (val);				\
@@ -152,22 +130,20 @@ extern THREAD_LOCAL struct thread_info *ti; /* thread_info for the current threa
 #define HA_ATOMIC_STORE(val, new)    ({*(val) = new;})
 #define HA_ATOMIC_UPDATE_MAX(val, new)					\
 	({								\
-		typeof(val) __val = (val);                              \
 		typeof(*(val)) __new_max = (new);			\
 									\
-		if (*__val < __new_max)					\
-			*__val = __new_max;				\
-		*__val;							\
+		if (*(val) < __new_max)					\
+			*(val) = __new_max;				\
+		*(val);							\
 	})
 
 #define HA_ATOMIC_UPDATE_MIN(val, new)					\
 	({								\
-		typeof(val) __val = (val);                              \
 		typeof(*(val)) __new_min = (new);			\
 									\
-		if (*__val > __new_min)					\
-			*__val = __new_min;				\
-		*__val;							\
+		if (*(val) > __new_min)					\
+			*(val) = __new_min;				\
+		*(val);							\
 	})
 
 #define HA_BARRIER() do { } while (0)
@@ -191,12 +167,7 @@ extern THREAD_LOCAL struct thread_info *ti; /* thread_info for the current threa
 
 static inline void ha_set_tid(unsigned int tid)
 {
-	ti = &thread_info[tid];
-}
-
-static inline unsigned long long ha_get_pthread_id(unsigned int thr)
-{
-	return 0;
+	ti = &ha_thread_info[tid];
 }
 
 static inline void ha_thread_relax(void)
@@ -314,8 +285,6 @@ static inline unsigned long thread_isolated()
 #define HA_ATOMIC_XADD(val, i)       __sync_fetch_and_add(val, i)
 #define HA_ATOMIC_AND(val, flags)    __sync_and_and_fetch(val, flags)
 #define HA_ATOMIC_OR(val, flags)     __sync_or_and_fetch(val,  flags)
-#define HA_ATOMIC_FETCH_AND(val, flags) __sync_fetch_and_and(val, flags)
-#define HA_ATOMIC_FETCH_OR(val, flags)  __sync_fetch_and_or(val,  flags)
 
 /* the CAS is a bit complicated. The older API doesn't support returning the
  * value and the swap's result at the same time. So here we take what looks
@@ -393,8 +362,6 @@ static inline unsigned long thread_isolated()
 #define HA_ATOMIC_SUB(val, i)        __atomic_sub_fetch(val, i, __ATOMIC_SEQ_CST)
 #define HA_ATOMIC_AND(val, flags)    __atomic_and_fetch(val, flags, __ATOMIC_SEQ_CST)
 #define HA_ATOMIC_OR(val, flags)     __atomic_or_fetch(val,  flags, __ATOMIC_SEQ_CST)
-#define HA_ATOMIC_FETCH_AND(val, flags) __atomic_fetch_and(val, flags, __ATOMIC_SEQ_CST)
-#define HA_ATOMIC_FETCH_OR(val, flags)  __atomic_fetch_or(val,  flags, __ATOMIC_SEQ_CST)
 #define HA_ATOMIC_BTS(val, bit)						\
 	({								\
 		typeof(*(val)) __b_bts = (1UL << (bit));		\
@@ -433,23 +400,21 @@ static inline unsigned long thread_isolated()
 
 #define HA_ATOMIC_UPDATE_MAX(val, new)					\
 	({								\
-		typeof(val) __val = (val);                              \
-		typeof(*(val)) __old_max = *__val;			\
+		typeof(*(val)) __old_max = *(val);			\
 		typeof(*(val)) __new_max = (new);			\
 									\
 		while (__old_max < __new_max &&				\
-		       !HA_ATOMIC_CAS(__val, &__old_max, __new_max));	\
-		*__val;							\
+		       !HA_ATOMIC_CAS(val, &__old_max, __new_max));	\
+		*(val);							\
 	})
 #define HA_ATOMIC_UPDATE_MIN(val, new)					\
 	({								\
-		typeof(val) __val = (val);                              \
-		typeof(*(val)) __old_min = *__val;			\
+		typeof(*(val)) __old_min = *(val);			\
 		typeof(*(val)) __new_min = (new);			\
 									\
 		while (__old_min > __new_min &&				\
-		       !HA_ATOMIC_CAS(__val, &__old_min, __new_min));	\
-		*__val;							\
+		       !HA_ATOMIC_CAS(val, &__old_min, __new_min));	\
+		*(val);							\
 	})
 
 #define HA_BARRIER() pl_barrier()
@@ -472,7 +437,7 @@ extern struct thread_info {
 	/* pad to cache line (64B) */
 	char __pad[0];            /* unused except to check remaining room */
 	char __end[0] __attribute__((aligned(64)));
-} thread_info[MAX_THREADS];
+} ha_thread_info[MAX_THREADS];
 
 extern THREAD_LOCAL unsigned int tid;     /* The thread id */
 extern THREAD_LOCAL unsigned long tid_bit; /* The bit corresponding to the thread id */
@@ -515,38 +480,7 @@ static inline void ha_set_tid(unsigned int data)
 {
 	tid     = data;
 	tid_bit = (1UL << tid);
-	ti      = &thread_info[tid];
-}
-
-/* Retrieves the opaque pthread_t of thread <thr> cast to an unsigned long long
- * since POSIX took great care of not specifying its representation, making it
- * hard to export for post-mortem analysis. For this reason we copy it into a
- * union and will use the smallest scalar type at least as large as its size,
- * which will keep endianness and alignment for all regular sizes. As a last
- * resort we end up with a long long ligned to the first bytes in memory, which
- * will be endian-dependent if pthread_t is larger than a long long (not seen
- * yet).
- */
-static inline unsigned long long ha_get_pthread_id(unsigned int thr)
-{
-	union {
-		pthread_t t;
-		unsigned long long ll;
-		unsigned int i;
-		unsigned short s;
-		unsigned char c;
-	} u;
-
-	memset(&u, 0, sizeof(u));
-	u.t = thread_info[thr].pthread;
-
-	if (sizeof(u.t) <= sizeof(u.c))
-		return u.c;
-	else if (sizeof(u.t) <= sizeof(u.s))
-		return u.s;
-	else if (sizeof(u.t) <= sizeof(u.i))
-		return u.i;
-	return u.ll;
+	ti      = &ha_thread_info[tid];
 }
 
 static inline void ha_thread_relax(void)
@@ -578,7 +512,7 @@ static inline void thread_harmless_end()
 {
 	while (1) {
 		HA_ATOMIC_AND(&threads_harmless_mask, ~tid_bit);
-		if (likely((threads_want_rdv_mask & ~tid_bit) == 0))
+		if (likely((threads_want_rdv_mask & all_threads_mask) == 0))
 			break;
 		thread_harmless_till_end();
 	}
@@ -628,6 +562,8 @@ enum lock_label {
 	LOGSRV_LOCK,
 	DICT_LOCK,
 	PROTO_LOCK,
+	CKCH_LOCK,
+	SNI_LOCK,
 	OTHER_LOCK,
 	LOCK_LABELS
 };
@@ -647,7 +583,7 @@ extern struct lock_stat lock_stats[LOCK_LABELS];
 #define __SPIN_INIT(l)         ({ (*l) = 0; })
 #define __SPIN_DESTROY(l)      ({ (*l) = 0; })
 #define __SPIN_LOCK(l)         pl_take_s(l)
-#define __SPIN_TRYLOCK(l)      (!pl_try_s(l))
+#define __SPIN_TRYLOCK(l)      !pl_try_s(l)
 #define __SPIN_UNLOCK(l)       pl_drop_s(l)
 
 #define __HA_RWLOCK_T		unsigned long
@@ -655,10 +591,10 @@ extern struct lock_stat lock_stats[LOCK_LABELS];
 #define __RWLOCK_INIT(l)          ({ (*l) = 0; })
 #define __RWLOCK_DESTROY(l)       ({ (*l) = 0; })
 #define __RWLOCK_WRLOCK(l)        pl_take_w(l)
-#define __RWLOCK_TRYWRLOCK(l)     (!pl_try_w(l))
+#define __RWLOCK_TRYWRLOCK(l)     !pl_try_w(l)
 #define __RWLOCK_WRUNLOCK(l)      pl_drop_w(l)
 #define __RWLOCK_RDLOCK(l)        pl_take_r(l)
-#define __RWLOCK_TRYRDLOCK(l)     (!pl_try_r(l))
+#define __RWLOCK_TRYRDLOCK(l)     !pl_try_r(l)
 #define __RWLOCK_RDUNLOCK(l)      pl_drop_r(l)
 
 #define HA_SPINLOCK_T       struct ha_spinlock
@@ -745,6 +681,8 @@ static inline const char *lock_label(enum lock_label label)
 	case LOGSRV_LOCK:          return "LOGSRV";
 	case DICT_LOCK:            return "DICT";
 	case PROTO_LOCK:           return "PROTO";
+	case CKCH_LOCK:            return "CKCH";
+	case SNI_LOCK:             return "SNI";
 	case OTHER_LOCK:           return "OTHER";
 	case LOCK_LABELS:          break; /* keep compiler happy */
 	};
@@ -1044,7 +982,7 @@ static inline void __spin_unlock(enum lock_label lbl, struct ha_spinlock *l,
 #define HA_SPIN_INIT(l)         ({ (*l) = 0; })
 #define HA_SPIN_DESTROY(l)      ({ (*l) = 0; })
 #define HA_SPIN_LOCK(lbl, l)    pl_take_s(l)
-#define HA_SPIN_TRYLOCK(lbl, l) (!pl_try_s(l))
+#define HA_SPIN_TRYLOCK(lbl, l) !pl_try_s(l)
 #define HA_SPIN_UNLOCK(lbl, l)  pl_drop_s(l)
 
 #define HA_RWLOCK_T		unsigned long
@@ -1052,10 +990,10 @@ static inline void __spin_unlock(enum lock_label lbl, struct ha_spinlock *l,
 #define HA_RWLOCK_INIT(l)          ({ (*l) = 0; })
 #define HA_RWLOCK_DESTROY(l)       ({ (*l) = 0; })
 #define HA_RWLOCK_WRLOCK(lbl,l)    pl_take_w(l)
-#define HA_RWLOCK_TRYWRLOCK(lbl,l) (!pl_try_w(l))
+#define HA_RWLOCK_TRYWRLOCK(lbl,l) !pl_try_w(l)
 #define HA_RWLOCK_WRUNLOCK(lbl,l)  pl_drop_w(l)
 #define HA_RWLOCK_RDLOCK(lbl,l)    pl_take_r(l)
-#define HA_RWLOCK_TRYRDLOCK(lbl,l) (!pl_try_r(l))
+#define HA_RWLOCK_TRYRDLOCK(lbl,l) !pl_try_r(l)
 #define HA_RWLOCK_RDUNLOCK(lbl,l)  pl_drop_r(l)
 
 #endif  /* DEBUG_THREAD */
@@ -1228,17 +1166,17 @@ static __inline int __ha_cas_dw(void *target, void *compare, void *set)
 	uint64_t tmp1, tmp2;
 
 	__asm__ __volatile__("1:"
-                             "ldxp %0, %1, [%4]\n"
-                             "mov %2, %0\n"
-                             "mov %3, %1\n"
-                             "eor %0, %0, %5\n"
-                             "eor %1, %1, %6\n"
-                             "orr %1, %0, %1\n"
-                             "mov %w0, #0\n"
-                             "cbnz %1, 2f\n"
-                             "stxp %w0, %7, %8, [%4]\n"
-                             "cbnz %w0, 1b\n"
-                             "mov %w0, #1\n"
+                             "ldxp %0, %1, [%4];"
+                             "mov %2, %0;"
+                             "mov %3, %1;"
+                             "eor %0, %0, %5;"
+                             "eor %1, %1, %6;"
+                             "orr %1, %0, %1;"
+                             "mov %w0, #0;"
+                             "cbnz %1, 2f;"
+                             "stxp %w0, %7, %8, [%4];"
+                             "cbnz %w0, 1b;"
+                             "mov %w0, #1;"
                              "2:"
                              : "=&r" (tmp1), "=&r" (tmp2), "=&r" (value[0]), "=&r" (value[1])
                              : "r" (target), "r" (((void **)(compare))[0]), "r" (((void **)(compare))[1]), "r" (((void **)(set))[0]), "r" (((void **)(set))[1])
@@ -1296,18 +1234,10 @@ int thread_get_default_count();
 #define _HA_ATOMIC_AND HA_ATOMIC_AND
 #endif /* !_HA_ATOMIC_AND */
 
-#ifndef _HA_ATOMIC_FETCH_AND
-#define _HA_ATOMIC_FETCH_AND HA_ATOMIC_FETCH_AND
-#endif /* !_HA_ATOMIC_FETCH_AND */
-
 #ifndef _HA_ATOMIC_OR
 #define _HA_ATOMIC_OR HA_ATOMIC_OR
 #endif /* !_HA_ATOMIC_OR */
 
-#ifndef _HA_ATOMIC_FETCH_OR
-#define _HA_ATOMIC_FETCH_OR HA_ATOMIC_FETCH_OR
-#endif /* !_HA_ATOMIC_FETCH_OR */
-
 #ifndef _HA_ATOMIC_XCHG
 #define _HA_ATOMIC_XCHG HA_ATOMIC_XCHG
 #endif /* !_HA_ATOMIC_XCHG */
