@@ -129,23 +129,27 @@
  */
 
 /* HTX start-line flags */
-#define HTX_SL_F_NONE          0x00000000
-#define HTX_SL_F_IS_RESP       0x00000001 /* It is the response start-line (unset means the request one) */
-#define HTX_SL_F_XFER_LEN      0x00000002 /* The message xfer size can be dertermined */
-#define HTX_SL_F_XFER_ENC      0x00000004 /* The transfer-encoding header was found in message */
-#define HTX_SL_F_CLEN          0x00000008 /* The content-length header was found in message */
-#define HTX_SL_F_CHNK          0x00000010 /* The message payload is chunked */
-#define HTX_SL_F_VER_11        0x00000020 /* The message indicates version 1.1 or above */
-#define HTX_SL_F_BODYLESS      0x00000040 /* The message has no body (content-length = 0) */
-#define HTX_SL_F_HAS_SCHM      0x00000080 /* The scheme is explicitly specified */
-#define HTX_SL_F_SCHM_HTTP     0x00000100 /* The scheme HTTP should be used */
-#define HTX_SL_F_SCHM_HTTPS    0x00000200 /* The scheme HTTPS should be used */
+#define HTX_SL_F_NONE           0x00000000
+#define HTX_SL_F_IS_RESP        0x00000001 /* It is the response start-line (unset means the request one) */
+#define HTX_SL_F_XFER_LEN       0x00000002 /* The message xfer size can be dertermined */
+#define HTX_SL_F_XFER_ENC       0x00000004 /* The transfer-encoding header was found in message */
+#define HTX_SL_F_CLEN           0x00000008 /* The content-length header was found in message */
+#define HTX_SL_F_CHNK           0x00000010 /* The message payload is chunked */
+#define HTX_SL_F_VER_11         0x00000020 /* The message indicates version 1.1 or above */
+#define HTX_SL_F_BODYLESS       0x00000040 /* The message has no body (content-length = 0) */
+#define HTX_SL_F_HAS_SCHM       0x00000080 /* The scheme is explicitly specified */
+#define HTX_SL_F_SCHM_HTTP      0x00000100 /* The scheme HTTP should be used */
+#define HTX_SL_F_SCHM_HTTPS     0x00000200 /* The scheme HTTPS should be used */
+#define HTX_SL_F_HAS_AUTHORITY  0x00000400 /* The request authority is explicitly specified */
+#define HTX_SL_F_NORMALIZED_URI 0x00000800 /* The received URI is normalized (an implicit absolute-uri form) */
+
 
 /* HTX flags */
 #define HTX_FL_NONE              0x00000000
 #define HTX_FL_PARSING_ERROR     0x00000001 /* Set when a parsing error occurred */
-#define HTX_FL_UPGRADE           0x00000002 /* Set when an upgrade is in progress */
-#define HTX_FL_FRAGMENTED        0x00000020 /* Set when the HTX buffer is fragmented */
+#define HTX_FL_PROCESSING_ERROR  0x00000002 /* Set when a processing error occurred */
+#define HTX_FL_UPGRADE           0x00000004 /* Set when an upgrade is in progress */
+#define HTX_FL_PROXY_RESP        0x00000008 /* Set when the response was generated by HAProxy */
 
 
 /* HTX block's type (max 15). */
@@ -203,30 +207,29 @@ struct htx {
 			  * blocks (blocks and their contents), you need to add size used by blocks,
 			  * i.e. [ used * sizeof(struct htx_blk *) ] */
 
-	uint32_t used;   /* number of blocks in use */
-	uint32_t tail;   /* newest inserted block. -1 if the HTX message is empty */
-	uint32_t head;   /* oldest inserted block. -1 if the HTX message is empty */
+	int32_t tail;   /* newest inserted block. -1 if the HTX message is empty */
+	int32_t head;   /* oldest inserted block. -1 if the HTX message is empty */
+	int32_t first;  /* position of the first block to (re)start the analyse. -1 if unset. */
 
 	uint32_t tail_addr; /* start address of the free space in front of the the blocks table */
 	uint32_t head_addr; /* start address of the free space at the beginning */
 	uint32_t end_addr;  /* end address of the free space at the beginning */
 
-
 	uint64_t extra;  /* known bytes amount remaining to receive */
 	uint32_t flags;  /* HTX_FL_* */
 
-	int32_t  first;  /* position of the first block to (re)start the analyse. -1 if unset. */
+	/* XXX 4 bytes unused */
 
-	struct htx_blk blocks[0]; /* Blocks representing the HTTP message itself */
+	/* Blocks representing the HTTP message itself */
+	char blocks[0] __attribute__((aligned(8)));
 };
 
 
 extern struct htx htx_empty;
 
-struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk, uint32_t info);
+struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk);
 struct htx_blk *htx_add_blk(struct htx *htx, enum htx_blk_type type, uint32_t blksz);
 struct htx_blk *htx_remove_blk(struct htx *htx, struct htx_blk *blk);
-struct htx_ret htx_find_offset(struct htx *htx, uint32_t offset);
 void htx_truncate(struct htx *htx, uint32_t offset);
 struct htx_ret htx_drain(struct htx *htx, uint32_t max);
 
@@ -245,7 +248,6 @@ struct htx_blk *htx_replace_header(struct htx *htx, struct htx_blk *blk,
 
 struct htx_blk *htx_add_header(struct htx *htx, const struct ist name, const struct ist value);
 struct htx_blk *htx_add_trailer(struct htx *htx, const struct ist name, const struct ist value);
-struct htx_blk *htx_add_blk_type_size(struct htx *htx, enum htx_blk_type type, uint32_t blksz);
 struct htx_blk *htx_add_all_headers(struct htx *htx, const struct http_hdr *hdrs);
 struct htx_blk *htx_add_all_trailers(struct htx *htx, const struct http_hdr *hdrs);
 struct htx_blk *htx_add_endof(struct htx *htx, enum htx_blk_type type);
@@ -253,11 +255,7 @@ struct htx_blk *htx_add_data_atonce(struct htx *htx, struct ist data);
 size_t htx_add_data(struct htx *htx, const struct ist data);
 struct htx_blk *htx_add_last_data(struct htx *htx, struct ist data);
 void htx_move_blk_before(struct htx *htx, struct htx_blk **blk, struct htx_blk **ref);
-
-int htx_reqline_to_h1(const struct htx_sl *sl, struct buffer *chk);
-int htx_stline_to_h1(const struct htx_sl *sl, struct buffer *chk);
-int htx_hdr_to_h1(const struct ist n, const struct ist v, struct buffer *chk);
-int htx_data_to_h1(const struct ist data, struct buffer *chk, int chunked);
+int htx_append_msg(struct htx *dst, const struct htx *src);
 
 /* Functions and macros to get parts of the start-line or legnth of these
  * parts. Request and response start-lines are both composed of 3 parts.
@@ -332,23 +330,23 @@ static inline struct ist htx_sl_res_reason(const struct htx_sl *sl)
 }
 
 /* Converts a position to the corresponding relative address */
-static inline uint32_t htx_pos_to_idx(const struct htx *htx, uint32_t pos)
+static inline uint32_t htx_pos_to_addr(const struct htx *htx, uint32_t pos)
 {
-	return ((htx->size / sizeof(htx->blocks[0])) - pos - 1);
+	return htx->size - (pos + 1) * sizeof(struct htx_blk);
 }
 
 /* Returns the position of the block <blk>. It is the caller responsibility to
  * be sure <blk> is part of <htx>. */
 static inline uint32_t htx_get_blk_pos(const struct htx *htx, const struct htx_blk *blk)
 {
-	return (htx->blocks + (htx->size / sizeof(htx->blocks[0])) - blk - 1);
+	return ((htx->blocks + htx->size - (char *)blk) / sizeof(struct htx_blk) - 1);
 }
 
 /* Returns the block at the position <pos>. It is the caller responsibility to
  * be sure the block at the position <pos> exists. */
 static inline struct htx_blk *htx_get_blk(const struct htx *htx, uint32_t pos)
 {
-	return ((struct htx_blk *)(htx->blocks) + htx_pos_to_idx(htx, pos));
+	return (struct htx_blk *)(htx->blocks + htx_pos_to_addr(htx, pos));
 }
 
 /* Returns the type of the block <blk> */
@@ -373,15 +371,12 @@ static inline uint32_t htx_get_blksz(const struct htx_blk *blk)
 	}
 }
 
-/* Returns the position of the oldest entry (head).
- *
- * An signed 32-bits integer is returned to handle -1 case. Blocks position are
- * store on unsigned 32-bits integer, but it is impossible to have so much
- * blocks to overflow a 32-bits signed integer !
+/* Returns the position of the oldest entry (head). It returns a signed 32-bits
+ * integer, -1 means the HTX message is empty.
  */
 static inline int32_t htx_get_head(const struct htx *htx)
 {
-	return (htx->used ? htx->head : -1);
+	return htx->head;
 }
 
 /* Returns the oldest HTX block (head) if the HTX message is not
@@ -404,15 +399,12 @@ static inline enum htx_blk_type htx_get_head_type(const struct htx *htx)
 	return (blk ? htx_get_blk_type(blk) : HTX_BLK_UNUSED);
 }
 
-/* Returns the position of the newest entry (tail).
- *
- * An signed 32-bits integer is returned to handle -1 case. Blocks position are
- * store on unsigned 32-bits integer, but it is impossible to have so much
- * blocks to overflow a 32-bits signed integer !
+/* Returns the position of the newest entry (tail).  It returns a signed 32-bits
+ * integer, -1 means the HTX message is empty.
  */
 static inline int32_t htx_get_tail(const struct htx *htx)
 {
-	return (htx->used ? htx->tail : -1);
+	return htx->tail;
 }
 
 /* Returns the newest HTX block (tail) if the HTX message is not
@@ -435,17 +427,11 @@ static inline enum htx_blk_type htx_get_tail_type(const struct htx *htx)
 	return (blk ? htx_get_blk_type(blk) : HTX_BLK_UNUSED);
 }
 
-/* Returns the position of the first block in the HTX message <htx>. If unset,
- * or if <htx> is empty, -1 is returned.
- *
- * An signed 32-bits integer is returned to handle -1 case. Blocks position are
- * store on unsigned 32-bits integer, but it is impossible to have so much
- * blocks to overflow a 32-bits signed integer !
+/* Returns the position of the first block in the HTX message <htx>. -1 means
+ * the first block is unset or the HTS is empty.
  */
 static inline int32_t htx_get_first(const struct htx *htx)
 {
-	if (!htx->used)
-		return -1;
 	return htx->first;
 }
 
@@ -472,17 +458,10 @@ static inline enum htx_blk_type htx_get_first_type(const struct htx *htx)
 
 /* Returns the position of block immediately before the one pointed by <pos>. If
  * the message is empty or if <pos> is the position of the head, -1 returned.
- *
- * An signed 32-bits integer is returned to handle -1 case. Blocks position are
- * store on unsigned 32-bits integer, but it is impossible to have so much
- * blocks to overflow a 32-bits signed integer !
  */
 static inline int32_t htx_get_prev(const struct htx *htx, uint32_t pos)
 {
-	int32_t head;
-
-	head = htx_get_head(htx);
-	if (head == -1 || pos == head)
+	if (htx->head == -1 || pos == htx->head)
 		return -1;
 	return (pos - 1);
 }
@@ -501,14 +480,10 @@ static inline struct htx_blk *htx_get_prev_blk(const struct htx *htx,
 
 /* Returns the position of block immediately after the one pointed by <pos>. If
  * the message is empty or if <pos> is the position of the tail, -1 returned.
- *
- * An signed 32-bits integer is returned to handle -1 case. Blocks position are
- * store on unsigned 32-bits integer, but it is impossible to have so much
- * blocks to overflow a 32-bits signed integer !
  */
 static inline int32_t htx_get_next(const struct htx *htx, uint32_t pos)
 {
-	if (!htx->used || pos == htx->tail)
+	if (htx->tail == -1 || pos == htx->tail)
 		return -1;
 	return (pos + 1);
 
@@ -658,7 +633,10 @@ static inline void htx_cut_data_blk(struct htx *htx, struct htx_blk *blk, uint32
 /* Returns the space used by metadata in <htx>. */
 static inline uint32_t htx_meta_space(const struct htx *htx)
 {
-	return (htx->used * sizeof(htx->blocks[0]));
+	if (htx->tail == -1)
+		return 0;
+
+	return ((htx->tail + 1 - htx->head) * sizeof(struct htx_blk));
 }
 
 /* Returns the space used (payload + metadata) in <htx> */
@@ -680,20 +658,9 @@ static inline uint32_t htx_free_data_space(const struct htx *htx)
 {
 	uint32_t free = htx_free_space(htx);
 
-	if (free < sizeof(htx->blocks[0]))
+	if (free < sizeof(struct htx_blk))
 		return 0;
-	return (free - sizeof(htx->blocks[0]));
-}
-
-/* Returns non-zero only if the HTX message free space wraps */
-static inline int htx_space_wraps(const struct htx *htx)
-{
-	uint32_t headroom, tailroom;
-
-	headroom = (htx->end_addr - htx->head_addr);
-	tailroom = sizeof(htx->blocks[0]) * htx_pos_to_idx(htx, htx->tail) - htx->tail_addr;
-
-	return (headroom && tailroom);
+	return (free - sizeof(struct htx_blk));
 }
 
 /* Returns the maximum size for a block, not exceeding <max> bytes. <max> may be
@@ -705,9 +672,9 @@ static inline uint32_t htx_get_max_blksz(const struct htx *htx, int32_t max)
 
 	if (max != -1 && free > max)
 		free = max;
-	if (free < sizeof(htx->blocks[0]))
+	if (free < sizeof(struct htx_blk))
 		return 0;
-	return (free - sizeof(htx->blocks[0]));
+	return (free - sizeof(struct htx_blk));
 }
 
 /* Returns 1 if the message has less than 1/4 of its capacity free, otherwise 0 */
@@ -721,11 +688,11 @@ static inline int htx_almost_full(const struct htx *htx)
 /* Resets an HTX message */
 static inline void htx_reset(struct htx *htx)
 {
-	htx->data = htx->used = htx->tail = htx->head  = 0;
+	htx->tail = htx->head  = htx->first = -1;
+	htx->data = 0;
 	htx->tail_addr = htx->head_addr = htx->end_addr = 0;
 	htx->extra = 0;
 	htx->flags = HTX_FL_NONE;
-	htx->first = -1;
 }
 
 /* Returns the available room for raw data in buffer <buf> once HTX overhead is
@@ -787,7 +754,8 @@ static inline struct htx *htx_from_buf(struct buffer *buf)
 /* Update <buf> accordingly to the HTX message <htx> */
 static inline void htx_to_buf(struct htx *htx, struct buffer *buf)
 {
-	if (!htx->used && !(htx->flags & (HTX_FL_PARSING_ERROR|HTX_FL_UPGRADE))) {
+	if ((htx->head == -1) &&
+	    !(htx->flags & (HTX_FL_PARSING_ERROR|HTX_FL_PROCESSING_ERROR|HTX_FL_UPGRADE))) {
 		htx_reset(htx);
 		b_set_data(buf, 0);
 	}
@@ -800,7 +768,7 @@ static inline void htx_to_buf(struct htx *htx, struct buffer *buf)
  */
 static inline int htx_is_empty(const struct htx *htx)
 {
-	return !htx->used;
+	return (htx->head == -1);
 }
 
 /* Returns 1 if the message is not empty, otherwise it returns 0. Note that it
@@ -808,22 +776,17 @@ static inline int htx_is_empty(const struct htx *htx)
  */
 static inline int htx_is_not_empty(const struct htx *htx)
 {
-	return htx->used;
+	return (htx->head != -1);
 }
 
-static inline void htx_skip_msg_payload(struct htx *htx)
+/* Returns the number of used blocks in the HTX message <htx>. Note that it is
+ * illegal to call this function with htx == NULL. Note also blocks of type
+ * HTX_BLK_UNUSED are part of used blocks.
+ */
+static inline int htx_nbblks(const struct htx *htx)
 {
-	struct htx_blk *blk = htx_get_first_blk(htx);
-
-	while (blk) {
-		enum htx_blk_type type = htx_get_blk_type(blk);
-
-		blk = ((type > HTX_BLK_EOH && type < HTX_BLK_EOM)
-		       ? htx_remove_blk(htx, blk)
-		       : htx_get_next_blk(htx, blk));
-	}
+	return ((htx->head != -1) ? (htx->tail + 1 - htx->head) : 0);
 }
-
 /* For debugging purpose */
 static inline const char *htx_blk_type_str(enum htx_blk_type type)
 {
@@ -842,17 +805,19 @@ static inline const char *htx_blk_type_str(enum htx_blk_type type)
 }
 
 /* For debugging purpose */
-static inline void htx_dump(struct htx *htx)
+static inline void htx_dump(struct buffer *chunk, const struct htx *htx, int full)
 {
 	int32_t pos;
 
-	fprintf(stderr, "htx:%p [ size=%u - data=%u - used=%u - wrap=%s - extra=%llu]\n",
-		htx, htx->size, htx->data, htx->used, (!htx->head_addr) ? "NO" : "YES",
-		(unsigned long long)htx->extra);
-	fprintf(stderr, "\tfirst=%d - head=%u, tail=%u\n",
-		htx->first, htx->head, htx->tail);
-	fprintf(stderr, "\ttail_addr=%d - head_addr=%u, end_addr=%u\n",
-		htx->tail_addr, htx->head_addr, htx->end_addr);
+	chunk_appendf(chunk, " htx=%p(size=%u,data=%u,used=%u,wrap=%s,flags=0x%08x,extra=%llu,"
+		      "first=%d,head=%d,tail=%d,tail_addr=%d,head_addr=%d,end_addr=%d)",
+		      htx, htx->size, htx->data, htx_nbblks(htx), (!htx->head_addr) ? "NO" : "YES",
+		      htx->flags, (unsigned long long)htx->extra, htx->first, htx->head, htx->tail,
+		      htx->tail_addr, htx->head_addr, htx->end_addr);
+
+	if (!full || !htx_nbblks(htx))
+		return;
+	chunk_memcat(chunk, "\n", 1);
 
 	for (pos = htx_get_head(htx); pos != -1; pos = htx_get_next(htx, pos)) {
 		struct htx_sl     *sl;
@@ -866,23 +831,22 @@ static inline void htx_dump(struct htx *htx)
 
 		if (type == HTX_BLK_REQ_SL || type == HTX_BLK_RES_SL) {
 			sl = htx_get_blk_ptr(htx, blk);
-			fprintf(stderr, "\t\t[%u] type=%-17s - size=%-6u - addr=%-6u\t%.*s %.*s %.*s\n",
-				pos, htx_blk_type_str(type), sz, blk->addr,
-				HTX_SL_P1_LEN(sl), HTX_SL_P1_PTR(sl),
-				HTX_SL_P2_LEN(sl), HTX_SL_P2_PTR(sl),
-				HTX_SL_P3_LEN(sl), HTX_SL_P3_PTR(sl));
+			chunk_appendf(chunk, "\t\t[%u] type=%-17s - size=%-6u - addr=%-6u\t%.*s %.*s %.*s\n",
+				      pos, htx_blk_type_str(type), sz, blk->addr,
+				      HTX_SL_P1_LEN(sl), HTX_SL_P1_PTR(sl),
+				      HTX_SL_P2_LEN(sl), HTX_SL_P2_PTR(sl),
+				      HTX_SL_P3_LEN(sl), HTX_SL_P3_PTR(sl));
 		}
 		else if (type == HTX_BLK_HDR || type == HTX_BLK_TLR)
-			fprintf(stderr, "\t\t[%u] type=%-17s - size=%-6u - addr=%-6u\t%.*s: %.*s\n",
-				pos, htx_blk_type_str(type), sz, blk->addr,
-				(int)n.len, n.ptr,
-				(int)v.len, v.ptr);
+			chunk_appendf(chunk, "\t\t[%u] type=%-17s - size=%-6u - addr=%-6u\t%.*s: %.*s\n",
+				      pos, htx_blk_type_str(type), sz, blk->addr,
+				      (int)n.len, n.ptr,
+				      (int)v.len, v.ptr);
 		else
-			fprintf(stderr, "\t\t[%u] type=%-17s - size=%-6u - addr=%-6u%s\n",
-				pos, htx_blk_type_str(type), sz, blk->addr,
-				(!v.len ? "\t<empty>" : ""));
+			chunk_appendf(chunk, "\t\t[%u] type=%-17s - size=%-6u - addr=%-6u%s\n",
+				      pos, htx_blk_type_str(type), sz, blk->addr,
+				      (!v.len ? "\t<empty>" : ""));
 	}
-	fprintf(stderr, "\n");
 }
 
 #endif /* _COMMON_HTX_H */
