@@ -61,25 +61,11 @@
  * power of 2, and 0 otherwise */
 #define POWEROF2(x) (((x) & ((x)-1)) == 0)
 
-/* rotate left a 64-bit integer by <bits:[0-5]> bits */
-static inline uint64_t rotl64(uint64_t v, uint8_t bits)
-{
-#if !defined(__ARM_ARCH_8A) && !defined(__x86_64__)
-	bits &= 63;
-#endif
-	v = (v << bits) | (v >> (-bits & 63));
-	return v;
-}
-
-/* rotate right a 64-bit integer by <bits:[0-5]> bits */
-static inline uint64_t rotr64(uint64_t v, uint8_t bits)
-{
-#if !defined(__ARM_ARCH_8A) && !defined(__x86_64__)
-	bits &= 63;
-#endif
-	v = (v >> bits) | (v << (-bits & 63));
-	return v;
-}
+/* DEFNULL() returns either the argument as-is, or NULL if absent. This is for
+ * use in macros arguments.
+ */
+#define DEFNULL(...) _FIRST_ARG(NULL, ##__VA_ARGS__, NULL)
+#define _FIRST_ARG(a, b, ...) b
 
 /* operators to compare values. They're ordered that way so that the lowest bit
  * serves as a negation for the test and contains all tests that are not equal.
@@ -101,6 +87,12 @@ struct split_url {
 	int host_len;
 };
 
+/* generic structure associating a name and a value, for use in arrays */
+struct name_desc {
+	const char *name;
+	const char *desc;
+};
+
 extern THREAD_LOCAL int itoa_idx; /* index of next itoa_str to use */
 
 /*
@@ -215,6 +207,27 @@ static inline const char *LIM2A(unsigned long n, const char *alt)
 	return ret;
 }
 
+/* returns the number of bytes needed to encode <v> as a varint. Be careful, use
+ * it only with constants as it generates a large code (typ. 180 bytes). Use the
+ * varint_bytes() version instead in case of doubt.
+ */
+int varint_bytes(uint64_t v);
+static inline int __varint_bytes(uint64_t v)
+{
+	switch (v) {
+	case 0x0000000000000000 ... 0x00000000000000ef: return 1;
+	case 0x00000000000000f0 ... 0x00000000000008ef: return 2;
+	case 0x00000000000008f0 ... 0x00000000000408ef: return 3;
+	case 0x00000000000408f0 ... 0x00000000020408ef: return 4;
+	case 0x00000000020408f0 ... 0x00000001020408ef: return 5;
+	case 0x00000001020408f0 ... 0x00000081020408ef: return 6;
+	case 0x00000081020408f0 ... 0x00004081020408ef: return 7;
+	case 0x00004081020408f0 ... 0x00204081020408ef: return 8;
+	case 0x00204081020408f0 ... 0x10204081020408ef: return 9;
+	default: return 10;
+	}
+}
+
 /* Encode the integer <i> into a varint (variable-length integer). The encoded
  * value is copied in <*buf>. Here is the encoding format:
  *
@@ -498,8 +511,7 @@ char *encode_chunk(char *start, char *stop,
 
 /*
  * Tries to prefix characters tagged in the <map> with the <escape>
- * character. The input <string> is processed until string_stop
- * is reached or NULL-byte is encountered. The result will
+ * character. The input <string> must be zero-terminated. The result will
  * be stored between <start> (included) and <stop> (excluded). This
  * function will always try to terminate the resulting string with a '\0'
  * before <stop>, and will return its position if the conversion
@@ -507,7 +519,7 @@ char *encode_chunk(char *start, char *stop,
  */
 char *escape_string(char *start, char *stop,
 		    const char escape, const long *map,
-		    const char *string, const char *string_stop);
+		    const char *string);
 
 /*
  * Tries to prefix characters tagged in the <map> with the <escape>
@@ -562,12 +574,8 @@ static inline const char *csv_enc(const char *str, int quote,
  * be shorter. If some forbidden characters are found, the conversion is
  * aborted, the string is truncated before the issue and non-zero is returned,
  * otherwise the operation returns non-zero indicating success.
- * If the 'in_form' argument is non-nul the string is assumed to be part of
- * an "application/x-www-form-urlencoded" encoded string, and the '+' will be
- * turned to a space. If it's zero, this will only be done after a question
- * mark ('?').
  */
-int url_decode(char *string, int in_form);
+int url_decode(char *string);
 
 /* This one is 6 times faster than strtoul() on athlon, but does
  * no check at all.
@@ -795,10 +803,10 @@ static inline unsigned int mul32hi(unsigned int a, unsigned int b)
  */
 static inline unsigned int div64_32(unsigned long long o1, unsigned int o2)
 {
-	unsigned long long result;
+	unsigned int result;
 #ifdef __i386__
 	asm("divl %2"
-	    : "=A" (result)
+	    : "=a" (result)
 	    : "A"(o1), "rm"(o2));
 #else
 	result = o1 / o2;
@@ -1294,9 +1302,9 @@ char *env_expand(char *in);
  */
 void debug_hexdump(FILE *out, const char *pfx, const char *buf, unsigned int baseaddr, int len);
 
-/* this is used to emit traces when building with TRACE=1 */
+/* this is used to emit call traces when building with TRACE=1 */
 __attribute__((format(printf, 1, 2)))
-void trace(char *fmt, ...);
+void calltrace(char *fmt, ...);
 
 /* used from everywhere just to drain results we don't want to read and which
  * recent versions of gcc increasingly and annoyingly complain about.
@@ -1480,21 +1488,6 @@ static inline void *my_realloc2(void *ptr, size_t size)
 
 int parse_dotted_uints(const char *s, unsigned int **nums, size_t *sz);
 
-/* PRNG */
-void ha_random_seed(const unsigned char *seed, size_t len);
-void ha_random_jump96(uint32_t dist);
-uint64_t ha_random64();
-
-static inline uint32_t ha_random32()
-{
-	return ha_random64() >> 32;
-}
-
-static inline int32_t ha_random()
-{
-	return ha_random32() >> 1;
-}
-
 /* HAP_STRING() makes a string from a literal while HAP_XSTRING() first
  * evaluates the argument and is suited to pass macros.
  *
