@@ -60,12 +60,9 @@ extern THREAD_LOCAL unsigned int   samp_time;        /* total elapsed time over
 extern THREAD_LOCAL unsigned int   idle_time;        /* total idle time over current sample */
 extern THREAD_LOCAL struct timeval now;              /* internal date is a monotonic function of real clock */
 extern THREAD_LOCAL struct timeval date;             /* the real current date */
-extern struct timeval start_date;                    /* the process's start date */
-extern struct timeval ready_date;                    /* date when the process was considered ready */
+extern struct timeval start_date;       /* the process's start date */
 extern THREAD_LOCAL struct timeval before_poll;      /* system date before calling poll() */
 extern THREAD_LOCAL struct timeval after_poll;       /* system date after leaving poll() */
-extern volatile unsigned long long global_now;
-extern volatile unsigned int global_now_ms;
 
 
 /**** exported functions *************************************************/
@@ -106,6 +103,8 @@ REGPRM1 static inline struct timeval *tv_now(struct timeval *tv)
  */
 REGPRM2 void tv_update_date(int max_wait, int interrupted);
 
+char *timeofday_as_iso_us(int pad);
+
 /*
  * sets a struct timeval to its highest value so that it can never happen
  * note that only tv_usec is necessary to detect it since a tv_usec > 999999
