@@ -54,7 +54,6 @@ int ci_getblk_nc(const struct channel *chn, char **blk1, size_t *len1, char **bl
 int ci_insert_line2(struct channel *c, int pos, const char *str, int len);
 int co_inject(struct channel *chn, const char *msg, int len);
 int co_getline(const struct channel *chn, char *str, int len);
-int co_getdelim(const struct channel *chn, char *str, int len, const char *delim, char escape);
 int co_getblk(const struct channel *chn, char *blk, int len, int offset);
 int co_getline_nc(const struct channel *chn, const char **blk1, size_t *len1, const char **blk2, size_t *len2);
 int co_getblk_nc(const struct channel *chn, const char **blk1, size_t *len1, const char **blk2, size_t *len2);
@@ -437,39 +436,6 @@ static inline int channel_may_send(const struct channel *chn)
 	return chn_cons(chn)->state == SI_ST_EST;
 }
 
-/* HTX version of channel_may_recv(). Returns non-zero if the channel can still
- * receive data. */
-static inline int channel_htx_may_recv(const struct channel *chn, const struct htx *htx)
-{
-	uint32_t rem;
-
-	if (!htx->size)
-		return 1;
-
-	rem = htx_free_data_space(htx);
-	if (!rem)
-		return 0; /* htx already full */
-
-	if (rem > global.tune.maxrewrite)
-		return 1; /* reserve not yet reached */
-
-	if (!channel_may_send(chn))
-		return 0; /* don't touch reserve until we can send */
-
-	/* Now we know there's some room left in the reserve and we may
-	 * forward. As long as i-to_fwd < size-maxrw, we may still
-	 * receive. This is equivalent to i+maxrw-size < to_fwd,
-	 * which is logical since i+maxrw-size is what overlaps with
-	 * the reserve, and we want to ensure they're covered by scheduled
-	 * forwards.
-	 */
-	rem += co_data(chn);
-	if (rem > global.tune.maxrewrite)
-		return 1;
-
-	return (global.tune.maxrewrite - rem < chn->to_forward);
-}
-
 /* Returns non-zero if the channel can still receive data. This is used to
  * decide when to stop reading into a buffer when we want to ensure that we
  * leave the reserve untouched after all pending outgoing data are forwarded.
@@ -485,9 +451,6 @@ static inline int channel_may_recv(const struct channel *chn)
 {
 	int rem = chn->buf.size;
 
-	if (IS_HTX_STRM(chn_strm(chn)))
-		return channel_htx_may_recv(chn, htxbuf(&chn->buf));
-
 	if (b_is_null(&chn->buf))
 		return 1;
 
@@ -512,6 +475,39 @@ static inline int channel_may_recv(const struct channel *chn)
 	return rem < 0 || (unsigned int)rem < chn->to_forward;
 }
 
+/* HTX version of channel_may_recv(). Returns non-zero if the channel can still
+ * receive data. */
+static inline int channel_htx_may_recv(const struct channel *chn, const struct htx *htx)
+{
+	uint32_t rem;
+
+	if (!htx->size)
+		return 1;
+
+	if (!channel_may_send(chn))
+		return 0; /* don't touch reserve until we can send */
+
+	rem = htx_free_data_space(htx);
+	if (!rem)
+		return 0; /* htx already full */
+
+	if (rem > global.tune.maxrewrite)
+		return 1; /* reserve not yet reached */
+
+	/* Now we know there's some room left in the reserve and we may
+	 * forward. As long as i-to_fwd < size-maxrw, we may still
+	 * receive. This is equivalent to i+maxrw-size < to_fwd,
+	 * which is logical since i+maxrw-size is what overlaps with
+	 * the reserve, and we want to ensure they're covered by scheduled
+	 * forwards.
+	 */
+	rem += co_data(chn);
+	if (rem > global.tune.maxrewrite)
+		return 1;
+
+	return (global.tune.maxrewrite - rem < chn->to_forward);
+}
+
 /* Returns true if the channel's input is already closed */
 static inline int channel_input_closed(struct channel *chn)
 {
@@ -939,6 +935,22 @@ static inline int32_t channel_htx_fwd_headers(struct channel *chn, struct htx *h
 	return pos;
 }
 
+/* Copy an HTX message stored in the buffer <msg> to the channel's one. We
+ * take care to not overwrite existing data in the channel. All the message is
+ * copied or nothing. It returns 1 on success and 0 on error.
+ */
+static inline int channel_htx_copy_msg(struct channel *chn, struct htx *htx, const struct buffer *msg)
+{
+	/* The channel buffer is empty, we can do a raw copy */
+	if (c_empty(chn)) {
+		chn->buf.data = msg->data;
+		memcpy(chn->buf.area, msg->area, msg->data);
+		return 1;
+	}
+
+	/* Otherwise, we need to append the HTX message */
+	return htx_append_msg(htx, htxbuf(msg));
+}
 /*
  * Advance the channel buffer's read pointer by <len> bytes. This is useful
  * when data have been read directly from the buffer. It is illegal to call
