@@ -87,6 +87,10 @@ static inline int build_logline(struct stream *s, char *dst, size_t maxsize, str
 void strm_log(struct stream *s);
 void sess_log(struct session *sess);
 
+/* send a applicative log with custom list of log servers */
+void app_log(struct list *logsrvs, struct buffer *tag, int level, const char *format, ...)
+	__attribute__ ((format(printf, 4, 5)));
+
 /*
  * add to the logformat linked list
  */
@@ -141,7 +145,7 @@ void send_log(struct proxy *p, int level, const char *format, ...)
  * It doesn't care about errors nor does it report them.
  */
 
-void __send_log(struct proxy *p, int level, char *message, size_t size, char *sd, size_t sd_size);
+void __send_log(struct list *logsrvs, struct buffer *tag, int level, char *message, size_t size, char *sd, size_t sd_size);
 
 /*
  * returns log format for <fmt> or -1 if not found.
