@@ -144,8 +144,7 @@ enum {
 	/* unused : 0x00000010 */
 	CO_FL_XPRT_WR_ENA   = 0x00000020,  /* sending data is desired */
 	CO_FL_CURR_WR_ENA   = 0x00000040,  /* sending is currently desired */
-
-	CO_FL_WANT_DRAIN    = 0x00000080, /* try to drain pending data when closing */
+	/* unused : 0x00000080 */
 
 	/* These flags indicate whether the Control and Transport layers are initialized */
 	CO_FL_CTRL_READY    = 0x00000100, /* FD was registered, fd_delete() needed */
@@ -467,11 +466,11 @@ struct connection {
 
 	/* third cache line and beyond */
 	void (*destroy_cb)(struct connection *conn);  /* callback to notify of imminent death of the connection */
-	struct {
-		struct sockaddr_storage from;	/* client address, or address to spoof when connecting to the server */
-		struct sockaddr_storage to;	/* address reached by the client, or address to connect to */
-	} addr; /* addresses of the remote side, client for producer and server for consumer */
+	struct sockaddr_storage *src; /* source address (pool), when known, otherwise NULL */
+	struct sockaddr_storage *dst; /* destination address (pool), when known, otherwise NULL */
+	char *proxy_authority;	      /* Value of authority TLV received via PROXYv2 */
 	unsigned int idle_time;                 /* Time the connection was added to the idle list, or 0 if not in the idle list */
+	uint8_t proxy_authority_len;  /* Length of authority TLV received via PROXYv2 */
 };
 
 /* PROTO token registration */
@@ -479,8 +478,7 @@ enum proto_proxy_mode {
 	PROTO_MODE_NONE = 0,
 	PROTO_MODE_TCP  = 1 << 0, // must not be changed!
 	PROTO_MODE_HTTP = 1 << 1, // must not be changed!
-	PROTO_MODE_HTX  = 1 << 2, // must not be changed!
-	PROTO_MODE_ANY  = PROTO_MODE_TCP | PROTO_MODE_HTTP, // note: HTX is experimental and must not appear here
+	PROTO_MODE_ANY  = PROTO_MODE_TCP | PROTO_MODE_HTTP,
 };
 
 enum proto_proxy_side {
@@ -562,7 +560,6 @@ struct proxy_hdr_v2 {
 #define PP2_TYPE_AUTHORITY      0x02
 #define PP2_TYPE_CRC32C         0x03
 #define PP2_TYPE_NOOP           0x04
-#define PP2_TYPE_UNIQUE_ID      0x05
 #define PP2_TYPE_SSL            0x20
 #define PP2_SUBTYPE_SSL_VERSION 0x21
 #define PP2_SUBTYPE_SSL_CN      0x22
@@ -590,12 +587,15 @@ struct tlv_ssl {
 #define PP2_CLIENT_CERT_CONN     0x02
 #define PP2_CLIENT_CERT_SESS     0x04
 
+/* Max length of the authority TLV */
+#define PP2_AUTHORITY_MAX 255
 
-/* Max number of file descriptors we send in one sendmsg(). Linux seems to be
- * able to send 253 fds per sendmsg(), however musl is limited to 252, not sure
+/*
+ * Linux seems to be able to send 253 fds per sendmsg(), not sure
  * about the other OSes.
  */
-#define MAX_SEND_FD 252
+/* Max number of file descriptors we send in one sendmsg() */
+#define MAX_SEND_FD 253
 
 #define SOCKS4_HS_RSP_LEN 8
 
