@@ -40,7 +40,6 @@
 struct userlist *userlist = NULL;    /* list of all existing userlists */
 
 #ifdef USE_LIBCRYPT
-#define CRYPT_STATE_MSG "yes"
 #ifdef HA_HAVE_CRYPT_R
 /* context for crypt_r() */
 static THREAD_LOCAL struct crypt_data crypt_data = { .initialized = 0 };
@@ -48,11 +47,9 @@ static THREAD_LOCAL struct crypt_data crypt_data = { .initialized = 0 };
 /* lock for crypt() */
 __decl_hathreads(static HA_SPINLOCK_T auth_lock);
 #endif
-#else /* USE_LIBCRYPT */
-#define CRYPT_STATE_MSG "no"
 #endif
 
-/* find targets for selected gropus. The function returns pointer to
+/* find targets for selected groups. The function returns pointer to
  * the userlist struct ot NULL if name is NULL/empty or unresolvable.
  */
 
@@ -317,4 +314,4 @@ pat_match_auth(struct sample *smp, struct pattern_expr *expr, int fill)
 	return NULL;
 }
 
-REGISTER_BUILD_OPTS("Encrypted password support via crypt(3): "CRYPT_STATE_MSG);
+REGISTER_BUILD_OPTS("Encrypted password support via crypt(3): yes");
