@@ -22,11 +22,10 @@
 #include <proto/channel.h>
 #include <proto/cli.h>
 #include <proto/proxy.h>
-#include <proto/hdr_idx.h>
 #include <proto/http_htx.h>
 #include <proto/filters.h>
 #include <proto/http_rules.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/log.h>
 #include <proto/stream.h>
 #include <proto/stream_interface.h>
@@ -38,11 +37,6 @@
 #include <common/htx.h>
 #include <common/initcall.h>
 
-/* flt_cache_store */
-#define CACHE_F_LEGACY_HTTP 0x00000001 /* The cache is used to store raw HTTP
-					* messages (legacy implementation) */
-#define CACHE_F_HTX         0x00000002 /* The cache is used to store HTX messages */
-
 #define CACHE_FLT_F_IMPLICIT_DECL  0x00000001 /* The cache filtre was implicitly declared (ie without
 					       * the filter keyword) */
 
@@ -59,7 +53,6 @@ struct cache {
 	unsigned int maxblocks;
 	unsigned int maxobjsz;   /* max-object-size (in bytes) */
 	char id[33];             /* cache name */
-	unsigned int flags;      /* CACHE_F_* */
 };
 
 /* cache config for filters */
@@ -75,15 +68,13 @@ struct cache_flt_conf {
  * cache ctx for filters
  */
 struct cache_st {
-	int hdrs_len; // field used in legacy mode only
 	struct shared_block *first_block;
 };
 
 struct cache_entry {
 	unsigned int latest_validation;     /* latest validation date */
-	unsigned int expire;      /* expiration date (wall clock time) */
+	unsigned int expire;      /* expiration date */
 	unsigned int age;         /* Origin server "Age" header value */
-	unsigned int eoh;         /* Origin server end of headers offset. */ // field used in legacy mode only
 
 	struct eb32_node eb;     /* ebtree node used to hold the cache object */
 	char hash[20];
@@ -114,7 +105,7 @@ struct cache_entry *entry_exist(struct cache *cache, char *hash)
 	if (memcmp(entry->hash, hash, sizeof(entry->hash)))
 		return NULL;
 
-	if (entry->expire > date.tv_sec) {
+	if (entry->expire > now.tv_sec) {
 		return entry;
 	} else {
 		eb32_delete(node);
@@ -160,18 +151,13 @@ cache_store_check(struct proxy *px, struct flt_conf *fconf)
 	int comp = 0;
 
 	/* Find the cache corresponding to the name in the filter config.  The
-	 *  cache will not be referenced now in the filter config because it is
-	 *  not fully allocated. This step will be performed during the cache
-	 *  post_check.
-	 */
+	*  cache will not be referenced now in the filter config because it is
+	*  not fully allocated. This step will be performed during the cache
+	*  post_check.
+	*/
 	list_for_each_entry(cache, &caches_config, list) {
-		if (!strcmp(cache->id, cconf->c.name)) {
-			/* there can be only one filter per cache, so we free it there */
-			cache->flags |= ((px->options2 & PR_O2_USE_HTX)
-					 ? CACHE_F_HTX
-					 : CACHE_F_LEGACY_HTTP);
+		if (!strcmp(cache->id, cconf->c.name))
 			goto found;
-		}
 	}
 
 	ha_alert("config: %s '%s': unable to find the cache '%s' referenced by the filter 'cache'.\n",
@@ -195,18 +181,13 @@ cache_store_check(struct proxy *px, struct flt_conf *fconf)
 				return 1;
 			}
 		}
-		else if (f->id == http_comp_flt_id) {
-			if (!(px->options2 & PR_O2_USE_HTX)) {
-				ha_alert("config: %s '%s' : compression and cache filters cannot be "
-					 "both enabled on non HTX proxy.\n",
-					 proxy_type_str(px), px->id);
-				return 1;
-			}
+		else if (f->id == http_comp_flt_id)
 			comp = 1;
-		}
+		else if (f->id == fcgi_flt_id)
+			continue;
 		else if ((f->id != fconf->id) && (cconf->flags & CACHE_FLT_F_IMPLICIT_DECL)) {
 			/* Implicit declaration is only allowed with the
-			 * compression. For other filters, an implicit
+			 * compression and fcgi. For other filters, an implicit
 			 * declaration is required. */
 			ha_alert("config: %s '%s': require an explicit filter declaration "
 				 "to use the cache '%s'.\n", proxy_type_str(px), px->id, cache->id);
@@ -230,7 +211,6 @@ cache_store_chn_start_analyze(struct stream *s, struct filter *filter, struct ch
 		if (st == NULL)
 			return -1;
 
-		st->hdrs_len    = 0;
 		st->first_block = NULL;
 		filter->ctx     = st;
 
@@ -304,11 +284,8 @@ cache_store_http_headers(struct stream *s, struct filter *filter, struct http_ms
 	if (!(msg->chn->flags & CF_ISRESP) || !st)
 		return 1;
 
-	if (st->first_block) {
+	if (st->first_block)
 		register_data_filter(s, msg->chn, filter);
-		if (!IS_HTX_STRM(s))
-			st->hdrs_len = msg->sov;
-	}
 	return 1;
 }
 
@@ -336,7 +313,6 @@ cache_store_http_payload(struct stream *s, struct filter *filter, struct http_ms
 	struct htx *htx = htxbuf(&msg->chn->buf);
 	struct htx_blk *blk;
 	struct shared_block *fb;
-	struct htx_ret htxret;
 	unsigned int orig_len, to_forward;
 	int ret;
 
@@ -351,15 +327,16 @@ cache_store_http_payload(struct stream *s, struct filter *filter, struct http_ms
 	chunk_reset(&trash);
 	orig_len = len;
 	to_forward = 0;
-
-	htxret = htx_find_offset(htx, offset);
-	blk = htxret.blk;
-	offset = htxret.ret;
-	for (; blk && len; blk = htx_get_next_blk(htx, blk)) {
+	for (blk = htx_get_first_blk(htx); blk && len; blk = htx_get_next_blk(htx, blk)) {
 		enum htx_blk_type type = htx_get_blk_type(blk);
 		uint32_t info, sz = htx_get_blksz(blk);
 		struct ist v;
 
+		if (offset >= sz) {
+			offset -= sz;
+			continue;
+		}
+
 		switch (type) {
 			case HTX_BLK_UNUSED:
 				break;
@@ -418,81 +395,6 @@ cache_store_http_payload(struct stream *s, struct filter *filter, struct http_ms
 	return orig_len;
 }
 
-static int
-cache_store_http_forward_data(struct stream *s, struct filter *filter,
-		       struct http_msg *msg, unsigned int len)
-{
-	struct cache_st *st = filter->ctx;
-	struct cache_flt_conf *cconf = FLT_CONF(filter);
-	struct shared_context *shctx = shctx_ptr(cconf->c.cache);
-	int ret;
-
-	ret = 0;
-
-	/*
-	 * We need to skip the HTTP headers first, because we saved them in the
-	 * http-response action.
-	 */
-	if (!(msg->chn->flags & CF_ISRESP) || !st) {
-		/* should never happen */
-		unregister_data_filter(s, msg->chn, filter);
-		return len;
-	}
-
-	if (!len) {
-		/* Nothing to forward */
-		ret = len;
-	}
-	else if (st->hdrs_len >= len) {
-		/* Forward part of headers */
-		ret           = len;
-		st->hdrs_len -= len;
-	}
-	else {
-		/* Forward data */
-		if (st->first_block) {
-			int to_append, append;
-			struct shared_block *fb;
-
-			to_append = MIN(ci_contig_data(msg->chn), len - st->hdrs_len);
-
-			shctx_lock(shctx);
-			fb = shctx_row_reserve_hot(shctx, st->first_block, to_append);
-			if (!fb) {
-				shctx_unlock(shctx);
-				disable_cache_entry(st, filter, shctx);
-				unregister_data_filter(s, msg->chn, filter);
-				return len;
-			}
-			shctx_unlock(shctx);
-
-			/* Skip remaining headers to fill the cache */
-			c_adv(msg->chn, st->hdrs_len);
-			append = shctx_row_data_append(shctx, st->first_block, st->first_block->last_append,
-			                               (unsigned char *)ci_head(msg->chn), to_append);
-			ret = st->hdrs_len + to_append - append;
-			/* Rewind the buffer to forward all data */
-			c_rew(msg->chn, st->hdrs_len);
-			st->hdrs_len = 0;
-			if (ret < 0) {
-				disable_cache_entry(st, filter, shctx);
-				unregister_data_filter(s, msg->chn, filter);
-			}
-		}
-		else {
-			/* should never happen */
-			unregister_data_filter(s, msg->chn, filter);
-			ret = len;
-		}
-	}
-
-	if ((ret != len) ||
-	    (FLT_NXT(filter, msg->chn) != FLT_FWD(filter, msg->chn) + ret))
-		task_wakeup(s->task, TASK_WOKEN_MSG);
-
-	return ret;
-}
-
 static int
 cache_store_http_end(struct stream *s, struct filter *filter,
                      struct http_msg *msg)
@@ -580,66 +482,30 @@ char *directive_value(const char *sample, int slen, const char *word, int wlen)
  */
 int http_calc_maxage(struct stream *s, struct cache *cache)
 {
+	struct htx *htx = htxbuf(&s->res.buf);
+	struct http_hdr_ctx ctx = { .blk = NULL };
 	int smaxage = -1;
 	int maxage = -1;
 
+	while (http_find_header(htx, ist("cache-control"), &ctx, 0)) {
+		char *value;
 
-	if (IS_HTX_STRM(s)) {
-		/* HTX mode */
-		struct htx *htx = htxbuf(&s->res.buf);
-		struct http_hdr_ctx ctx = { .blk = NULL };
+		value = directive_value(ctx.value.ptr, ctx.value.len, "s-maxage", 8);
+		if (value) {
+			struct buffer *chk = get_trash_chunk();
 
-		while (http_find_header(htx, ist("cache-control"), &ctx, 0)) {
-			char *value;
-
-			value = directive_value(ctx.value.ptr, ctx.value.len, "s-maxage", 8);
-			if (value) {
-				struct buffer *chk = get_trash_chunk();
-
-				chunk_strncat(chk, value, ctx.value.len - 8 + 1);
-				chunk_strncat(chk, "", 1);
-				smaxage = atoi(chk->area);
-			}
-
-			value = directive_value(ctx.value.ptr, ctx.value.len, "max-age", 7);
-			if (value) {
-				struct buffer *chk = get_trash_chunk();
-
-				chunk_strncat(chk, value, ctx.value.len - 7 + 1);
-				chunk_strncat(chk, "", 1);
-				maxage = atoi(chk->area);
-			}
+			chunk_strncat(chk, value, ctx.value.len - 8 + 1);
+			chunk_strncat(chk, "", 1);
+			maxage = atoi(chk->area);
 		}
-	}
-	else {
-		/* Legacy mode */
-		struct http_txn *txn = s->txn;
-		struct hdr_ctx ctx;
 
-		ctx.idx = 0;
+		value = directive_value(ctx.value.ptr, ctx.value.len, "max-age", 7);
+		if (value) {
+			struct buffer *chk = get_trash_chunk();
 
-		/* loop on the Cache-Control values */
-		while (http_find_header2("Cache-Control", 13, ci_head(&s->res), &txn->hdr_idx, &ctx)) {
-			char *directive = ctx.line + ctx.val;
-			char *value;
-
-			value = directive_value(directive, ctx.vlen, "s-maxage", 8);
-			if (value) {
-				struct buffer *chk = get_trash_chunk();
-
-				chunk_strncat(chk, value, ctx.vlen - 8 + 1);
-				chunk_strncat(chk, "", 1);
-				smaxage = atoi(chk->area);
-			}
-
-			value = directive_value(ctx.line + ctx.val, ctx.vlen, "max-age", 7);
-			if (value) {
-				struct buffer *chk = get_trash_chunk();
-
-				chunk_strncat(chk, value, ctx.vlen - 7 + 1);
-				chunk_strncat(chk, "", 1);
-				maxage = atoi(chk->area);
-			}
+			chunk_strncat(chk, value, ctx.value.len - 7 + 1);
+			chunk_strncat(chk, "", 1);
+			smaxage = atoi(chk->area);
 		}
 	}
 
@@ -684,6 +550,10 @@ enum act_return http_action_store_cache(struct act_rule *rule, struct proxy *px,
 	struct cache_st *cache_ctx = NULL;
 	struct cache_entry *object, *old;
 	unsigned int key = *(unsigned int *)txn->cache_hash;
+	struct htx *htx;
+	struct http_hdr_ctx ctx;
+	size_t hdrs_len = 0;
+	int32_t pos;
 
 	/* Don't cache if the response came from a cache */
 	if ((obj_type(s->target) == OBJ_TYPE_APPLET) &&
@@ -719,96 +589,53 @@ enum act_return http_action_store_cache(struct act_rule *rule, struct proxy *px,
 	}
 
 	/* from there, cache_ctx is always defined */
+	htx = htxbuf(&s->res.buf);
 
-	if (IS_HTX_STRM(s)) {
-		struct htx *htx = htxbuf(&s->res.buf);
-		struct http_hdr_ctx ctx;
-		size_t hdrs_len = 0;
-		int32_t pos;
+	/* Do not cache too big objects. */
+	if ((msg->flags & HTTP_MSGF_CNT_LEN) && shctx->max_obj_size > 0 &&
+	    htx->data + htx->extra > shctx->max_obj_size)
+		goto out;
 
-		/* Do not cache too big objects. */
-		if ((msg->flags & HTTP_MSGF_CNT_LEN) && shctx->max_obj_size > 0 &&
-		    htx->data + htx->extra > shctx->max_obj_size)
-			goto out;
+	/* Does not manage Vary at the moment. We will need a secondary key later for that */
+	ctx.blk = NULL;
+	if (http_find_header(htx, ist("Vary"), &ctx, 0))
+		goto out;
 
-		/* Does not manage Vary at the moment. We will need a secondary key later for that */
-		ctx.blk = NULL;
-		if (http_find_header(htx, ist("Vary"), &ctx, 0))
-			goto out;
+	http_check_response_for_cacheability(s, &s->res);
 
-		htx_check_response_for_cacheability(s, &s->res);
+	if (!(txn->flags & TX_CACHEABLE) || !(txn->flags & TX_CACHE_COOK))
+		goto out;
 
-		if (!(txn->flags & TX_CACHEABLE) || !(txn->flags & TX_CACHE_COOK))
-			goto out;
-
-		age = 0;
-		ctx.blk = NULL;
-		if (http_find_header(htx, ist("Age"), &ctx, 0)) {
-			if (!strl2llrc(ctx.value.ptr, ctx.value.len, &hdr_age) && hdr_age > 0) {
-				if (unlikely(hdr_age > CACHE_ENTRY_MAX_AGE))
-					hdr_age = CACHE_ENTRY_MAX_AGE;
-				age = hdr_age;
-			}
-			http_remove_header(htx, &ctx);
+	age = 0;
+	ctx.blk = NULL;
+	if (http_find_header(htx, ist("Age"), &ctx, 0)) {
+		if (!strl2llrc(ctx.value.ptr, ctx.value.len, &hdr_age) && hdr_age > 0) {
+			if (unlikely(hdr_age > CACHE_ENTRY_MAX_AGE))
+				hdr_age = CACHE_ENTRY_MAX_AGE;
+			age = hdr_age;
 		}
-
-		chunk_reset(&trash);
-		for (pos = htx_get_first(htx); pos != -1; pos = htx_get_next(htx, pos)) {
-			struct htx_blk *blk = htx_get_blk(htx, pos);
-			enum htx_blk_type type = htx_get_blk_type(blk);
-			uint32_t sz = htx_get_blksz(blk);
-
-			hdrs_len += sizeof(*blk) + sz;
-			chunk_memcat(&trash, (char *)&blk->info, sizeof(blk->info));
-			chunk_memcat(&trash, htx_get_blk_ptr(htx, blk), sz);
-			if (type == HTX_BLK_EOH)
-				break;
-		}
-
-		/* Do not cache objects if the headers are too big. */
-		if (hdrs_len > htx->size - global.tune.maxrewrite)
-			goto out;
-
+		http_remove_header(htx, &ctx);
 	}
-	else {
-		struct hdr_ctx ctx;
 
-		/* Do not cache too big objects. */
-		if ((msg->flags & HTTP_MSGF_CNT_LEN) && shctx->max_obj_size > 0 &&
-		    msg->sov + msg->body_len > shctx->max_obj_size)
-			goto out;
+	chunk_reset(&trash);
+	for (pos = htx_get_first(htx); pos != -1; pos = htx_get_next(htx, pos)) {
+		struct htx_blk *blk = htx_get_blk(htx, pos);
+		enum htx_blk_type type = htx_get_blk_type(blk);
+		uint32_t sz = htx_get_blksz(blk);
 
-		/* Does not manage Vary at the moment. We will need a secondary key later for that */
-		ctx.idx = 0;
-		if (http_find_header2("Vary", 4, ci_head(txn->rsp.chn), &txn->hdr_idx, &ctx))
-			goto out;
-
-		check_response_for_cacheability(s, &s->res);
-
-		if (!(txn->flags & TX_CACHEABLE) || !(txn->flags & TX_CACHE_COOK))
-			goto out;
-
-		age = 0;
-		ctx.idx = 0;
-		if (http_find_header2("Age", 3, ci_head(txn->rsp.chn), &txn->hdr_idx, &ctx)) {
-			if (!strl2llrc(ctx.line + ctx.val, ctx.vlen, &hdr_age) && hdr_age > 0) {
-				if (unlikely(hdr_age > CACHE_ENTRY_MAX_AGE))
-					hdr_age = CACHE_ENTRY_MAX_AGE;
-				age = hdr_age;
-			}
-			http_remove_header2(msg, &txn->hdr_idx, &ctx);
-		}
-
-		/* Do not cache objects if the headers are too big. */
-		if (msg->sov > c_size(txn->rsp.chn) - global.tune.maxrewrite)
-			goto out;
+		hdrs_len += sizeof(*blk) + sz;
+		chunk_memcat(&trash, (char *)&blk->info, sizeof(blk->info));
+		chunk_memcat(&trash, htx_get_blk_ptr(htx, blk), sz);
+		if (type == HTX_BLK_EOH)
+			break;
 	}
 
+	/* Do not cache objects if the headers are too big. */
+	if (hdrs_len > htx->size - global.tune.maxrewrite)
+		goto out;
+
 	shctx_lock(shctx);
-	if (IS_HTX_STRM(s))
-		first = shctx_row_reserve_hot(shctx, NULL, sizeof(struct cache_entry) + trash.data);
-	else
-		first = shctx_row_reserve_hot(shctx, NULL, sizeof(struct cache_entry) + msg->sov);
+	first = shctx_row_reserve_hot(shctx, NULL, sizeof(struct cache_entry) + trash.data);
 	if (!first) {
 		shctx_unlock(shctx);
 		goto out;
@@ -822,8 +649,6 @@ enum act_return http_action_store_cache(struct act_rule *rule, struct proxy *px,
 	object->eb.node.leaf_p = NULL;
 	object->eb.key = 0;
 	object->age = age;
-	if (!IS_HTX_STRM(s))
-		object->eoh = msg->eoh;
 
 	/* reserve space for the cache_entry structure */
 	first->len = sizeof(struct cache_entry);
@@ -836,14 +661,8 @@ enum act_return http_action_store_cache(struct act_rule *rule, struct proxy *px,
 
 	/* does not need to be locked because it's in the "hot" list,
 	 * copy the headers */
-	if (IS_HTX_STRM(s)) {
-		if (shctx_row_data_append(shctx, first, NULL, (unsigned char *)trash.area, trash.data) < 0)
-			goto out;
-	}
-	else {
-		if (shctx_row_data_append(shctx, first, NULL, (unsigned char *)ci_head(&s->res), msg->sov) < 0)
-			goto out;
-	}
+	if (shctx_row_data_append(shctx, first, NULL, (unsigned char *)trash.area, trash.data) < 0)
+		goto out;
 
 	/* register the buffer in the filter ctx for filling it with data*/
 	if (cache_ctx) {
@@ -864,8 +683,8 @@ enum act_return http_action_store_cache(struct act_rule *rule, struct proxy *px,
 		shctx_unlock(shctx);
 
 		/* store latest value and expiration time */
-		object->latest_validation = date.tv_sec;
-		object->expire = date.tv_sec + http_calc_maxage(s, cconf->c.cache);
+		object->latest_validation = now.tv_sec;
+		object->expire = now.tv_sec + http_calc_maxage(s, cconf->c.cache);
 		return ACT_RET_CONT;
 	}
 
@@ -882,11 +701,6 @@ out:
 	return ACT_RET_CONT;
 }
 
-#define 	HTTP_CACHE_INIT   0  /* Initial state. */
-#define 	HTTP_CACHE_HEADER 1  /* Cache entry headers forwarded. */
-#define 	HTTP_CACHE_FWD    2  /* Cache entry completely forwarded. */
-#define 	HTTP_CACHE_END    3  /* Cache entry treatment terminated. */
-
 #define 	HTX_CACHE_INIT   0  /* Initial state. */
 #define 	HTX_CACHE_HEADER 1  /* Cache entry headers forwarding */
 #define 	HTX_CACHE_DATA   2  /* Cache entry data forwarding */
@@ -1059,7 +873,7 @@ static int htx_cache_add_age_hdr(struct appctx *appctx, struct htx *htx)
 	char *end;
 
 	chunk_reset(&trash);
-	age = MAX(0, (int)(date.tv_sec - cache_ptr->latest_validation)) + cache_ptr->age;
+	age = MAX(0, (int)(now.tv_sec - cache_ptr->latest_validation)) + cache_ptr->age;
 	if (unlikely(age > CACHE_ENTRY_MAX_AGE))
 		age = CACHE_ENTRY_MAX_AGE;
 	end = ultoa_o(age, b_head(&trash), b_size(&trash));
@@ -1069,7 +883,7 @@ static int htx_cache_add_age_hdr(struct appctx *appctx, struct htx *htx)
 	return 1;
 }
 
-static void htx_cache_io_handler(struct appctx *appctx)
+static void http_cache_io_handler(struct appctx *appctx)
 {
 	struct cache_entry *cache_ptr = appctx->ctx.cache.entry;
 	struct shared_block *first = block_ptr(cache_ptr);
@@ -1081,7 +895,7 @@ static void htx_cache_io_handler(struct appctx *appctx)
 	unsigned int len;
 	size_t ret, total = 0;
 
-	res_htx = htx_from_buf(&res->buf);
+	res_htx = htxbuf(&res->buf);
 	total = res_htx->data;
 
 	if (unlikely(si->state == SI_ST_DIS || si->state == SI_ST_CLO))
@@ -1136,7 +950,6 @@ static void htx_cache_io_handler(struct appctx *appctx)
 			si_rx_room_blk(si);
 			goto out;
 		}
-		res->flags |= CF_EOI;
 		appctx->st0 = HTX_CACHE_END;
 	}
 
@@ -1163,7 +976,7 @@ static void htx_cache_io_handler(struct appctx *appctx)
   error:
 	/* Sent and HTTP error 500 */
 	b_reset(&res->buf);
-	errmsg = &htx_err_chunks[HTTP_ERR_500];
+	errmsg = &http_err_chunks[HTTP_ERR_500];
 	res->buf.data = b_data(errmsg);
 	memcpy(res->buf.area, b_head(errmsg), b_data(errmsg));
 	res_htx = htx_from_buf(&res->buf);
@@ -1174,139 +987,6 @@ static void htx_cache_io_handler(struct appctx *appctx)
 }
 
 
-/*
- * Append an "Age" header into <chn> channel for this <ce> cache entry.
- * This is the responsibility of the caller to insure there is enough
- * data in the channel.
- *
- * Returns the number of bytes inserted if succeeded, 0 if failed.
- */
-static int cache_channel_append_age_header(struct cache_entry *ce, struct channel *chn)
-{
-	unsigned int age;
-
-	age = MAX(0, (int)(now.tv_sec - ce->latest_validation)) + ce->age;
-	if (unlikely(age > CACHE_ENTRY_MAX_AGE))
-		age = CACHE_ENTRY_MAX_AGE;
-
-	chunk_reset(&trash);
-	chunk_printf(&trash, "Age: %u", age);
-
-	return ci_insert_line2(chn, ce->eoh, trash.area, trash.data);
-}
-
-static int cache_channel_row_data_get(struct appctx *appctx, int len)
-{
-	int ret, total;
-	struct stream_interface *si = appctx->owner;
-	struct channel *res = si_ic(si);
-	struct cache_flt_conf *cconf = appctx->rule->arg.act.p[0];
-	struct cache *cache = cconf->c.cache;
-	struct shared_context *shctx = shctx_ptr(cache);
-	struct cache_entry *cache_ptr = appctx->ctx.cache.entry;
-	struct shared_block *blk, *next = appctx->ctx.cache.next;
-	int offset;
-
-	total = 0;
-	offset = 0;
-
-	if (!next) {
-		offset = sizeof(struct cache_entry);
-		next =  block_ptr(cache_ptr);
-	}
-
-	blk = next;
-	list_for_each_entry_from(blk, &shctx->hot, list) {
-		int sz;
-
-		if (len <= 0)
-			break;
-
-		sz = MIN(len, shctx->block_size - offset);
-
-		ret = ci_putblk(res, (const char *)blk->data + offset, sz);
-		if (unlikely(offset))
-			offset = 0;
-		if (ret <= 0) {
-			if (ret == -3 || ret == -1) {
-				si_rx_room_blk(si);
-				break;
-			}
-			return -1;
-		}
-
-		total += sz;
-		len -= sz;
-	}
-	appctx->ctx.cache.next = blk;
-
-	return total;
-}
-
-static void http_cache_io_handler(struct appctx *appctx)
-{
-	struct stream_interface *si = appctx->owner;
-	struct stream *s = si_strm(si);
-	struct channel *res = si_ic(si);
-	struct cache_entry *cache_ptr = appctx->ctx.cache.entry;
-	struct shared_block *first = block_ptr(cache_ptr);
-	unsigned int *sent = &appctx->ctx.cache.sent;
-
-	if (IS_HTX_STRM(s))
-		return htx_cache_io_handler(appctx);
-
-	if (unlikely(si->state == SI_ST_DIS || si->state == SI_ST_CLO))
-		goto out;
-
-	/* Check if the input buffer is available. */
-	if (res->buf.size == 0) {
-		/* buf.size==0 means we failed to get a buffer and were
-		 * already subscribed to a wait list to get a buffer.
-		 */
-		goto out;
-	}
-
-	if (res->flags & (CF_SHUTW|CF_SHUTW_NOW|CF_SHUTR))
-		appctx->st0 = HTTP_CACHE_END;
-
-	/* buffer are aligned there, should be fine */
-	if (appctx->st0 == HTTP_CACHE_HEADER || appctx->st0 == HTTP_CACHE_INIT) {
-		int len = first->len - *sent - sizeof(struct cache_entry);
-		if (len > 0) {
-			int ret;
-
-			ret = cache_channel_row_data_get(appctx, len);
-			if (ret == -1)
-				appctx->st0 = HTTP_CACHE_END;
-			else
-				*sent += ret;
-			if (appctx->st0 == HTTP_CACHE_INIT && *sent > cache_ptr->eoh &&
-				cache_channel_append_age_header(cache_ptr, res))
-				appctx->st0 = HTTP_CACHE_HEADER;
-			else if (ret == len) {
-				*sent = 0;
-				appctx->st0 = HTTP_CACHE_FWD;
-			}
-		}
-		else {
-			*sent = 0;
-			appctx->st0 = HTTP_CACHE_FWD;
-		}
-	}
-
-	if (appctx->st0 == HTTP_CACHE_FWD)
-		appctx->st0 = HTTP_CACHE_END;
-
-	if (!(res->flags & CF_SHUTR) && appctx->st0 == HTTP_CACHE_END) {
-		res->flags |= CF_READ_NULL;
-		si_shutr(si);
-	}
-out:
-	/* eat the whole request */
-	if (co_data(si_oc(si)))
-	    co_skip(si_oc(si), co_data(si_oc(si)));
-}
-
 static int parse_cache_rule(struct proxy *proxy, const char *name, struct act_rule *rule, char **err)
 {
 	struct flt_conf *fconf;
@@ -1374,52 +1054,56 @@ enum act_parse_ret parse_cache_store(const char **args, int *orig_arg, struct pr
 	return ACT_RET_PRS_OK;
 }
 
-/* This produces a sha1 hash of the concatenation of the first
- * occurrence of the Host header followed by the path component if it
- * begins with a slash ('/'). */
+/* This produces a sha1 hash of the concatenation of the HTTP method,
+ * the first occurrence of the Host header followed by the path component
+ * if it begins with a slash ('/'). */
 int sha1_hosturi(struct stream *s)
 {
 	struct http_txn *txn = s->txn;
+	struct htx *htx = htxbuf(&s->req.buf);
+	struct htx_sl *sl;
+	struct http_hdr_ctx ctx;
+	struct ist uri;
 	blk_SHA_CTX sha1_ctx;
 	struct buffer *trash;
 
 	trash = get_trash_chunk();
+	ctx.blk = NULL;
 
-	if (IS_HTX_STRM(s)) {
-		struct htx *htx = htxbuf(&s->req.buf);
-		struct htx_sl *sl;
-		struct http_hdr_ctx ctx;
-		struct ist path;
+	switch (txn->meth) {
+	case HTTP_METH_HEAD:
+	case HTTP_METH_GET:
+		chunk_memcat(trash, "GET", 3);
+		break;
+	default:
+		return 0;
+	}
 
-		ctx.blk = NULL;
+	sl = http_get_stline(htx);
+	uri = htx_sl_req_uri(sl); // whole uri
+	if (!uri.len)
+		return 0;
+
+	/* In HTTP/1, most URIs are seen in origin form ('/path/to/resource'),
+	 * unless haproxy is deployed in front of an outbound cache. In HTTP/2,
+	 * URIs are almost always sent in absolute form with their scheme. In
+	 * this case, the scheme is almost always "https". In order to support
+	 * sharing of cache objects between H1 and H2, we'll hash the absolute
+	 * URI whenever known, or prepend "https://" + the Host header for
+	 * relative URIs. The difference will only appear on absolute HTTP/1
+	 * requests sent to an origin server, which practically is never met in
+	 * the real world so we don't care about the ability to share the same
+	 * key here.URIs are normalized from the absolute URI to an origin form as
+	 * well.
+	 */
+	if (!(sl->flags & HTX_SL_F_HAS_AUTHORITY)) {
+		chunk_istcat(trash, ist("https://"));
 		if (!http_find_header(htx, ist("Host"), &ctx, 0))
 			return 0;
-		chunk_memcat(trash, ctx.value.ptr, ctx.value.len);
-
-		sl = http_get_stline(htx);
-		path = htx_sl_req_uri(sl); // whole uri
-		if (!path.len || *path.ptr != '/')
-			return 0;
-		chunk_memcat(trash, path.ptr, path.len);
+		chunk_istcat(trash, ctx.value);
 	}
-	else {
-		struct hdr_ctx ctx;
-		char *path;
-		char *end;
 
-		/* retrive the host */
-		ctx.idx = 0;
-		if (!http_find_header2("Host", 4, ci_head(txn->req.chn), &txn->hdr_idx, &ctx))
-			return 0;
-		chunk_strncat(trash, ctx.line + ctx.val, ctx.vlen);
-
-		/* now retrieve the path */
-		end = ci_head(txn->req.chn) + txn->req.sl.rq.u + txn->req.sl.rq.u_l;
-		path = ci_head(txn->req.chn) + txn->req.sl.rq.u;
-		if (!txn->req.sl.rq.u_l || *path != '/')
-			return 0;
-		chunk_strncat(trash, path, end - path);
-	}
+	chunk_memcat(trash, uri.ptr, uri.len);
 
 	/* hash everything */
 	blk_SHA1_Init(&sha1_ctx);
@@ -1444,10 +1128,7 @@ enum act_return http_action_req_cache_use(struct act_rule *rule, struct proxy *p
 	    (txn->meth != HTTP_METH_GET && txn->meth != HTTP_METH_HEAD))
 		txn->flags |= TX_CACHE_IGNORE;
 
-	if (IS_HTX_STRM(s))
-		htx_check_request_for_cacheability(s, &s->req);
-	else
-		check_request_for_cacheability(s, &s->req);
+	http_check_request_for_cacheability(s, &s->req);
 
 	if ((s->txn->flags & (TX_CACHE_IGNORE|TX_CACHEABLE)) == TX_CACHE_IGNORE)
 		return ACT_RET_CONT;
@@ -1471,7 +1152,7 @@ enum act_return http_action_req_cache_use(struct act_rule *rule, struct proxy *p
 		shctx_unlock(shctx_ptr(cache));
 		s->target = &http_cache_applet.obj_type;
 		if ((appctx = si_register_handler(&s->si[1], objt_applet(s->target)))) {
-			appctx->st0 = HTTP_CACHE_INIT;
+			appctx->st0 = HTX_CACHE_INIT;
 			appctx->rule = rule;
 			appctx->ctx.cache.entry = res;
 			appctx->ctx.cache.next = NULL;
@@ -1483,11 +1164,10 @@ enum act_return http_action_req_cache_use(struct act_rule *rule, struct proxy *p
 				_HA_ATOMIC_ADD(&px->be_counters.p.http.cache_hits, 1);
 			return ACT_RET_CONT;
 		} else {
-			s->target = NULL;
 			shctx_lock(shctx_ptr(cache));
 			shctx_row_dec_hot(shctx_ptr(cache), block_ptr(res));
 			shctx_unlock(shctx_ptr(cache));
-			return ACT_RET_CONT;
+			return ACT_RET_YIELD;
 		}
 	}
 	shctx_unlock(shctx_ptr(cache));
@@ -1543,7 +1223,6 @@ int cfg_parse_cache(const char *file, int linenum, char **args, int kwm)
 			tmp_cache_config->maxage = 60;
 			tmp_cache_config->maxblocks = 0;
 			tmp_cache_config->maxobjsz = 0;
-			tmp_cache_config->flags = 0;
 		}
 	} else if (strcmp(args[0], "total-max-size") == 0) {
 		unsigned long int maxsize;
@@ -1716,29 +1395,6 @@ out:
 
 }
 
-/*
- * Resolve the cache name to a pointer once the file is completely read.
- */
-int cfg_cache_postparser()
-{
-	struct cache *cache;
-	int err = 0;
-
-	/* Check if the cache is used by HTX and legacy HTTP proxies in same
-	 * time
-	 */
-	list_for_each_entry(cache, &caches_config, list) {
-		if ((cache->flags & (CACHE_F_HTX|CACHE_F_LEGACY_HTTP)) == (CACHE_F_HTX|CACHE_F_LEGACY_HTTP)) {
-			ha_alert("Cache '%s': cannot be used by HTX and legacy HTTP proxies in same time.\n",
-				 cache->id);
-			err++;
-		}
-	}
-
-	return err;
-}
-
-
 struct flt_ops cache_ops = {
 	.init   = cache_store_init,
 	.check  = cache_store_check,
@@ -1753,9 +1409,6 @@ struct flt_ops cache_ops = {
 	.http_headers        = cache_store_http_headers,
 	.http_payload        = cache_store_http_payload,
 	.http_end            = cache_store_http_end,
-
-	.http_forward_data   = cache_store_http_forward_data,
-
 };
 
 
@@ -1769,17 +1422,19 @@ parse_cache_flt(char **args, int *cur_arg, struct proxy *px,
 	char *name = NULL;
 	int pos = *cur_arg;
 
-	/* Get the cache filter name. <pos> point on "cache" keyword */
-	if (!*args[pos + 1]) {
-		memprintf(err, "%s : expects a <name> argument", args[pos]);
-		goto error;
+	/* Get the cache filter name*/
+	if (!strcmp(args[pos], "cache")) {
+		if (!*args[pos + 1]) {
+			memprintf(err, "%s : expects an <id> argument", args[pos]);
+			goto error;
+		}
+		name = strdup(args[pos + 1]);
+		if (!name) {
+			memprintf(err, "%s '%s' : out of memory", args[pos], args[pos + 1]);
+			goto error;
+		}
+		pos += 2;
 	}
-	name = strdup(args[pos + 1]);
-	if (!name) {
-		memprintf(err, "%s '%s' : out of memory", args[pos], args[pos + 1]);
-		goto error;
-	}
-	pos += 2;
 
 	/* Check if an implicit filter with the same name already exists. If so,
 	 * we remove the implicit filter to use the explicit one. */
@@ -1875,19 +1530,9 @@ static int cli_io_handler_show_cache(struct appctx *appctx)
 			}
 
 			entry = container_of(node, struct cache_entry, eb);
+			chunk_printf(&trash, "%p hash:%u size:%u (%u blocks), refcount:%u, expire:%d\n", entry, (*(unsigned int *)entry->hash), block_ptr(entry)->len, block_ptr(entry)->block_count, block_ptr(entry)->refcount, entry->expire - (int)now.tv_sec);
+
 			next_key = node->key + 1;
-
-			if (entry->expire > date.tv_sec) {
-				chunk_printf(&trash, "%p hash:%u size:%u (%u blocks), refcount:%u, expire:%d\n",
-					     entry, (*(unsigned int *)entry->hash),
-					     block_ptr(entry)->len, block_ptr(entry)->block_count,
-					     block_ptr(entry)->refcount, entry->expire - (int)date.tv_sec);
-			} else {
-				/* time to remove that one */
-				eb32_delete(&entry->eb);
-				entry->eb.key = 0;
-			}
-
 			appctx->ctx.cli.i0 = next_key;
 
 			shctx_unlock(shctx_ptr(cache));
@@ -1947,5 +1592,4 @@ struct applet http_cache_applet = {
 
 /* config parsers for this section */
 REGISTER_CONFIG_SECTION("cache", cfg_parse_cache, cfg_post_parse_section_cache);
-REGISTER_CONFIG_POSTPARSER("cache", cfg_cache_postparser);
 REGISTER_POST_CHECK(post_check_cache);
