@@ -41,7 +41,6 @@
 #include <common/base64.h>
 
 #include <types/applet.h>
-#include <types/cli.h>
 #include <types/global.h>
 #include <types/dns.h>
 #include <types/stats.h>
@@ -50,6 +49,7 @@
 #include <proto/backend.h>
 #include <proto/channel.h>
 #include <proto/checks.h>
+#include <proto/cli.h>
 #include <proto/compression.h>
 #include <proto/stats.h>
 #include <proto/fd.h>
@@ -96,7 +96,7 @@ static struct cli_kw_list cli_keywords = {
 
 extern const char *stat_status_codes[];
 
-struct proxy *mworker_proxy; /* CLI proxy of the master */
+static struct proxy *mworker_proxy; /* CLI proxy of the master */
 
 static char *cli_gen_usage_msg(struct appctx *appctx)
 {
@@ -125,6 +125,10 @@ static char *cli_gen_usage_msg(struct appctx *appctx)
 			if (master == 1 && !(kw->level & (ACCESS_MASTER_ONLY|ACCESS_MASTER)))
 				goto next_kw;
 
+			/* only show expert commands in expert mode */
+			if ((kw->level & ~appctx->cli_level) & ACCESS_EXPERT)
+				goto next_kw;
+
 			if (kw->usage)
 				chunk_appendf(tmp, "  %s\n", kw->usage);
 
@@ -185,39 +189,6 @@ struct cli_kw* cli_find_kw(char **args)
 	return NULL;
 }
 
-struct cli_kw* cli_find_kw_exact(char **args)
-{
-	struct cli_kw_list *kw_list;
-	int found = 0;
-	int i;
-	int j;
-
-	if (LIST_ISEMPTY(&cli_keywords.list))
-		return NULL;
-
-	list_for_each_entry(kw_list, &cli_keywords.list, list) {
-		for (i = 0; kw_list->kw[i].str_kw[0]; i++) {
-			found = 1;
-			for (j = 0; j < CLI_PREFIX_KW_NB; j++) {
-				if (args[j] == NULL && kw_list->kw[i].str_kw[j] == NULL) {
-					break;
-				}
-				if (args[j] == NULL || kw_list->kw[i].str_kw[j] == NULL) {
-					found = 0;
-					break;
-				}
-				if (strcmp(args[j], kw_list->kw[i].str_kw[j]) != 0) {
-					found = 0;
-					break;
-				}
-			}
-			if (found)
-				return &kw_list->kw[i];
-		}
-	}
-	return NULL;
-}
-
 void cli_register_kw(struct cli_kw_list *kw_list)
 {
 	LIST_ADDQ(&cli_keywords.list, &kw_list->list);
@@ -490,9 +461,7 @@ int cli_has_level(struct appctx *appctx, int level)
 {
 
 	if ((appctx->cli_level & ACCESS_LVL_MASK) < level) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = stats_permission_denied_msg;
-		appctx->st0 = CLI_ST_PRINT;
+		cli_err(appctx, stats_permission_denied_msg);
 		return 0;
 	}
 	return 1;
@@ -533,10 +502,27 @@ static int cli_parse_request(struct appctx *appctx)
 
 	appctx->st2 = 0;
 	memset(&appctx->ctx.cli, 0, sizeof(appctx->ctx.cli));
+	LIST_INIT(&appctx->ctx.cli.l0);
 
 	p = appctx->chunk->area;
 	end = p + appctx->chunk->data;
 
+	/*
+	 * Get the payload start if there is one.
+	 * For the sake of simplicity, the payload pattern is looked up
+	 * everywhere from the start of the input but it can only be found
+	 * at the end of the first line if APPCTX_CLI_ST1_PAYLOAD is set.
+	 *
+	 * The input string was zero terminated so it is safe to use
+	 * the str*() functions throughout the parsing
+	 */
+	if (appctx->st1 & APPCTX_CLI_ST1_PAYLOAD) {
+		payload = strstr(p, PAYLOAD_PATTERN);
+		end = payload;
+		/* skip the pattern */
+		payload += strlen(PAYLOAD_PATTERN);
+	}
+
 	/*
 	 * Get pointers on words.
 	 * One extra slot is reserved to store a pointer on a null byte.
@@ -549,28 +535,8 @@ static int cli_parse_request(struct appctx *appctx)
 		if (!*p)
 			break;
 
-		if (strcmp(p, PAYLOAD_PATTERN) == 0) {
-			/* payload pattern recognized here, this is not an arg anymore,
-			 * the payload starts at the first byte that follows the zero
-			 * after the pattern.
-			 */
-			payload = p + strlen(PAYLOAD_PATTERN) + 1;
-			break;
-		}
-
 		args[i] = p;
-		while (1) {
-			p += strcspn(p, " \t\\");
-			/* escaped chars using backlashes (\) */
-			if (*p == '\\') {
-				if (!*++p)
-					break;
-				if (!*++p)
-					break;
-			} else {
-				break;
-			}
-		}
+		p += strcspn(p, " \t");
 		*p++ = 0;
 
 		/* unescape backslashes (\) */
@@ -605,6 +571,10 @@ static int cli_parse_request(struct appctx *appctx)
 	if (master == 1 && !(kw->level & (ACCESS_MASTER_ONLY|ACCESS_MASTER)))
 		return 0;
 
+	/* only accept expert commands in expert mode */
+	if ((kw->level & ~appctx->cli_level) & ACCESS_EXPERT)
+		return 0;
+
 	appctx->io_handler = kw->io_handler;
 	appctx->io_release = kw->io_release;
 
@@ -698,7 +668,6 @@ static void cli_io_handler(struct appctx *appctx)
 			 */
 			si_shutw(si);
 			free_trash_chunk(appctx->chunk);
-			appctx->chunk = NULL;
 			break;
 		}
 		else if (appctx->st0 == CLI_ST_GETREQ) {
@@ -723,20 +692,9 @@ static void cli_io_handler(struct appctx *appctx)
 				break;
 			}
 
-			/* payload doesn't take escapes nor does it end on semi-colons, so
-			 * we use the regular getline. Normal mode however must stop on
-			 * LFs and semi-colons that are not prefixed by a backslash. Note
-			 * that we reserve one byte at the end to insert a trailing nul byte.
-			 */
-
-			if (appctx->st1 & APPCTX_CLI_ST1_PAYLOAD)
-				reql = co_getline(si_oc(si), str,
-				                  appctx->chunk->size - appctx->chunk->data - 1);
-			else
-				reql = co_getdelim(si_oc(si), str,
-				                   appctx->chunk->size - appctx->chunk->data - 1,
-				                   "\n;", '\\');
-
+			/* '- 1' is to ensure a null byte can always be inserted at the end */
+			reql = co_getline(si_oc(si), str,
+					  appctx->chunk->size - appctx->chunk->data - 1);
 			if (reql <= 0) { /* closed or EOL not found */
 				if (reql == 0)
 					break;
@@ -809,10 +767,8 @@ static void cli_io_handler(struct appctx *appctx)
 				 * Its location is not remembered here, this is just to switch
 				 * to a gathering mode.
 				 */
-				if (strcmp(appctx->chunk->area + appctx->chunk->data - strlen(PAYLOAD_PATTERN), PAYLOAD_PATTERN) == 0) {
+				if (!strcmp(appctx->chunk->area + appctx->chunk->data - strlen(PAYLOAD_PATTERN), PAYLOAD_PATTERN))
 					appctx->st1 |= APPCTX_CLI_ST1_PAYLOAD;
-					appctx->chunk->data++; // keep the trailing \0 after '<<'
-				}
 				else {
 					/* no payload, the command is complete: parse the request */
 					if (!cli_parse_request(appctx))
@@ -827,30 +783,47 @@ static void cli_io_handler(struct appctx *appctx)
 			req->flags |= CF_READ_DONTWAIT; /* we plan to read small requests */
 		}
 		else {	/* output functions */
+			const char *msg;
+			int sev;
+
 			switch (appctx->st0) {
 			case CLI_ST_PROMPT:
 				break;
-			case CLI_ST_PRINT:
-				if (cli_output_msg(res, appctx->ctx.cli.msg, appctx->ctx.cli.severity,
-							cli_get_severity_output(appctx)) != -1)
-					appctx->st0 = CLI_ST_PROMPT;
-				else
-					si_rx_room_blk(si);
-				break;
-			case CLI_ST_PRINT_FREE: {
-				const char *msg = appctx->ctx.cli.err;
+			case CLI_ST_PRINT:       /* print const message in msg */
+			case CLI_ST_PRINT_ERR:   /* print const error in msg */
+			case CLI_ST_PRINT_DYN:   /* print dyn message in msg, free */
+			case CLI_ST_PRINT_FREE:  /* print dyn error in err, free */
+				if (appctx->st0 == CLI_ST_PRINT || appctx->st0 == CLI_ST_PRINT_ERR) {
+					sev = appctx->st0 == CLI_ST_PRINT_ERR ?
+						LOG_ERR : appctx->ctx.cli.severity;
+					msg = appctx->ctx.cli.msg;
+				}
+				else if (appctx->st0 == CLI_ST_PRINT_DYN || appctx->st0 == CLI_ST_PRINT_FREE) {
+					sev = appctx->st0 == CLI_ST_PRINT_FREE ?
+						LOG_ERR : appctx->ctx.cli.severity;
+					msg = appctx->ctx.cli.err;
+					if (!msg) {
+						sev = LOG_ERR;
+						msg = "Out of memory.\n";
+					}
+				}
+				else {
+					sev = LOG_ERR;
+					msg = "Internal error.\n";
+				}
 
-				if (!msg)
-					msg = "Out of memory.\n";
-
-				if (cli_output_msg(res, msg, LOG_ERR, cli_get_severity_output(appctx)) != -1) {
-					free(appctx->ctx.cli.err);
+				if (cli_output_msg(res, msg, sev, cli_get_severity_output(appctx)) != -1) {
+					if (appctx->st0 == CLI_ST_PRINT_FREE ||
+					    appctx->st0 == CLI_ST_PRINT_DYN) {
+						free(appctx->ctx.cli.err);
+						appctx->ctx.cli.err = NULL;
+					}
 					appctx->st0 = CLI_ST_PROMPT;
 				}
 				else
 					si_rx_room_blk(si);
 				break;
-			}
+
 			case CLI_ST_CALLBACK: /* use custom pointer */
 				if (appctx->io_handler)
 					if (appctx->io_handler(appctx)) {
@@ -910,17 +883,6 @@ static void cli_io_handler(struct appctx *appctx)
 			appctx->st0 = CLI_ST_GETREQ;
 			/* reactivate the \n at the end of the response for the next command */
 			appctx->st1 &= ~APPCTX_CLI_ST1_NOLF;
-
-			/* this forces us to yield between pipelined commands and
-			 * avoid extremely long latencies (e.g. "del map" etc). In
-			 * addition this increases the likelihood that the stream
-			 * refills the buffer with new bytes in non-interactive
-			 * mode, avoiding to close on apparently empty commands.
-			 */
-			if (co_data(si_oc(si))) {
-				appctx_wakeup(appctx);
-				goto out;
-			}
 		}
 	}
 
@@ -958,14 +920,11 @@ static void cli_io_handler(struct appctx *appctx)
  */
 static void cli_release_handler(struct appctx *appctx)
 {
-	free_trash_chunk(appctx->chunk);
-	appctx->chunk = NULL;
-
 	if (appctx->io_release) {
 		appctx->io_release(appctx);
 		appctx->io_release = NULL;
 	}
-	else if (appctx->st0 == CLI_ST_PRINT_FREE) {
+	else if (appctx->st0 == CLI_ST_PRINT_FREE || appctx->st0 == CLI_ST_PRINT_DYN) {
 		free(appctx->ctx.cli.err);
 		appctx->ctx.cli.err = NULL;
 	}
@@ -1058,13 +1017,11 @@ static int cli_io_handler_show_fd(struct appctx *appctx)
 			li = fdt.owner;
 
 		chunk_printf(&trash,
-			     "  %5d : st=0x%02x(R:%c%c%c W:%c%c%c) ev=0x%02x(%c%c%c%c%c) [%c%c] cnext=%d cprev=%d tmask=0x%lx umask=0x%lx owner=%p iocb=%p(%s)",
+			     "  %5d : st=0x%02x(R:%c%c W:%c%c) ev=0x%02x(%c%c%c%c%c) [%c%c] tmask=0x%lx umask=0x%lx owner=%p iocb=%p(%s)",
 			     fd,
 			     fdt.state,
-			     (fdt.state & FD_EV_POLLED_R) ? 'P' : 'p',
 			     (fdt.state & FD_EV_READY_R)  ? 'R' : 'r',
 			     (fdt.state & FD_EV_ACTIVE_R) ? 'A' : 'a',
-			     (fdt.state & FD_EV_POLLED_W) ? 'P' : 'p',
 			     (fdt.state & FD_EV_READY_W)  ? 'R' : 'r',
 			     (fdt.state & FD_EV_ACTIVE_W) ? 'A' : 'a',
 			     fdt.ev,
@@ -1075,8 +1032,6 @@ static int cli_io_handler_show_fd(struct appctx *appctx)
 			     (fdt.ev & FD_POLL_IN)  ? 'I' : 'i',
 			     fdt.linger_risk ? 'L' : 'l',
 			     fdt.cloned ? 'C' : 'c',
-			     fdt.cache.next,
-			     fdt.cache.prev,
 			     fdt.thread_mask, fdt.update_mask,
 			     fdt.owner,
 			     fdt.iocb,
@@ -1098,7 +1053,7 @@ static int cli_io_handler_show_fd(struct appctx *appctx)
 			if (px)
 				chunk_appendf(&trash, " px=%s", px->id);
 			else if (sv)
-				chunk_appendf(&trash, " sv=%s/%s", sv->proxy->id, sv->id);
+				chunk_appendf(&trash, " sv=%s/%s", sv->id, sv->proxy->id);
 			else if (li)
 				chunk_appendf(&trash, " fe=%s", li->bind_conf->frontend->id);
 
@@ -1160,10 +1115,8 @@ static int cli_io_handler_show_activity(struct appctx *appctx)
 		unsigned int _v[MAX_THREADS];				\
 		unsigned int _tot;					\
 		const unsigned int _nbt = global.nbthread;		\
-		_tot = t = 0;						\
-		do {							\
+		for (_tot = t = 0; t < _nbt; t++)			\
 			_tot += _v[t] = (x);				\
-		} while (++t < _nbt);					\
 		if (_nbt == 1) {					\
 			chunk_appendf(&trash, " %u\n", _tot);		\
 			break;						\
@@ -1180,10 +1133,8 @@ static int cli_io_handler_show_activity(struct appctx *appctx)
 		unsigned int _v[MAX_THREADS];				\
 		unsigned int _tot;					\
 		const unsigned int _nbt = global.nbthread;		\
-		_tot = t = 0;						\
-		do {							\
+		for (_tot = t = 0; t < _nbt; t++)			\
 			_tot += _v[t] = (x);				\
-		} while (++t < _nbt);					\
 		if (_nbt == 1) {					\
 			chunk_appendf(&trash, " %u\n", _tot);		\
 			break;						\
@@ -1197,7 +1148,6 @@ static int cli_io_handler_show_activity(struct appctx *appctx)
 	chunk_appendf(&trash, "thread_id: %u (%u..%u)\n", tid + 1, 1, global.nbthread);
 	chunk_appendf(&trash, "date_now: %lu.%06lu\n", (long)now.tv_sec, (long)now.tv_usec);
 	chunk_appendf(&trash, "loops:");        SHOW_TOT(thr, activity[thr].loops);
-	chunk_appendf(&trash, "wake_cache:");   SHOW_TOT(thr, activity[thr].wake_cache);
 	chunk_appendf(&trash, "wake_tasks:");   SHOW_TOT(thr, activity[thr].wake_tasks);
 	chunk_appendf(&trash, "wake_signal:");  SHOW_TOT(thr, activity[thr].wake_signal);
 	chunk_appendf(&trash, "poll_exp:");     SHOW_TOT(thr, activity[thr].poll_exp);
@@ -1245,11 +1195,11 @@ static int cli_io_handler_show_activity(struct appctx *appctx)
 
 /*
  * CLI IO handler for `show cli sockets`.
- * Uses ctx.cli.p0 to store the bind_conf pointer, and cli.p1 for the listener.
+ * Uses ctx.cli.p0 to store the restart pointer.
  */
 static int cli_io_handler_show_cli_sock(struct appctx *appctx)
 {
-	struct bind_conf *bind_conf = appctx->ctx.cli.p0;
+	struct bind_conf *bind_conf;
 	struct stream_interface *si = appctx->owner;
 
 	chunk_reset(&trash);
@@ -1265,16 +1215,23 @@ static int cli_io_handler_show_cli_sock(struct appctx *appctx)
 
 		case STAT_ST_LIST:
 			if (global.stats_fe) {
-				if (!bind_conf)
-					bind_conf = LIST_ELEM(global.stats_fe->conf.bind.n, typeof(bind_conf), by_fe);
+				list_for_each_entry(bind_conf, &global.stats_fe->conf.bind, by_fe) {
+					struct listener *l;
 
-				list_for_each_entry_from(bind_conf, &global.stats_fe->conf.bind, by_fe) {
-					struct listener *l = appctx->ctx.cli.p1;
+					/*
+					 * get the latest dumped node in appctx->ctx.cli.p0
+					 * if the current node is the first of the list
+					 */
 
-					if (!l)
-						l = LIST_ELEM(bind_conf->listeners.n, typeof(l), by_bind);
+					if (appctx->ctx.cli.p0  &&
+					    &bind_conf->by_fe == (&global.stats_fe->conf.bind)->n) {
+						/* change the current node to the latest dumped and continue the loop */
+						bind_conf = LIST_ELEM(appctx->ctx.cli.p0, typeof(bind_conf), by_fe);
+						continue;
+					}
+
+					list_for_each_entry(l, &bind_conf->listeners, by_bind) {
 
-					list_for_each_entry_from(l, &bind_conf->listeners, by_bind) {
 						char addr[46];
 						char port[6];
 
@@ -1324,13 +1281,11 @@ static int cli_io_handler_show_cli_sock(struct appctx *appctx)
 						}
 
 						if (ci_putchk(si_ic(si), &trash) == -1) {
-							/* buffer full, we must yield */
-							appctx->ctx.cli.p0 = bind_conf;
-							appctx->ctx.cli.p1 = l;
 							si_rx_room_blk(si);
 							return 0;
 						}
 					}
+					appctx->ctx.cli.p0 = &bind_conf->by_fe; /* store the latest list node dumped */
 				}
 			}
 		default:
@@ -1362,12 +1317,9 @@ static int cli_parse_show_env(char **args, char *payload, struct appctx *appctx,
 			    (*var)[len] == '=')
 				break;
 		}
-		if (!*var) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Variable not found\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!*var)
+			return cli_err(appctx, "Variable not found\n");
+
 		appctx->st2 = STAT_ST_END;
 	}
 	appctx->ctx.cli.p0 = var;
@@ -1402,31 +1354,19 @@ static int cli_parse_set_timeout(char **args, char *payload, struct appctx *appc
 		unsigned timeout;
 		const char *res;
 
-		if (!*args[3]) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Expects an integer value.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!*args[3])
+			return cli_err(appctx, "Expects an integer value.\n");
 
 		res = parse_time_err(args[3], &timeout, TIME_UNIT_S);
-		if (res || timeout < 1) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Invalid timeout value.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (res || timeout < 1)
+			return cli_err(appctx, "Invalid timeout value.\n");
 
 		s->req.rto = s->res.wto = 1 + MS_TO_TICKS(timeout*1000);
 		task_wakeup(s->task, TASK_WOKEN_MSG); // recompute timeouts
 		return 1;
 	}
-	else {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "'set timeout' only supports 'cli'.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+
+	return cli_err(appctx, "'set timeout' only supports 'cli'.\n");
 }
 
 /* parse a "set maxconn global" command. It always returns 1. */
@@ -1437,20 +1377,12 @@ static int cli_parse_set_maxconn_global(char **args, char *payload, struct appct
 	if (!cli_has_level(appctx, ACCESS_LVL_ADMIN))
 		return 1;
 
-	if (!*args[3]) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Expects an integer value.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (!*args[3])
+		return cli_err(appctx, "Expects an integer value.\n");
 
 	v = atoi(args[3]);
-	if (v > global.hardmaxconn) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Value out of range.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (v > global.hardmaxconn)
+		return cli_err(appctx, "Value out of range.\n");
 
 	/* check for unlimited values */
 	if (v <= 0)
@@ -1459,7 +1391,7 @@ static int cli_parse_set_maxconn_global(char **args, char *payload, struct appct
 	global.maxconn = v;
 
 	/* Dequeues all of the listeners waiting for a resource */
-	if (!LIST_ISEMPTY(&global_listener_queue))
+	if (!MT_LIST_ISEMPTY(&global_listener_queue))
 		dequeue_all_listeners(&global_listener_queue);
 
 	return 1;
@@ -1488,30 +1420,21 @@ static int cli_parse_set_severity_output(char **args, char *payload, struct appc
 	if (*args[2] && set_severity_output(&appctx->cli_severity_output, args[2]))
 		return 0;
 
-	appctx->ctx.cli.severity = LOG_ERR;
-	appctx->ctx.cli.msg = "one of 'none', 'number', 'string' is a required argument\n";
-	appctx->st0 = CLI_ST_PRINT;
-	return 1;
+	return cli_err(appctx, "one of 'none', 'number', 'string' is a required argument\n");
 }
 
 
 /* show the level of the current CLI session */
 static int cli_parse_show_lvl(char **args, char *payload, struct appctx *appctx, void *private)
 {
-
-	appctx->ctx.cli.severity = LOG_INFO;
 	if ((appctx->cli_level & ACCESS_LVL_MASK) == ACCESS_LVL_ADMIN)
-		appctx->ctx.cli.msg = "admin\n";
+		return cli_msg(appctx, LOG_INFO, "admin\n");
 	else if ((appctx->cli_level & ACCESS_LVL_MASK) == ACCESS_LVL_OPER)
-		appctx->ctx.cli.msg = "operator\n";
+		return cli_msg(appctx, LOG_INFO, "operator\n");
 	else if ((appctx->cli_level & ACCESS_LVL_MASK) == ACCESS_LVL_USER)
-		appctx->ctx.cli.msg = "user\n";
+		return cli_msg(appctx, LOG_INFO, "user\n");
 	else
-		appctx->ctx.cli.msg = "unknown\n";
-
-	appctx->st0 = CLI_ST_PRINT;
-	return 1;
-
+		return cli_msg(appctx, LOG_INFO, "unknown\n");
 }
 
 /* parse and set the CLI level dynamically */
@@ -1535,23 +1458,29 @@ static int cli_parse_set_lvl(char **args, char *payload, struct appctx *appctx,
 		appctx->cli_level &= ~ACCESS_LVL_MASK;
 		appctx->cli_level |= ACCESS_LVL_USER;
 	}
+	appctx->cli_level &= ~ACCESS_EXPERT;
 	return 1;
 }
 
-/* shows HAProxy version */
-static int cli_parse_show_version(char **args, char *payload, struct appctx *appctx, void *private)
+
+/* parse and set the CLI expert-mode dynamically */
+static int cli_parse_expert_mode(char **args, char *payload, struct appctx *appctx, void *private)
 {
-	static char *msg = NULL;
+	if (!cli_has_level(appctx, ACCESS_LVL_ADMIN))
+		return 1;
 
-	if (!msg)
-		memprintf(&msg, "%s\n", haproxy_version);
+	if (!*args[1])
+		return (appctx->cli_level & ACCESS_EXPERT)
+			? cli_msg(appctx, LOG_INFO, "expert-mode is ON\n")
+			: cli_msg(appctx, LOG_INFO, "expert-mode is OFF\n");
 
-	appctx->ctx.cli.severity = LOG_INFO;
-	appctx->ctx.cli.msg = msg;
-	appctx->st0 = CLI_ST_PRINT;
+	appctx->cli_level &= ~ACCESS_EXPERT;
+	if (strcmp(args[1], "on") == 0)
+		appctx->cli_level |= ACCESS_EXPERT;
 	return 1;
 }
 
+
 int cli_parse_default(char **args, char *payload, struct appctx *appctx, void *private)
 {
 	return 0;
@@ -1580,38 +1509,27 @@ static int cli_parse_set_ratelimit(char **args, char *payload, struct appctx *ap
 		mul = 1024;
 	}
 	else {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg =
+		return cli_err(appctx,
 			"'set rate-limit' only supports :\n"
 			"   - 'connections global' to set the per-process maximum connection rate\n"
 			"   - 'sessions global' to set the per-process maximum session rate\n"
 #ifdef USE_OPENSSL
 			"   - 'ssl-sessions global' to set the per-process maximum SSL session rate\n"
 #endif
-			"   - 'http-compression global' to set the per-process maximum compression speed in kB/s\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
+			"   - 'http-compression global' to set the per-process maximum compression speed in kB/s\n");
 	}
 
-	if (!*args[4]) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Expects an integer value.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (!*args[4])
+		return cli_err(appctx, "Expects an integer value.\n");
 
 	v = atoi(args[4]);
-	if (v < 0) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Value out of range.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (v < 0)
+		return cli_err(appctx, "Value out of range.\n");
 
 	*res = v * mul;
 
 	/* Dequeues all of the listeners waiting for a resource */
-	if (!LIST_ISEMPTY(&global_listener_queue))
+	if (!MT_LIST_ISEMPTY(&global_listener_queue))
 		dequeue_all_listeners(&global_listener_queue);
 
 	return 1;
@@ -2134,8 +2052,8 @@ int pcli_find_and_exec_kw(struct stream *s, char **args, int argl, char **errmsg
  */
 int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int *next_pid)
 {
-	char *str;
-	char *end;
+	char *str = (char *)ci_head(req);
+	char *end = (char *)ci_stop(req);
 	char *args[MAX_STATS_ARGS + 1]; /* +1 for storing a NULL */
 	int argl; /* number of args */
 	char *p;
@@ -2146,15 +2064,6 @@ int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int
 	int ret;
 	int i = 0;
 
-	/* we cannot deal with a wrapping buffer, so let's take care of this
-	 * first.
-	 */
-	if (b_head(&req->buf) + b_data(&req->buf) > b_wrap(&req->buf))
-		b_slow_realign(&req->buf, trash.area, co_data(req));
-
-	str = (char *)ci_head(req);
-	end = (char *)ci_stop(req);
-
 	p = str;
 
 	if (!(s->pcli_flags & PCLI_F_PAYLOAD)) {
@@ -2163,7 +2072,7 @@ int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int
 		while (p+reql < end) {
 			/* handle escaping */
 			if (p[reql] == '\\') {
-				reql+=2;
+				reql++;
 				continue;
 			}
 			if (p[reql] == ';' || p[reql] == '\n') {
@@ -2189,7 +2098,7 @@ int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int
 	end = p + reql;
 
 	/* there is no end to this command, need more to parse ! */
-	if (!reql || *(end-1) != '\n') {
+	if (*(end-1) != '\n') {
 		return -1;
 	}
 
@@ -2203,25 +2112,29 @@ int pcli_parse_request(struct stream *s, struct channel *req, char **errmsg, int
 
 	/* splits the command in words */
 	while (i < MAX_STATS_ARGS && p < end) {
+		int j, k;
+
 		/* skip leading spaces/tabs */
 		p += strspn(p, " \t");
 		if (!*p)
 			break;
 
 		args[i] = p;
-		while (1) {
-			p += strcspn(p, " \t\\");
-			/* escaped chars using backlashes (\) */
-			if (*p == '\\') {
-				if (!*++p)
-					break;
-				if (!*++p)
-					break;
-			} else {
-				break;
-			}
-		}
+		p += strcspn(p, " \t");
 		*p++ = 0;
+
+		/* unescape backslashes (\) */
+		for (j = 0, k = 0; args[i][k]; k++) {
+			if (args[i][k] == '\\') {
+				if (args[i][k + 1] == '\\')
+					k++;
+				else
+					continue;
+			}
+			args[i][j] = args[i][k];
+			j++;
+		}
+		args[i][j] = 0;
 		i++;
 	}
 
@@ -2288,13 +2201,6 @@ int pcli_wait_for_request(struct stream *s, struct channel *req, int an_bit)
 	int to_forward;
 	char *errmsg = NULL;
 
-	/* Don't read the next command if still processing the reponse of the
-	 * current one. Just wait. At this stage, errors should be handled by
-	 * the response analyzer.
-	 */
-	if (s->res.analysers & AN_RES_WAIT_CLI)
-		return 0;
-
 	if ((s->pcli_flags & ACCESS_LVL_MASK) == ACCESS_LVL_NONE)
 		s->pcli_flags |= strm_li(s)->bind_conf->level & ACCESS_LVL_MASK;
 
@@ -2307,11 +2213,20 @@ read_again:
 	/* We don't know yet to which server we will connect */
 	channel_dont_connect(req);
 
+
+	/* we are not waiting for a response, there is no more request and we
+	 * receive a close from the client, we can leave */
+	if (!(ci_data(req)) && req->flags & CF_SHUTR) {
+		channel_shutw_now(&s->res);
+		s->req.analysers &= ~AN_REQ_WAIT_CLI;
+		return 1;
+	}
+
 	req->flags |= CF_READ_DONTWAIT;
 
 	/* need more data */
 	if (!ci_data(req))
-		goto missing_data;
+		return 0;
 
 	/* If there is data available for analysis, log the end of the idle time. */
 	if (c_data(req) && s->logs.t_idle == -1)
@@ -2333,6 +2248,13 @@ read_again:
 		}
 
 		s->res.flags |= CF_WAKE_ONCE; /* need to be called again */
+
+		/* remove the XFER_DATA analysers, which forwards all
+		 * the data, we don't want to forward the next requests
+		 * We need to add CF_FLT_ANALYZE to abort the forward too.
+		 */
+		req->analysers &= ~(AN_REQ_FLT_XFER_DATA|AN_REQ_WAIT_CLI);
+		req->analysers |= AN_REQ_FLT_END|CF_FLT_ANALYZE;
 		s->res.analysers |= AN_RES_WAIT_CLI;
 
 		if (!(s->flags & SF_ASSIGNED)) {
@@ -2343,9 +2265,6 @@ read_again:
 			/* we can connect now */
 			s->target = pcli_pid_to_server(target_pid);
 
-			if (!s->target)
-				goto server_disconnect;
-
 			s->flags |= (SF_DIRECT | SF_ASSIGNED);
 			channel_auto_connect(req);
 		}
@@ -2354,14 +2273,13 @@ read_again:
 		/* we trimmed things but we might have other commands to consume */
 		pcli_write_prompt(s);
 		goto read_again;
-	} else if (to_forward == -1) {
-                if (errmsg) {
-                        /* there was an error during the parsing */
-                        pcli_reply_and_close(s, errmsg);
-                        s->req.analysers &= ~AN_REQ_WAIT_CLI;
-                        return 0;
-                }
-                goto missing_data;
+	} else if (to_forward == -1 && errmsg) {
+		/* there was an error during the parsing */
+			pcli_reply_and_close(s, errmsg);
+			return 0;
+	} else if (to_forward == -1 && channel_full(req, global.tune.maxrewrite)) {
+		/* buffer is full and we didn't catch the end of a command */
+		goto send_help;
 	}
 
 	return 0;
@@ -2370,24 +2288,6 @@ send_help:
 	b_reset(&req->buf);
 	b_putblk(&req->buf, "help\n", 5);
 	goto read_again;
-
-missing_data:
-        if (req->flags & CF_SHUTR) {
-                /* There is no more request or a only a partial one and we
-                 * receive a close from the client, we can leave */
-                channel_shutw_now(&s->res);
-                s->req.analysers &= ~AN_REQ_WAIT_CLI;
-                return 1;
-        }
-        else if (channel_full(req, global.tune.maxrewrite)) {
-                /* buffer is full and we didn't catch the end of a command */
-                goto send_help;
-        }
-        return 0;
-
-server_disconnect:
-	pcli_reply_and_close(s, "Can't connect to the target CLI!\n");
-	return 0;
 }
 
 int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
@@ -2395,10 +2295,8 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 	struct proxy *fe = strm_fe(s);
 	struct proxy *be = s->be;
 
-	if ((rep->flags & (CF_READ_ERROR|CF_READ_TIMEOUT|CF_WRITE_ERROR|CF_WRITE_TIMEOUT)) ||
-	    ((rep->flags & CF_SHUTW) && (rep->to_forward || co_data(rep)))) {
+	if (rep->flags & CF_READ_ERROR) {
 		pcli_reply_and_close(s, "Can't connect to the target CLI!\n");
-		s->req.analysers &= ~AN_REQ_WAIT_CLI;
 		s->res.analysers &= ~AN_RES_WAIT_CLI;
 		return 0;
 	}
@@ -2410,6 +2308,7 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 	channel_dont_close(&s->req);
 
 	if (s->pcli_flags & PCLI_F_PAYLOAD) {
+		s->req.analysers |= AN_REQ_WAIT_CLI;
 		s->res.analysers &= ~AN_RES_WAIT_CLI;
 		s->req.flags |= CF_WAKE_ONCE; /* need to be called again if there is some command left in the request */
 		return 0;
@@ -2487,7 +2386,7 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 				HA_ATOMIC_SUB(&__objt_server(s->target)->cur_sess, 1);
 			}
 			if (may_dequeue_tasks(__objt_server(s->target), be))
-				process_srv_queue(__objt_server(s->target), 0);
+				process_srv_queue(__objt_server(s->target));
 		}
 
 		s->target = NULL;
@@ -2500,12 +2399,14 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 			s->srv_conn = NULL;
 		}
 
+		sockaddr_free(&s->target_addr);
+
 		s->si[1].state     = s->si[1].prev_state = SI_ST_INI;
 		s->si[1].err_type  = SI_ET_NONE;
 		s->si[1].conn_retries = 0;  /* used for logging too */
 		s->si[1].exp       = TICK_ETERNITY;
 		s->si[1].flags    &= SI_FL_ISBACK | SI_FL_DONT_WAKE; /* we're in the context of process_stream */
-		s->req.flags &= ~(CF_SHUTW|CF_SHUTW_NOW|CF_AUTO_CONNECT|CF_WRITE_ERROR|CF_STREAMER|CF_STREAMER_FAST|CF_NEVER_WAIT|CF_WAKE_CONNECT|CF_WROTE_DATA);
+		s->req.flags &= ~(CF_SHUTW|CF_SHUTW_NOW|CF_AUTO_CONNECT|CF_WRITE_ERROR|CF_STREAMER|CF_STREAMER_FAST|CF_NEVER_WAIT|CF_WROTE_DATA);
 		s->res.flags &= ~(CF_SHUTR|CF_SHUTR_NOW|CF_READ_ATTACHED|CF_READ_ERROR|CF_READ_NOEXP|CF_STREAMER|CF_STREAMER_FAST|CF_WRITE_PARTIAL|CF_NEVER_WAIT|CF_WROTE_DATA|CF_READ_NULL);
 		s->flags &= ~(SF_DIRECT|SF_ASSIGNED|SF_ADDR_SET|SF_BE_ASSIGNED|SF_FORCE_PRST|SF_IGNORE_PRST);
 		s->flags &= ~(SF_CURR_SESS|SF_REDIRECTABLE|SF_SRV_REUSED);
@@ -2528,6 +2429,7 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 
 		s->req.flags |= CF_WAKE_ONCE; /* need to be called again if there is some command left in the request */
 
+		s->req.analysers |= AN_REQ_WAIT_CLI;
 		s->res.analysers &= ~AN_RES_WAIT_CLI;
 
 		/* We must trim any excess data from the response buffer, because we
@@ -2882,10 +2784,10 @@ static struct cli_kw_list cli_kws = {{ },{
 	{ { "show", "cli", "level", NULL },    "show cli level   : display the level of the current CLI session", cli_parse_show_lvl, NULL, NULL, NULL, ACCESS_MASTER},
 	{ { "show", "fd", NULL }, "show fd [num] : dump list of file descriptors in use", cli_parse_show_fd, cli_io_handler_show_fd, NULL },
 	{ { "show", "activity", NULL }, "show activity : show per-thread activity stats (for support/developers)", cli_parse_default, cli_io_handler_show_activity, NULL },
-	{ { "show", "version", NULL },  "show version  : show version of the current process", cli_parse_show_version, NULL, NULL, NULL, ACCESS_MASTER },
 	{ { "operator", NULL },  "operator       : lower the level of the current CLI session to operator", cli_parse_set_lvl, NULL, NULL, NULL, ACCESS_MASTER},
 	{ { "user", NULL },      "user           : lower the level of the current CLI session to user", cli_parse_set_lvl, NULL, NULL, NULL, ACCESS_MASTER},
 	{ { "_getsocks", NULL }, NULL,  _getsocks, NULL },
+	{ { "expert-mode", NULL },  NULL,  cli_parse_expert_mode, NULL }, // not listed
 	{{},}
 }};
 
