@@ -39,7 +39,7 @@
 #include <proto/checks.h>
 #include <proto/dns.h>
 #include <proto/fd.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/http_rules.h>
 #include <proto/log.h>
 #include <proto/sample.h>
@@ -77,19 +77,6 @@ struct dns_resolvers *find_resolvers_by_id(const char *id)
 	return NULL;
 }
 
-/* Compare hostnames in a case-insensitive way .
- * Returns 0 if they are the same, non-zero otherwise
- */
-static __inline int dns_hostname_cmp(const char *name1, const char *name2, int len)
-{
-	int i;
-
-	for (i = 0; i < len; i++)
-		if (tolower(name1[i]) != tolower(name2[i]))
-			return -1;
-	return 0;
-}
-
 /* Returns a pointer on the SRV request matching the name <name> for the proxy
  * <px>. NULL is returned if no match is found.
  */
@@ -113,7 +100,7 @@ struct dns_srvrq *new_dns_srvrq(struct server *srv, char *fqdn)
 	int fqdn_len, hostname_dn_len;
 
 	fqdn_len = strlen(fqdn);
-	hostname_dn_len = dns_str_to_dn_label(fqdn, fqdn_len, trash.area,
+	hostname_dn_len = dns_str_to_dn_label(fqdn, fqdn_len + 1, trash.area,
 					      trash.size);
 	if (hostname_dn_len == -1) {
 		ha_alert("config : %s '%s', server '%s': failed to parse FQDN '%s'\n",
@@ -175,11 +162,11 @@ static void dns_update_resolvers_timeout(struct dns_resolvers *resolvers)
 	next = tick_add(now_ms, resolvers->timeout.resolve);
 	if (!LIST_ISEMPTY(&resolvers->resolutions.curr)) {
 		res  = LIST_NEXT(&resolvers->resolutions.curr, struct dns_resolution *, list);
-		next = tick_first(next, tick_add(res->last_query, resolvers->timeout.retry));
+		next = MIN(next, tick_add(res->last_query, resolvers->timeout.retry));
 	}
 
 	list_for_each_entry(res, &resolvers->resolutions.wait, list)
-		next = tick_first(next, tick_add(res->last_resolution, dns_resolution_timeout(res)));
+		next = MIN(next, tick_add(res->last_resolution, dns_resolution_timeout(res)));
 
 	resolvers->t->expire = next;
 	task_queue(resolvers->t);
@@ -303,7 +290,7 @@ static int dns_send_query(struct dns_resolution *resolution)
 	resolution->last_query   = now_ms;
 
 	/* Push the resolution at the end of the active list */
-	LIST_DEL_INIT(&resolution->list);
+	LIST_DEL(&resolution->list);
 	LIST_ADDQ(&resolvers->resolutions.curr, &resolution->list);
 	return 0;
 }
@@ -518,23 +505,21 @@ static void dns_check_dns_response(struct dns_resolution *res)
 					HA_SPIN_LOCK(SERVER_LOCK, &srv->lock);
 					if (srv->srvrq == srvrq && srv->svc_port == item->port &&
 					    item->data_len == srv->hostname_dn_len &&
-					    !dns_hostname_cmp(srv->hostname_dn, item->data.target, item->data_len)) {
-						dns_unlink_resolution(srv->dns_requester, 0);
-						srvrq_update_srv_status(srv, 1);
+					    !memcmp(srv->hostname_dn, item->target, item->data_len)) {
+						snr_update_srv_status(srv, 1);
 						free(srv->hostname);
 						free(srv->hostname_dn);
 						srv->hostname        = NULL;
 						srv->hostname_dn     = NULL;
 						srv->hostname_dn_len = 0;
-						memset(&srv->addr, 0, sizeof(srv->addr));
-						srv->svc_port = 0;
+						dns_unlink_resolution(srv->dns_requester);
 					}
 					HA_SPIN_UNLOCK(SERVER_LOCK, &srv->lock);
 				}
 			}
 
 		  rm_obselete_item:
-			LIST_DEL_INIT(&item->list);
+			LIST_DEL(&item->list);
 			pool_free(dns_answer_item_pool, item);
 			continue;
 		}
@@ -552,7 +537,8 @@ static void dns_check_dns_response(struct dns_resolution *res)
 				HA_SPIN_LOCK(SERVER_LOCK, &srv->lock);
 				if (srv->srvrq == srvrq && srv->svc_port == item->port &&
 				    item->data_len == srv->hostname_dn_len &&
-				    !dns_hostname_cmp(srv->hostname_dn, item->data.target, item->data_len)) {
+				    !memcmp(srv->hostname_dn, item->target, item->data_len) &&
+				    !srv->dns_opts.ignore_weight) {
 					int ha_weight;
 
 					/* DNS weight range if from 0 to 65535
@@ -587,10 +573,10 @@ static void dns_check_dns_response(struct dns_resolution *res)
 				const char *msg = NULL;
 				char weight[9];
 				int ha_weight;
-				char hostname[DNS_MAX_NAME_SIZE+1];
+				char hostname[DNS_MAX_NAME_SIZE];
 
-				if (dns_dn_label_to_str(item->data.target, item->data_len,
-							hostname, sizeof(hostname)) == -1) {
+				if (dns_dn_label_to_str(item->target, item->data_len+1,
+							hostname, DNS_MAX_NAME_SIZE) == -1) {
 					HA_SPIN_UNLOCK(SERVER_LOCK, &srv->lock);
 					continue;
 				}
@@ -604,15 +590,17 @@ static void dns_check_dns_response(struct dns_resolution *res)
 				    !(srv->flags & SRV_F_CHECKPORT))
 					srv->check.port = item->port;
 
-				/* DNS weight range if from 0 to 65535
-				 * HAProxy weight is from 0 to 256
-				 * The rule below ensures that weight 0 is well respected
-				 * while allowing a "mapping" from DNS weight into HAProxy's one.
-				 */
-				ha_weight = (item->weight + 255) / 256;
+				if (!srv->dns_opts.ignore_weight) {
+					/* DNS weight range if from 0 to 65535
+					 * HAProxy weight is from 0 to 256
+					 * The rule below ensures that weight 0 is well respected
+					 * while allowing a "mapping" from DNS weight into HAProxy's one.
+					 */
+					ha_weight = (item->weight + 255) / 256;
 
-				snprintf(weight, sizeof(weight), "%d", ha_weight);
-				server_parse_weight_change_request(srv, weight);
+					snprintf(weight, sizeof(weight), "%d", ha_weight);
+					server_parse_weight_change_request(srv, weight);
+				}
 				HA_SPIN_UNLOCK(SERVER_LOCK, &srv->lock);
 			}
 		}
@@ -750,17 +738,8 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 		reader += 2;
 	}
 
-	/* Let's just make gcc happy. The tests above make it clear that
-	 * qdcount==1 hence that we necessarily enter into the loop at least
-	 * once, but gcc seems to be having difficulties following it and
-	 * warns about the risk of NULL dereference at the next line, even
-	 * if a BUG_ON(!query) is used.
-	 */
-	ALREADY_CHECKED(dns_query);
-
 	/* TRUNCATED flag must be checked after we could read the query type
-	 * because a TRUNCATED SRV query type response can still be exploited
-	 */
+	 * because a TRUNCATED SRV query type response can still be exploited */
 	if (dns_query->type != DNS_RTYPE_SRV && flags & DNS_FLAG_TRUNCATED)
 		return DNS_RESP_TRUNCATED;
 
@@ -774,9 +753,6 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 		if (dns_answer_record == NULL)
 			return (DNS_RESP_INVALID);
 
-		/* initialization */
-		LIST_INIT(&dns_answer_record->list);
-
 		offset = 0;
 		len = dns_read_name(resp, bufend, reader, tmpname, DNS_MAX_NAME_SIZE, &offset, 0);
 
@@ -787,7 +763,7 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 
 		/* Check if the current record dname is valid.  previous_dname
 		 * points either to queried dname or last CNAME target */
-		if (dns_query->type != DNS_RTYPE_SRV && dns_hostname_cmp(previous_dname, tmpname, len) != 0) {
+		if (dns_query->type != DNS_RTYPE_SRV && memcmp(previous_dname, tmpname, len) != 0) {
 			pool_free(dns_answer_item_pool, dns_answer_record);
 			if (i == 0) {
 				/* First record, means a mismatch issue between
@@ -860,8 +836,9 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 					pool_free(dns_answer_item_pool, dns_answer_record);
 					return DNS_RESP_INVALID;
 				}
-				dns_answer_record->data.in4.sin_family = AF_INET;
-				memcpy(&dns_answer_record->data.in4.sin_addr, reader, dns_answer_record->data_len);
+				dns_answer_record->address.sa_family = AF_INET;
+				memcpy(&(((struct sockaddr_in *)&dns_answer_record->address)->sin_addr),
+						reader, dns_answer_record->data_len);
 				break;
 
 			case DNS_RTYPE_CNAME:
@@ -885,9 +862,9 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 					return DNS_RESP_INVALID;
 				}
 
-				memcpy(dns_answer_record->data.target, tmpname, len);
-				dns_answer_record->data.target[len] = 0;
-				previous_dname = dns_answer_record->data.target;
+				memcpy(dns_answer_record->target, tmpname, len);
+				dns_answer_record->target[len] = 0;
+				previous_dname = dns_answer_record->target;
 				break;
 
 
@@ -915,8 +892,8 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 					return DNS_RESP_INVALID;
 				}
 				dns_answer_record->data_len = len;
-				memcpy(dns_answer_record->data.target, tmpname, len);
-				dns_answer_record->data.target[len] = 0;
+				memcpy(dns_answer_record->target, tmpname, len);
+				dns_answer_record->target[len] = 0;
 				break;
 
 			case DNS_RTYPE_AAAA:
@@ -925,8 +902,9 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 					pool_free(dns_answer_item_pool, dns_answer_record);
 					return DNS_RESP_INVALID;
 				}
-				dns_answer_record->data.in6.sin6_family = AF_INET6;
-				memcpy(&dns_answer_record->data.in6.sin6_addr, reader, dns_answer_record->data_len);
+				dns_answer_record->address.sa_family = AF_INET6;
+				memcpy(&(((struct sockaddr_in6 *)&dns_answer_record->address)->sin6_addr),
+						reader, dns_answer_record->data_len);
 				break;
 
 		} /* switch (record type) */
@@ -949,22 +927,22 @@ static int dns_validate_dns_response(unsigned char *resp, unsigned char *bufend,
 
 			switch(tmp_record->type) {
 				case DNS_RTYPE_A:
-					if (!memcmp(&dns_answer_record->data.in4.sin_addr,
-						    &tmp_record->data.in4.sin_addr,
-						    sizeof(dns_answer_record->data.in4.sin_addr)))
+					if (!memcmp(&((struct sockaddr_in *)&dns_answer_record->address)->sin_addr,
+						    &((struct sockaddr_in *)&tmp_record->address)->sin_addr,
+						    sizeof(in_addr_t)))
 						found = 1;
 					break;
 
 				case DNS_RTYPE_AAAA:
-					if (!memcmp(&dns_answer_record->data.in6.sin6_addr,
-						    &tmp_record->data.in6.sin6_addr,
-						    sizeof(dns_answer_record->data.in6.sin6_addr)))
+					if (!memcmp(&((struct sockaddr_in6 *)&dns_answer_record->address)->sin6_addr,
+						    &((struct sockaddr_in6 *)&tmp_record->address)->sin6_addr,
+						    sizeof(struct in6_addr)))
 						found = 1;
 					break;
 
 			case DNS_RTYPE_SRV:
                                 if (dns_answer_record->data_len == tmp_record->data_len &&
-				    !dns_hostname_cmp(dns_answer_record->data.target, tmp_record->data.target, dns_answer_record->data_len) &&
+				    !memcmp(dns_answer_record->target, tmp_record->target, dns_answer_record->data_len) &&
 				    dns_answer_record->port == tmp_record->port) {
 					tmp_record->weight = dns_answer_record->weight;
                                         found = 1;
@@ -1044,12 +1022,12 @@ int dns_get_ip_from_response(struct dns_response_packet *dns_p,
 		unsigned char ip_type;
 
 		if (record->type == DNS_RTYPE_A) {
+			ip = &(((struct sockaddr_in *)&record->address)->sin_addr);
 			ip_type = AF_INET;
-			ip = &record->data.in4.sin_addr;
 		}
 		else if (record->type == DNS_RTYPE_AAAA) {
 			ip_type = AF_INET6;
-			ip = &record->data.in6.sin6_addr;
+			ip = &(((struct sockaddr_in6 *)&record->address)->sin6_addr);
 		}
 		else
 			continue;
@@ -1169,21 +1147,19 @@ int dns_get_ip_from_response(struct dns_response_packet *dns_p,
 	list_for_each_entry(record, &dns_p->answer_list, list) {
 		/* Move the first record to the end of the list, for internal
 		 * round robin */
-		LIST_DEL_INIT(&record->list);
+		LIST_DEL(&record->list);
 		LIST_ADDQ(&dns_p->answer_list, &record->list);
 		break;
 	}
 	return DNS_UPD_SRVIP_NOT_FOUND;
 }
 
-/* Turns a domain name label into a string: 3www7haproxy3org into www.haproxy.org
+/* Turns a domain name label into a string.
  *
- * <dn> contains the input label of <dn_len> bytes long and does not need to be
- * null-terminated. <str> must be allocated large enough to contain a full host
- * name plus the trailing zero, and the allocated size must be passed in
- * <str_len>.
+ * <dn> must be a null-terminated string. <dn_len> must include the terminating
+ * null byte. <str> must be allocated and its size must be passed in <str_len>.
  *
- * In case of error, -1 is returned, otherwise, the number of bytes copied in
+ *  In case of error, -1 is returned, otherwise, the number of bytes copied in
  * <str> (including the terminating null byte).
  */
 int dns_dn_label_to_str(const char *dn, int dn_len, char *str, int str_len)
@@ -1191,11 +1167,11 @@ int dns_dn_label_to_str(const char *dn, int dn_len, char *str, int str_len)
 	char *ptr;
 	int i, sz;
 
-	if (str_len < dn_len)
+	if (str_len < dn_len - 1)
 		return -1;
 
 	ptr = str;
-	for (i = 0; i < dn_len; ++i) {
+	for (i = 0; i < dn_len-1; ++i) {
 		sz = dn[i];
 		if (i)
 			*ptr++ = '.';
@@ -1209,19 +1185,18 @@ int dns_dn_label_to_str(const char *dn, int dn_len, char *str, int str_len)
 
 /* Turns a string into domain name label: www.haproxy.org into 3www7haproxy3org
  *
- * <str> contains the input string that is <str_len> bytes long (trailing zero
- * not needed). <dn> buffer must be allocated large enough to contain the
- * encoded string and a trailing zero, so it must be at least str_len+2, and
- * this allocated buffer size must be passed in <dn_len>.
+ * <str> must be a null-terminated string. <str_len> must include the
+ * terminating null byte. <dn> buffer must be allocated and its size must be
+ * passed in <dn_len>.
  *
- * In case of error, -1 is returned, otherwise, the number of bytes copied in
+ *  In case of error, -1 is returned, otherwise, the number of bytes copied in
  * <dn> (excluding the terminating null byte).
  */
 int dns_str_to_dn_label(const char *str, int str_len, char *dn, int dn_len)
 {
 	int i, offset;
 
-	if (dn_len < str_len + 2)
+	if (dn_len < str_len + 1)
 		return -1;
 
 	/* First byte of dn will be used to store the length of the first
@@ -1233,19 +1208,15 @@ int dns_str_to_dn_label(const char *str, int str_len, char *dn, int dn_len)
 			if (i == offset)
 				return -1;
 
-			/* ignore trailing dot */
-			if (i + 1 == str_len)
-				break;
-
 			dn[offset] = (i - offset);
 			offset = i+1;
 			continue;
 		}
 		dn[i+1] = str[i];
 	}
-	dn[offset] = (i - offset);
-	dn[i+1] = '\0';
-	return i+1;
+	dn[offset] = (i - offset - 1);
+	dn[i] = '\0';
+	return i;
 }
 
 /* Validates host name:
@@ -1317,7 +1288,7 @@ static struct dns_resolution *dns_pick_resolution(struct dns_resolvers *resolver
 			continue;
 		if ((query_type == res->prefered_query_type) &&
 		    hostname_dn_len == res->hostname_dn_len  &&
-		    !dns_hostname_cmp(*hostname_dn, res->hostname_dn, hostname_dn_len))
+		    !memcmp(*hostname_dn, res->hostname_dn, hostname_dn_len))
 			return res;
 	}
 
@@ -1327,7 +1298,7 @@ static struct dns_resolution *dns_pick_resolution(struct dns_resolvers *resolver
 			continue;
 		if ((query_type == res->prefered_query_type) &&
 		    hostname_dn_len == res->hostname_dn_len  &&
-		    !dns_hostname_cmp(*hostname_dn, res->hostname_dn, hostname_dn_len))
+		    !memcmp(*hostname_dn, res->hostname_dn, hostname_dn_len))
 			return res;
 	}
 
@@ -1358,17 +1329,6 @@ static struct dns_resolution *dns_pick_resolution(struct dns_resolvers *resolver
 	return res;
 }
 
-/* deletes and frees all answer_items from the resolution's answer_list */
-static void dns_purge_resolution_answer_records(struct dns_resolution *resolution)
-{
-	struct dns_answer_item *item, *itemback;
-
-	list_for_each_entry_safe(item, itemback, &resolution->response.answer_list, list) {
-		LIST_DEL_INIT(&item->list);
-		pool_free(dns_answer_item_pool, item);
-	}
-}
-
 /* Releases a resolution from its requester(s) and move it back to the pool */
 static void dns_free_resolution(struct dns_resolution *resolution)
 {
@@ -1380,12 +1340,11 @@ static void dns_free_resolution(struct dns_resolution *resolution)
 	resolution->hostname_dn_len = 0;
 
 	list_for_each_entry_safe(req, reqback, &resolution->requesters, list) {
-		LIST_DEL_INIT(&req->list);
+		LIST_DEL(&req->list);
 		req->resolution = NULL;
 	}
 
-	dns_purge_resolution_answer_records(resolution);
-	LIST_DEL_INIT(&resolution->list);
+	LIST_DEL(&resolution->list);
 	pool_free(dns_resolution_pool, resolution);
 }
 
@@ -1470,7 +1429,7 @@ int dns_link_resolution(void *requester, int requester_type, int requester_locke
 			req = srvrq->dns_requester;
 
 		req->requester_cb       = snr_resolution_cb;
-		req->requester_error_cb = srvrq_resolution_error_cb;
+		req->requester_error_cb = snr_resolution_error_cb;
 	}
 	else if (stream) {
 		if (stream->dns_ctx.dns_requester == NULL) {
@@ -1501,9 +1460,8 @@ int dns_link_resolution(void *requester, int requester_type, int requester_locke
 
 /* Removes a requester from a DNS resoltion. It takes takes care of all the
  * consequences. It also cleans up some parameters from the requester.
- * if <safe> is set to 1, the corresponding resolution is not released.
  */
-void dns_unlink_resolution(struct dns_requester *requester, int safe)
+void dns_unlink_resolution(struct dns_requester *requester)
 {
 	struct dns_resolution *res;
 	struct dns_requester  *req;
@@ -1514,22 +1472,13 @@ void dns_unlink_resolution(struct dns_requester *requester, int safe)
 	res = requester->resolution;
 
 	/* Clean up the requester */
-	LIST_DEL_INIT(&requester->list);
+	LIST_DEL(&requester->list);
 	requester->resolution = NULL;
 
 	/* We need to find another requester linked on this resolution */
 	if (!LIST_ISEMPTY(&res->requesters))
 		req = LIST_NEXT(&res->requesters, struct dns_requester *, list);
 	else {
-		if (safe) {
-			/* Don't release it yet. */
-			dns_reset_resolution(res);
-			res->hostname_dn = NULL;
-			res->hostname_dn_len = 0;
-			dns_purge_resolution_answer_records(res);
-			return;
-		}
-
 		dns_free_resolution(res);
 		return;
 	}
@@ -1575,22 +1524,22 @@ static void dns_resolve_recv(struct dgram_conn *dgram)
 	unsigned short query_id;
 	struct eb32_node *eb;
 	struct dns_requester *req;
-	int keep_answer_items;
 
-	/* no need to go further if we can't retrieve the nameserver */
-	if ((ns = dgram->owner) == NULL)
-		return;
-
-	resolvers = ns->resolvers;
-	HA_SPIN_LOCK(DNS_LOCK, &resolvers->lock);
 	fd = dgram->t.sock.fd;
 
 	/* check if ready for reading */
-	if ((fd == -1) || !fd_recv_ready(fd)) {
-		HA_SPIN_UNLOCK(DNS_LOCK, &resolvers->lock);
+	if (!fd_recv_ready(fd))
+		return;
+
+	/* no need to go further if we can't retrieve the nameserver */
+	if ((ns = dgram->owner) == NULL) {
+		_HA_ATOMIC_AND(&fdtab[fd].ev, ~(FD_POLL_HUP|FD_POLL_ERR));
+		fd_stop_recv(fd);
 		return;
 	}
 
+	resolvers = ns->resolvers;
+	HA_SPIN_LOCK(DNS_LOCK, &resolvers->lock);
 
 	/* process all pending input messages */
 	while (fd_recv_ready(fd)) {
@@ -1720,7 +1669,7 @@ static void dns_resolve_recv(struct dgram_conn *dgram)
 		 * sent. We can check only the first query of the list. We send
 		 * one query at a time so we get one query in the response */
 		query = LIST_NEXT(&res->response.query_list, struct dns_query_item *, list);
-		if (query && dns_hostname_cmp(query->name, res->hostname_dn, res->hostname_dn_len) != 0) {
+		if (query && memcmp(query->name, res->hostname_dn, res->hostname_dn_len) != 0) {
 			dns_resp = DNS_RESP_WRONG_NAME;
 			ns->counters.other++;
 			goto report_res_error;
@@ -1733,13 +1682,10 @@ static void dns_resolve_recv(struct dgram_conn *dgram)
 		goto report_res_success;
 
 	report_res_error:
-		keep_answer_items = 0;
 		list_for_each_entry(req, &res->requesters, list)
-			keep_answer_items |= req->requester_error_cb(req, dns_resp);
-		if (!keep_answer_items)
-			dns_purge_resolution_answer_records(res);
+			req->requester_error_cb(req, dns_resp);
 		dns_reset_resolution(res);
-		LIST_DEL_INIT(&res->list);
+		LIST_DEL(&res->list);
 		LIST_ADDQ(&resolvers->resolutions.wait, &res->list);
 		continue;
 
@@ -1759,7 +1705,7 @@ static void dns_resolve_recv(struct dgram_conn *dgram)
 		}
 
 		dns_reset_resolution(res);
-		LIST_DEL_INIT(&res->list);
+		LIST_DEL(&res->list);
 		LIST_ADDQ(&resolvers->resolutions.wait, &res->list);
 		continue;
 	}
@@ -1775,22 +1721,21 @@ static void dns_resolve_send(struct dgram_conn *dgram)
 	struct dns_resolution *res;
 	int fd;
 
+	fd = dgram->t.sock.fd;
+
+	/* check if ready for sending */
+	if (!fd_send_ready(fd))
+		return;
+
+	/* we don't want/need to be waked up any more for sending */
+	fd_stop_send(fd);
+
 	/* no need to go further if we can't retrieve the nameserver */
 	if ((ns = dgram->owner) == NULL)
 		return;
 
 	resolvers = ns->resolvers;
 	HA_SPIN_LOCK(DNS_LOCK, &resolvers->lock);
-	fd = dgram->t.sock.fd;
-
-	/* check if ready for sending */
-	if ((fd == -1) || !fd_send_ready(fd)) {
-		HA_SPIN_UNLOCK(DNS_LOCK, &resolvers->lock);
-		return;
-	}
-
-	/* we don't want/need to be waked up any more for sending */
-	fd_stop_send(fd);
 
 	list_for_each_entry(res, &resolvers->resolutions.curr, list) {
 		int ret, len;
@@ -1798,10 +1743,6 @@ static void dns_resolve_send(struct dgram_conn *dgram)
 		if (res->nb_queries == resolvers->nb_nameservers)
 			continue;
 
-		/* if fd was detected broken by previous send */
-		if (fd == -1)
-			goto snd_error;
-
 		len = dns_build_query(res->query_id, res->query_type,
 		                      resolvers->accepted_payload_size,
 		                      res->hostname_dn, res->hostname_dn_len,
@@ -1810,24 +1751,8 @@ static void dns_resolve_send(struct dgram_conn *dgram)
 			goto snd_error;
 
 		ret = send(fd, trash.area, len, 0);
-		if (ret != len) {
-			if (ret == -1) {
-				if (errno == EAGAIN) {
-					/* retry once the socket is ready */
-					fd_cant_send(fd);
-					continue;
-				}
-
-				/* purge the fd and set sock.fd to -1
-				 * to create a new one during next
-				 * send_query attempt to the same
-				 * nameserver
-				 */
-				fd_delete(fd);
-				fd = dgram->t.sock.fd = -1;
-			}
+		if (ret != len)
 			goto snd_error;
-		}
 
 		ns->counters.sent++;
 		res->nb_queries++;
@@ -1854,11 +1779,6 @@ static struct task *dns_process_resolvers(struct task *t, void *context, unsigne
 
 	/* Handle all expired resolutions from the active list */
 	list_for_each_entry_safe(res, resback, &resolvers->resolutions.curr, list) {
-		if (LIST_ISEMPTY(&res->requesters)) {
-			dns_free_resolution(res);
-			continue;
-		}
-
 		/* When we find the first resolution in the future, then we can
 		 * stop here */
 		exp = tick_add(res->last_query, resolvers->timeout.retry);
@@ -1870,20 +1790,17 @@ static struct task *dns_process_resolvers(struct task *t, void *context, unsigne
 		 * the list */
 		if (!res->try) {
 			struct dns_requester *req;
-			int keep_answer_items = 0;
 
 			/* Notify the result to the requesters */
 			if (!res->nb_responses)
 				res->status = RSLV_STATUS_TIMEOUT;
 			list_for_each_entry(req, &res->requesters, list)
-				keep_answer_items |= req->requester_error_cb(req, res->status);
-			if (!keep_answer_items)
-				dns_purge_resolution_answer_records(res);
+				req->requester_error_cb(req, res->status);
 
 			/* Clean up resolution info and remove it from the
 			 * current list */
 			dns_reset_resolution(res);
-			LIST_DEL_INIT(&res->list);
+			LIST_DEL(&res->list);
 			LIST_ADDQ(&resolvers->resolutions.wait, &res->list);
 		}
 		else {
@@ -1910,18 +1827,13 @@ static struct task *dns_process_resolvers(struct task *t, void *context, unsigne
 
 	/* Handle all resolutions in the wait list */
 	list_for_each_entry_safe(res, resback, &resolvers->resolutions.wait, list) {
-		if (LIST_ISEMPTY(&res->requesters)) {
-			dns_free_resolution(res);
-			continue;
-		}
-
 		exp = tick_add(res->last_resolution, dns_resolution_timeout(res));
 		if (tick_isset(res->last_resolution) && !tick_is_expired(exp, now_ms))
 			continue;
 
 		if (dns_run_resolution(res) != 1) {
 			res->last_resolution = now_ms;
-			LIST_DEL_INIT(&res->list);
+			LIST_DEL(&res->list);
 			LIST_ADDQ(&resolvers->resolutions.wait, &res->list);
 		}
 	}
@@ -1953,13 +1865,13 @@ static void dns_deinit(void)
 			if (ns->dgram && ns->dgram->t.sock.fd != -1)
 				fd_delete(ns->dgram->t.sock.fd);
 			free(ns->dgram);
-			LIST_DEL_INIT(&ns->list);
+			LIST_DEL(&ns->list);
 			free(ns);
 		}
 
 		list_for_each_entry_safe(res, resback, &resolvers->resolutions.curr, list) {
 			list_for_each_entry_safe(req, reqback, &res->requesters, list) {
-				LIST_DEL_INIT(&req->list);
+				LIST_DEL(&req->list);
 				pool_free(dns_requester_pool, req);
 			}
 			dns_free_resolution(res);
@@ -1967,7 +1879,7 @@ static void dns_deinit(void)
 
 		list_for_each_entry_safe(res, resback, &resolvers->resolutions.wait, list) {
 			list_for_each_entry_safe(req, reqback, &res->requesters, list) {
-				LIST_DEL_INIT(&req->list);
+				LIST_DEL(&req->list);
 				pool_free(dns_requester_pool, req);
 			}
 			dns_free_resolution(res);
@@ -1976,21 +1888,21 @@ static void dns_deinit(void)
 		free(resolvers->id);
 		free((char *)resolvers->conf.file);
 		task_destroy(resolvers->t);
-		LIST_DEL_INIT(&resolvers->list);
+		LIST_DEL(&resolvers->list);
 		free(resolvers);
 	}
 
 	list_for_each_entry_safe(srvrq, srvrqback, &dns_srvrq_list, list) {
 		free(srvrq->name);
 		free(srvrq->hostname_dn);
-		LIST_DEL_INIT(&srvrq->list);
+		LIST_DEL(&srvrq->list);
 		free(srvrq);
 	}
 }
 
 /* Finalizes the DNS configuration by allocating required resources and checking
  * live parameters.
- * Returns 0 on success, 1 on error.
+ * Returns 0 on success, ERR_* flags otherwise.
  */
 static int dns_finalize_config(void)
 {
@@ -2058,13 +1970,6 @@ static int dns_finalize_config(void)
 	for (px = proxies_list; px; px = px->next) {
 		struct server *srv;
 
-		if (px->state == PR_STSTOPPED) {
-			/* must not run and will not work anyway since
-			 * nothing in the proxy is initialized.
-			 */
-			continue;
-		}
-
 		for (srv = px->srv; srv; srv = srv->next) {
 			struct dns_resolvers *resolvers;
 
@@ -2100,10 +2005,10 @@ static int dns_finalize_config(void)
 	if (err_code & (ERR_ALERT|ERR_ABORT))
 		goto err;
 
-	return 0;
+	return err_code;
   err:
 	dns_deinit();
-	return 1;
+	return err_code;
 
 }
 
@@ -2119,12 +2024,8 @@ static int cli_parse_stat_resolvers(char **args, char *payload, struct appctx *a
 				break;
 			}
 		}
-		if (appctx->ctx.cli.p0 == NULL) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Can't find that resolvers section\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (appctx->ctx.cli.p0 == NULL)
+			return cli_err(appctx, "Can't find that resolvers section\n");
 	}
 	return 0;
 }
@@ -2211,17 +2112,18 @@ INITCALL1(STG_REGISTER, cli_register_kw, &cli_kws);
  * Returns -1 in case of any allocation failure, 0 if not.
  * On error, a global failure counter is also incremented.
  */
-static int action_prepare_for_resolution(struct stream *stream, const char *hostname, int hostname_len)
+static int action_prepare_for_resolution(struct stream *stream, const char *hostname)
 {
 	char *hostname_dn;
-	int   hostname_dn_len;
+	int   hostname_len, hostname_dn_len;
 	struct buffer *tmp = get_trash_chunk();
 
 	if (!hostname)
 		return 0;
 
+	hostname_len    = strlen(hostname);
 	hostname_dn     = tmp->area;
-	hostname_dn_len = dns_str_to_dn_label(hostname, hostname_len,
+	hostname_dn_len = dns_str_to_dn_label(hostname, hostname_len + 1,
 				      hostname_dn, tmp->size);
 	if (hostname_dn_len == -1)
 		goto err;
@@ -2249,32 +2151,21 @@ enum act_return dns_action_do_resolve(struct act_rule *rule, struct proxy *px,
 {
 	struct dns_resolution *resolution;
 	struct sample *smp;
+	char *fqdn;
 	struct dns_requester *req;
 	struct dns_resolvers  *resolvers;
 	struct dns_resolution *res;
-	int exp, locked = 0;
-	enum act_return ret = ACT_RET_CONT;
-
-	resolvers = rule->arg.dns.resolvers;
+	int exp;
 
 	/* we have a response to our DNS resolution */
  use_cache:
 	if (s->dns_ctx.dns_requester && s->dns_ctx.dns_requester->resolution != NULL) {
 		resolution = s->dns_ctx.dns_requester->resolution;
-		if (!locked) {
-			HA_SPIN_LOCK(DNS_LOCK, &resolvers->lock);
-			locked = 1;
+		if (resolution->step == RSLV_STEP_RUNNING) {
+			return ACT_RET_YIELD;
 		}
-
-		if (resolution->step == RSLV_STEP_RUNNING)
-			goto yield;
 		if (resolution->step == RSLV_STEP_NONE) {
-			/* We update the variable only if we have a valid
-			 * response. If the response was not received yet, we
-			 * must yield.
-			 */
-			if (resolution->status == RSLV_STATUS_NONE)
-				goto yield;
+			/* We update the variable only if we have a valid response. */
 			if (resolution->status == RSLV_STATUS_VALID) {
 				struct sample smp;
 				short ip_sin_family = 0;
@@ -2306,58 +2197,45 @@ enum act_return dns_action_do_resolve(struct act_rule *rule, struct proxy *px,
 			}
 		}
 
-		goto release_requester;
+		free(s->dns_ctx.hostname_dn); s->dns_ctx.hostname_dn = NULL;
+		s->dns_ctx.hostname_dn_len = 0;
+		dns_unlink_resolution(s->dns_ctx.dns_requester);
+
+		pool_free(dns_requester_pool, s->dns_ctx.dns_requester);
+		s->dns_ctx.dns_requester = NULL;
+
+		return ACT_RET_CONT;
 	}
 
 	/* need to configure and start a new DNS resolution */
 	smp = sample_fetch_as_type(px, sess, s, SMP_OPT_DIR_REQ|SMP_OPT_FINAL, rule->arg.dns.expr, SMP_T_STR);
 	if (smp == NULL)
-		goto end;
+		return ACT_RET_CONT;
 
-	if (action_prepare_for_resolution(s, smp->data.u.str.area, smp->data.u.str.data) == -1)
-		goto end; /* on error, ignore the action */
+	fqdn = smp->data.u.str.area;
+	if (action_prepare_for_resolution(s, fqdn) == -1)
+		return ACT_RET_ERR;
 
 	s->dns_ctx.parent = rule;
-
-	HA_SPIN_LOCK(DNS_LOCK, &resolvers->lock);
-	locked = 1;
-
 	dns_link_resolution(s, OBJ_TYPE_STREAM, 0);
 
 	/* Check if there is a fresh enough response in the cache of our associated resolution */
 	req = s->dns_ctx.dns_requester;
-	if (!req || !req->resolution)
-		goto release_requester; /* on error, ignore the action */
-	res = req->resolution;
+	if (!req || !req->resolution) {
+		dns_trigger_resolution(s->dns_ctx.dns_requester);
+		return ACT_RET_YIELD;
+	}
+	res       = req->resolution;
+	resolvers = res->resolvers;
 
 	exp = tick_add(res->last_resolution, resolvers->hold.valid);
 	if (resolvers->t && res->status == RSLV_STATUS_VALID && tick_isset(res->last_resolution)
-	    && !tick_is_expired(exp, now_ms)) {
+		       && !tick_is_expired(exp, now_ms)) {
 		goto use_cache;
 	}
 
 	dns_trigger_resolution(s->dns_ctx.dns_requester);
-
-  yield:
-	if (flags & ACT_FLAG_FINAL)
-		goto release_requester;
-	ret = ACT_RET_YIELD;
-
-  end:
-	if (locked)
-		HA_SPIN_UNLOCK(DNS_LOCK, &resolvers->lock);
-	return ret;
-
-  release_requester:
-	free(s->dns_ctx.hostname_dn);
-	s->dns_ctx.hostname_dn = NULL;
-	s->dns_ctx.hostname_dn_len = 0;
-	if (s->dns_ctx.dns_requester) {
-		dns_unlink_resolution(s->dns_ctx.dns_requester, 0);
-		pool_free(dns_requester_pool, s->dns_ctx.dns_requester);
-		s->dns_ctx.dns_requester = NULL;
-	}
-	goto end;
+	return ACT_RET_YIELD;
 }
 
 
