@@ -48,29 +48,39 @@ static void _update_fd(int fd, int *max_add_fd)
 	 * don't check the tid_bit. First thread to see the update
 	 * takes it for every other one.
 	 */
-	if (!(en & FD_EV_POLLED_RW)) {
-		if (!polled_mask[fd]) {
+	if (!(en & FD_EV_ACTIVE_RW)) {
+		if (!(polled_mask[fd].poll_recv | polled_mask[fd].poll_send)) {
 			/* fd was not watched, it's still not */
 			return;
 		}
 		/* fd totally removed from poll list */
 		hap_fd_clr(fd, fd_evts[DIR_RD]);
 		hap_fd_clr(fd, fd_evts[DIR_WR]);
-		_HA_ATOMIC_AND(&polled_mask[fd], 0);
+		_HA_ATOMIC_AND(&polled_mask[fd].poll_recv, 0);
+		_HA_ATOMIC_AND(&polled_mask[fd].poll_send, 0);
 	}
 	else {
 		/* OK fd has to be monitored, it was either added or changed */
-		if (!(en & FD_EV_POLLED_R))
+		if (!(en & FD_EV_ACTIVE_R)) {
 			hap_fd_clr(fd, fd_evts[DIR_RD]);
-		else
+			if (polled_mask[fd].poll_recv & tid_bit)
+				_HA_ATOMIC_AND(&polled_mask[fd].poll_recv, ~tid_bit);
+		} else {
 			hap_fd_set(fd, fd_evts[DIR_RD]);
+			if (!(polled_mask[fd].poll_recv & tid_bit))
+				_HA_ATOMIC_OR(&polled_mask[fd].poll_recv, tid_bit);
+		}
 
-		if (!(en & FD_EV_POLLED_W))
+		if (!(en & FD_EV_ACTIVE_W)) {
 			hap_fd_clr(fd, fd_evts[DIR_WR]);
-		else
+			if (polled_mask[fd].poll_send & tid_bit)
+				_HA_ATOMIC_AND(&polled_mask[fd].poll_send, ~tid_bit);
+		} else {
 			hap_fd_set(fd, fd_evts[DIR_WR]);
+			if (!(polled_mask[fd].poll_send & tid_bit))
+				_HA_ATOMIC_OR(&polled_mask[fd].poll_send, tid_bit);
+		}
 
-		_HA_ATOMIC_OR(&polled_mask[fd], tid_bit);
 		if (fd > *max_add_fd)
 			*max_add_fd = fd;
 	}
@@ -176,6 +186,8 @@ REGPRM3 static void _do_poll(struct poller *p, int exp, int wake)
 	tv_leaving_poll(delta_ms, status);
 
 	thread_harmless_end();
+	if (sleeping_thread_mask & tid_bit)
+		_HA_ATOMIC_AND(&sleeping_thread_mask, ~tid_bit);
 
 	if (status <= 0)
 		return;
@@ -198,10 +210,10 @@ REGPRM3 static void _do_poll(struct poller *p, int exp, int wake)
 			}
 
 			if (FD_ISSET(fd, tmp_evts[DIR_RD]))
-				n |= FD_POLL_IN;
+				n |= FD_EV_READY_R;
 
 			if (FD_ISSET(fd, tmp_evts[DIR_WR]))
-				n |= FD_POLL_OUT;
+				n |= FD_EV_READY_W;
 
 			fd_update_events(fd, n);
 		}
@@ -237,12 +249,13 @@ static void deinit_select_per_thread()
  */
 REGPRM1 static int _do_init(struct poller *p)
 {
+	__label__ fail_swevt, fail_srevt, fail_revt;
 	int fd_set_bytes;
 
 	p->private = NULL;
 
 	if (global.maxsock > FD_SETSIZE)
-		goto fail_srevt;
+		goto fail_revt;
 
 	fd_set_bytes = sizeof(fd_set) * (global.maxsock + FD_SETSIZE - 1) / FD_SETSIZE;
 
@@ -259,6 +272,9 @@ REGPRM1 static int _do_init(struct poller *p)
  fail_swevt:
 	free(fd_evts[DIR_RD]);
  fail_srevt:
+	free(tmp_evts[DIR_WR]);
+	free(tmp_evts[DIR_RD]);
+ fail_revt:
 	p->pref = 0;
 	return 0;
 }
