@@ -30,6 +30,7 @@
 #include <common/h2.h>
 #include <common/http-hdr.h>
 #include <common/ist.h>
+#include <types/global.h>
 
 struct h2_frame_definition h2_frame_definition[H2_FT_ENTRIES] =	{
 	 [H2_FT_DATA         ] = { .dir = 3, .min_id = 1, .max_id = H2_MAX_STREAM_ID, .min_len = 0, .max_len = H2_MAX_FRAME_LEN, },
@@ -61,419 +62,6 @@ static int has_forbidden_char(const struct ist ist, const char *start)
 	return 0;
 }
 
-/* Looks into <ist> for forbidden characters for :path values (0x00..0x1F,
- * 0x20, 0x23), starting at pointer <start> which must be within <ist>.
- * Returns non-zero if such a character is found, 0 otherwise. When run on
- * unlikely header match, it's recommended to first check for the presence
- * of control chars using ist_find_ctl().
- */
-static inline int http_path_has_forbidden_char(const struct ist ist, const char *start)
-{
-	do {
-		if ((uint8_t)*start <= 0x23) {
-			if ((uint8_t)*start < 0x20)
-				return 1;
-			if ((1U << ((uint8_t)*start & 0x1F)) & ((1<<3) | (1<<0)))
-				return 1;
-		}
-		start++;
-	} while (start < istend(ist));
-	return 0;
-}
-
-/* Prepare the request line into <*ptr> (stopping at <end>) from pseudo headers
- * stored in <phdr[]>. <fields> indicates what was found so far. This should be
- * called once at the detection of the first general header field or at the end
- * of the request if no general header field was found yet. Returns 0 on success
- * or a negative error code on failure. Upon success, <msgf> is updated with a
- * few H2_MSGF_* flags indicating what was found while parsing.
- */
-static int h2_prepare_h1_reqline(uint32_t fields, struct ist *phdr, char **ptr, char *end, unsigned int *msgf)
-{
-	char *out = *ptr;
-	int uri_idx = H2_PHDR_IDX_PATH;
-
-	if ((fields & H2_PHDR_FND_METH) && isteq(phdr[H2_PHDR_IDX_METH], ist("CONNECT"))) {
-		/* RFC 7540 #8.2.6 regarding CONNECT: ":scheme" and ":path"
-		 * MUST be omitted ; ":authority" contains the host and port
-		 * to connect to.
-		 */
-		if (fields & H2_PHDR_FND_SCHM) {
-			/* scheme not allowed */
-			goto fail;
-		}
-		else if (fields & H2_PHDR_FND_PATH) {
-			/* path not allowed */
-			goto fail;
-		}
-		else if (!(fields & H2_PHDR_FND_AUTH)) {
-			/* missing authority */
-			goto fail;
-		}
-		// otherwise OK ; let's use the authority instead of the URI
-		uri_idx = H2_PHDR_IDX_AUTH;
-		*msgf |= H2_MSGF_BODY_TUNNEL;
-	}
-	else if ((fields & (H2_PHDR_FND_METH|H2_PHDR_FND_SCHM|H2_PHDR_FND_PATH)) !=
-	         (H2_PHDR_FND_METH|H2_PHDR_FND_SCHM|H2_PHDR_FND_PATH)) {
-		/* RFC 7540 #8.1.2.3 : all requests MUST include exactly one
-		 * valid value for the ":method", ":scheme" and ":path" phdr
-		 * unless it is a CONNECT request.
-		 */
-		if (!(fields & H2_PHDR_FND_METH)) {
-			/* missing method */
-			goto fail;
-		}
-		else if (!(fields & H2_PHDR_FND_SCHM)) {
-			/* missing scheme */
-			goto fail;
-		}
-		else {
-			/* missing path */
-			goto fail;
-		}
-	}
-
-	/* 7540#8.1.2.3: :path must not be empty */
-	if (!phdr[uri_idx].len)
-		goto fail;
-
-	if (out + phdr[H2_PHDR_IDX_METH].len + 1 + phdr[uri_idx].len + 11 > end) {
-		/* too large */
-		goto fail;
-	}
-
-	memcpy(out, phdr[H2_PHDR_IDX_METH].ptr, phdr[H2_PHDR_IDX_METH].len);
-	out += phdr[H2_PHDR_IDX_METH].len;
-	*(out++) = ' ';
-
-	memcpy(out, phdr[uri_idx].ptr, phdr[uri_idx].len);
-	out += phdr[uri_idx].len;
-	memcpy(out, " HTTP/1.1\r\n", 11);
-	out += 11;
-
-	*ptr = out;
-	return 0;
- fail:
-	return -1;
-}
-
-/* Takes an H2 request present in the headers list <list> terminated by a name
- * being <NULL,0> and emits the equivalent HTTP/1.1 request according to the
- * rules documented in RFC7540 #8.1.2. The output contents are emitted in <out>
- * for a max of <osize> bytes, and the amount of bytes emitted is returned. In
- * case of error, a negative error code is returned.
- *
- * Upon success, <msgf> is filled with a few H2_MSGF_* flags indicating what
- * was found while parsing. The caller must set it to zero in or H2_MSGF_BODY
- * if a body is detected (!ES).
- *
- * The headers list <list> must be composed of :
- *   - n.name != NULL, n.len  > 0 : literal header name
- *   - n.name == NULL, n.len  > 0 : indexed pseudo header name number <n.len>
- *                                  among H2_PHDR_IDX_*
- *   - n.name ignored, n.len == 0 : end of list
- *   - in all cases except the end of list, v.name and v.len must designate a
- *     valid value.
- *
- * The Cookie header will be reassembled at the end, and for this, the <list>
- * will be used to create a linked list, so its contents may be destroyed.
- *
- * When <relaxed> is non-nul, some non-dangerous checks will be ignored. This
- * is in order to satisfy "option accept-invalid-http-request" for
- * interoperability purposes.
- */
-int h2_make_h1_request(struct http_hdr *list, char *out, int osize, unsigned int *msgf, unsigned long long *body_len, int relaxed)
-{
-	struct ist phdr_val[H2_PHDR_NUM_ENTRIES];
-	char *out_end = out + osize;
-	const char *ctl;
-	uint32_t fields; /* bit mask of H2_PHDR_FND_* */
-	uint32_t idx;
-	int ck, lck; /* cookie index and last cookie index */
-	int phdr;
-	int ret;
-	int i;
-
-	lck = ck = -1; // no cookie for now
-	fields = 0;
-	for (idx = 0; list[idx].n.len != 0; idx++) {
-		if (!list[idx].n.ptr) {
-			/* this is an indexed pseudo-header */
-			phdr = list[idx].n.len;
-		}
-		else {
-			/* this can be any type of header */
-			/* RFC7540#8.1.2: upper case not allowed in header field names.
-			 * #10.3: header names must be valid (i.e. match a token).
-			 * For pseudo-headers we check from 2nd char and for other ones
-			 * from the first char, because HTTP_IS_TOKEN() also excludes
-			 * the colon.
-			 */
-			phdr = h2_str_to_phdr(list[idx].n);
-
-			for (i = !!phdr; i < list[idx].n.len; i++)
-				if ((uint8_t)(list[idx].n.ptr[i] - 'A') < 'Z' - 'A' || !HTTP_IS_TOKEN(list[idx].n.ptr[i]))
-					goto fail;
-		}
-
-		/* RFC7540#10.3: intermediaries forwarding to HTTP/1 must take care of
-		 * rejecting NUL, CR and LF characters.
-		 */
-		if (phdr == H2_PHDR_IDX_PATH && !relaxed) {
-			ctl = ist_find_range(list[idx].v, 0, '#');
-			if (unlikely(ctl) && http_path_has_forbidden_char(list[idx].v, ctl))
-				goto fail;
-		} else {
-			ctl = ist_find_ctl(list[idx].v);
-			if (unlikely(ctl) && has_forbidden_char(list[idx].v, ctl))
-				goto fail;
-		}
-
-		if (phdr > 0 && phdr < H2_PHDR_NUM_ENTRIES) {
-			/* insert a pseudo header by its index (in phdr) and value (in value) */
-			if (fields & ((1 << phdr) | H2_PHDR_FND_NONE)) {
-				if (fields & H2_PHDR_FND_NONE) {
-					/* pseudo header field after regular headers */
-					goto fail;
-				}
-				else {
-					/* repeated pseudo header field */
-					goto fail;
-				}
-			}
-			fields |= 1 << phdr;
-			phdr_val[phdr] = list[idx].v;
-			continue;
-		}
-		else if (phdr != 0) {
-			/* invalid pseudo header -- should never happen here */
-			goto fail;
-		}
-
-		/* regular header field in (name,value) */
-		if (!(fields & H2_PHDR_FND_NONE)) {
-			/* no more pseudo-headers, time to build the request line */
-			ret = h2_prepare_h1_reqline(fields, phdr_val, &out, out_end, msgf);
-			if (ret != 0)
-				goto leave;
-			fields |= H2_PHDR_FND_NONE;
-		}
-
-		if (isteq(list[idx].n, ist("host")))
-			fields |= H2_PHDR_FND_HOST;
-
-		if (isteq(list[idx].n, ist("content-length"))) {
-			ret = h2_parse_cont_len_header(msgf, &list[idx].v, body_len);
-			if (ret < 0)
-				goto fail;
-
-			if (ret == 0)
-				continue; // skip this duplicate
-		}
-
-		/* these ones are forbidden in requests (RFC7540#8.1.2.2) */
-		if (isteq(list[idx].n, ist("connection")) ||
-		    isteq(list[idx].n, ist("proxy-connection")) ||
-		    isteq(list[idx].n, ist("keep-alive")) ||
-		    isteq(list[idx].n, ist("upgrade")) ||
-		    isteq(list[idx].n, ist("transfer-encoding")))
-			goto fail;
-
-		if (isteq(list[idx].n, ist("te")) && !isteq(list[idx].v, ist("trailers")))
-			goto fail;
-
-		/* cookie requires special processing at the end */
-		if (isteq(list[idx].n, ist("cookie"))) {
-			list[idx].n.len = -1;
-
-			if (ck < 0)
-				ck = idx;
-			else
-				list[lck].n.len = idx;
-
-			lck = idx;
-			continue;
-		}
-
-		if (out + list[idx].n.len + 2 + list[idx].v.len + 2 > out_end) {
-			/* too large */
-			goto fail;
-		}
-
-		/* copy "name: value" */
-		memcpy(out, list[idx].n.ptr, list[idx].n.len);
-		out += list[idx].n.len;
-		*(out++) = ':';
-		*(out++) = ' ';
-
-		memcpy(out, list[idx].v.ptr, list[idx].v.len);
-		out += list[idx].v.len;
-		*(out++) = '\r';
-		*(out++) = '\n';
-	}
-
-	/* RFC7540#8.1.2.1 mandates to reject response pseudo-headers (:status) */
-	if (fields & H2_PHDR_FND_STAT)
-		goto fail;
-
-	/* Let's dump the request now if not yet emitted. */
-	if (!(fields & H2_PHDR_FND_NONE)) {
-		ret = h2_prepare_h1_reqline(fields, phdr_val, &out, out_end, msgf);
-		if (ret != 0)
-			goto leave;
-	}
-
-	/* complete with missing Host if needed */
-	if ((fields & (H2_PHDR_FND_HOST|H2_PHDR_FND_AUTH)) == H2_PHDR_FND_AUTH) {
-		/* missing Host field, use :authority instead */
-		if (out + 6 + phdr_val[H2_PHDR_IDX_AUTH].len + 2 > out_end) {
-			/* too large */
-			goto fail;
-		}
-
-		memcpy(out, "host: ", 6);
-		memcpy(out + 6, phdr_val[H2_PHDR_IDX_AUTH].ptr, phdr_val[H2_PHDR_IDX_AUTH].len);
-		out += 6 + phdr_val[H2_PHDR_IDX_AUTH].len;
-		*(out++) = '\r';
-		*(out++) = '\n';
-	}
-
-	if ((*msgf & (H2_MSGF_BODY|H2_MSGF_BODY_TUNNEL|H2_MSGF_BODY_CL)) == H2_MSGF_BODY) {
-		/* add chunked encoding */
-		if (out + 28 > out_end)
-			goto fail;
-		memcpy(out, "transfer-encoding: chunked\r\n", 28);
-		out += 28;
-	}
-
-	/* now we may have to build a cookie list. We'll dump the values of all
-	 * visited headers.
-	 */
-	if (ck >= 0) {
-		if (out + 8 > out_end) {
-			/* too large */
-			goto fail;
-		}
-		memcpy(out, "cookie: ", 8);
-		out += 8;
-
-		do {
-			if (out + list[ck].v.len + 2 > out_end) {
-				/* too large */
-				goto fail;
-			}
-			memcpy(out, list[ck].v.ptr, list[ck].v.len);
-			out += list[ck].v.len;
-			ck = list[ck].n.len;
-
-			if (ck >= 0) {
-				*(out++) = ';';
-				*(out++) = ' ';
-			}
-		} while (ck >= 0);
-
-		if (out + 2 > out_end) {
-			/* too large */
-			goto fail;
-		}
-		*(out++) = '\r';
-		*(out++) = '\n';
-	}
-
-	/* And finish */
-	if (out + 2 > out_end) {
-		/* too large */
-		goto fail;
-	}
-
-	*(out++) = '\r';
-	*(out++) = '\n';
-	ret = out + osize - out_end;
- leave:
-	return ret;
-
- fail:
-	return -1;
-}
-
-/* Takes an H2 headers list <list> terminated by a name being <NULL,0> and
- * emits the equivalent HTTP/1.1 trailers block not including the empty line.
- * The output contents are emitted in <out> for a max of <osize> bytes, and the
- * amount of bytes emitted is returned. In case of error, a negative error code
- * is returned. The caller must have verified that the message in the buffer is
- * compatible with receipt of trailers.
- *
- * The headers list <list> must be composed of :
- *   - n.name != NULL, n.len  > 0 : literal header name
- *   - n.name == NULL, n.len  > 0 : indexed pseudo header name number <n.len>
- *                                  among H2_PHDR_IDX_* (illegal here)
- *   - n.name ignored, n.len == 0 : end of list
- *   - in all cases except the end of list, v.name and v.len must designate a
- *     valid value.
- */
-int h2_make_h1_trailers(struct http_hdr *list, char *out, int osize)
-{
-	char *out_end = out + osize;
-	const char *ctl;
-	uint32_t idx;
-	int i;
-
-	for (idx = 0; list[idx].n.len != 0; idx++) {
-		if (!list[idx].n.ptr) {
-			/* This is an indexed pseudo-header (RFC7540#8.1.2.1) */
-			goto fail;
-		}
-
-		/* RFC7540#8.1.2: upper case not allowed in header field names.
-		 * #10.3: header names must be valid (i.e. match a token). This
-		 * also catches pseudo-headers which are forbidden in trailers.
-		 */
-		for (i = 0; i < list[idx].n.len; i++)
-			if ((uint8_t)(list[idx].n.ptr[i] - 'A') < 'Z' - 'A' || !HTTP_IS_TOKEN(list[idx].n.ptr[i]))
-				goto fail;
-
-		/* these ones are forbidden in trailers (RFC7540#8.1.2.2) */
-		if (isteq(list[idx].n, ist("host")) ||
-		    isteq(list[idx].n, ist("content-length")) ||
-		    isteq(list[idx].n, ist("connection")) ||
-		    isteq(list[idx].n, ist("proxy-connection")) ||
-		    isteq(list[idx].n, ist("keep-alive")) ||
-		    isteq(list[idx].n, ist("upgrade")) ||
-		    isteq(list[idx].n, ist("te")) ||
-		    isteq(list[idx].n, ist("transfer-encoding")))
-			goto fail;
-
-		if (out + list[idx].n.len + 2 + list[idx].v.len + 2 > out_end) {
-			/* too large */
-			goto fail;
-		}
-
-		/* RFC7540#10.3: intermediaries forwarding to HTTP/1 must take care of
-		 * rejecting NUL, CR and LF characters.
-		 */
-		ctl = ist_find_ctl(list[idx].v);
-		if (unlikely(ctl) && has_forbidden_char(list[idx].v, ctl))
-			goto fail;
-
-		/* copy "name: value" */
-		memcpy(out, list[idx].n.ptr, list[idx].n.len);
-		out += list[idx].n.len;
-		*(out++) = ':';
-		*(out++) = ' ';
-
-		memcpy(out, list[idx].v.ptr, list[idx].v.len);
-		out += list[idx].v.len;
-		*(out++) = '\r';
-		*(out++) = '\n';
-	}
-
-	return out + osize - out_end;
-
- fail:
-	return -1;
-}
-
 /* Parse the Content-Length header field of an HTTP/2 request. The function
  * checks all possible occurrences of a comma-delimited value, and verifies
  * if any of them doesn't match a previous value. It returns <0 if a value
@@ -488,20 +76,13 @@ int h2_parse_cont_len_header(unsigned int *msgf, struct ist *value, unsigned lon
 	int not_first = !!(*msgf & H2_MSGF_BODY_CL);
 	struct ist word;
 
-	word.ptr = value->ptr;
+	word.ptr = value->ptr - 1; // -1 for next loop's pre-increment
 	e = value->ptr + value->len;
 
-	while (1) {
-		if (word.ptr >= e) {
-			/* empty header or empty value */
-			goto fail;
-		}
-
-		/* skip leading delimiter and blanks */
-		if (unlikely(HTTP_IS_LWS(*word.ptr))) {
-			word.ptr++;
+	while (++word.ptr < e) {
+		/* skip leading delimitor and blanks */
+		if (unlikely(HTTP_IS_LWS(*word.ptr)))
 			continue;
-		}
 
 		/* digits only now */
 		for (cl = 0, n = word.ptr; n < e; n++) {
@@ -512,14 +93,6 @@ int h2_parse_cont_len_header(unsigned int *msgf, struct ist *value, unsigned lon
 					goto fail;
 				break;
 			}
-
-			if (unlikely(!cl && n > word.ptr)) {
-				/* There was a leading zero before this digit,
-				 * let's trim it.
-				 */
-				word.ptr = n;
-			}
-
 			if (unlikely(cl > ULLONG_MAX / 10ULL))
 				goto fail; /* multiply overflow */
 			cl = cl * 10ULL;
@@ -548,13 +121,6 @@ int h2_parse_cont_len_header(unsigned int *msgf, struct ist *value, unsigned lon
 		*msgf |= H2_MSGF_BODY_CL;
 		*body_len = cl;
 		*value = word;
-
-		/* Now either n==e and we're done, or n points to the comma,
-		 * and we skip it and continue.
-		 */
-		if (n++ == e)
-			break;
-
 		word.ptr = n;
 	}
 	/* here we've reached the end with a single value or a series of
@@ -574,10 +140,37 @@ int h2_parse_cont_len_header(unsigned int *msgf, struct ist *value, unsigned lon
  * no general header field was found yet. Returns the created start line on
  * success, or NULL on failure. Upon success, <msgf> is updated with a few
  * H2_MSGF_* flags indicating what was found while parsing.
+ *
+ * The rules below deserve a bit of explanation. There tends to be some
+ * confusion regarding H2's authority vs the Host header. They are different
+ * though may sometimes be exchanged. In H2, the request line is broken into :
+ *   - :method
+ *   - :scheme
+ *   - :authority
+ *   - :path
+ *
+ * An equivalent HTTP/1.x absolute-form request would then look like :
+ *   <:method> <:scheme>://<:authority><:path> HTTP/x.y
+ *
+ * Except for CONNECT which doesn't have scheme nor path and looks like :
+ *   <:method> <:authority> HTTP/x.y
+ *
+ * It's worth noting that H2 still supports an encoding to map H1 origin-form
+ * and asterisk-form requests. These ones do not specify the authority. However
+ * in H2 they must still specify the scheme, which is not present in H1. Also,
+ * when encoding an absolute-form H1 request without a path, the path
+ * automatically becomes "/" except for the OPTIONS method where it
+ * becomes "*".
+ *
+ * As such it is explicitly permitted for an H2 client to send a request
+ * featuring a Host header and no :authority, though it's not the recommended
+ * way to use H2 for a client. It is however the only permitted way to encode
+ * an origin-form H1 request over H2. Thus we need to respect such differences
+ * as much as possible when re-encoding the H2 request into HTX.
  */
 static struct htx_sl *h2_prepare_htx_reqline(uint32_t fields, struct ist *phdr, struct htx *htx, unsigned int *msgf)
 {
-	int uri_idx = H2_PHDR_IDX_PATH;
+	struct ist uri;
 	unsigned int flags = HTX_SL_F_NONE;
 	struct htx_sl *sl;
 	size_t i;
@@ -599,8 +192,6 @@ static struct htx_sl *h2_prepare_htx_reqline(uint32_t fields, struct ist *phdr,
 			/* missing authority */
 			goto fail;
 		}
-		// otherwise OK ; let's use the authority instead of the URI
-		uri_idx = H2_PHDR_IDX_AUTH;
 		*msgf |= H2_MSGF_BODY_TUNNEL;
 	}
 	else if ((fields & (H2_PHDR_FND_METH|H2_PHDR_FND_SCHM|H2_PHDR_FND_PATH)) !=
@@ -622,22 +213,68 @@ static struct htx_sl *h2_prepare_htx_reqline(uint32_t fields, struct ist *phdr,
 			goto fail;
 		}
 	}
-
-	/* The method is a non-empty token (RFC7231#4.1) */
-	if (!phdr[H2_PHDR_IDX_METH].len)
-		goto fail;
-	for (i = 0; i < phdr[H2_PHDR_IDX_METH].len; i++) {
-		if (!HTTP_IS_TOKEN(phdr[H2_PHDR_IDX_METH].ptr[i]))
-			htx->flags |= HTX_FL_PARSING_ERROR;
+	else { /* regular methods */
+		/* RFC3986#6.2.2.1: scheme is case-insensitive. We need to
+		 * classify the scheme as "present/http", "present/https",
+		 * "present/other", "absent" so as to decide whether or not
+		 * we're facing a normalized URI that will have to be encoded
+		 * in origin or absolute form. Indeed, 7540#8.1.2.3 says that
+		 * clients should use the absolute form, thus we cannot infer
+		 * whether or not the client wanted to use a proxy here.
+		 */
+		flags |= HTX_SL_F_HAS_SCHM;
+		if (isteqi(phdr[H2_PHDR_IDX_SCHM], ist("http")))
+			flags |= HTX_SL_F_SCHM_HTTP;
+		else if (isteqi(phdr[H2_PHDR_IDX_SCHM], ist("https")))
+			flags |= HTX_SL_F_SCHM_HTTPS;
 	}
 
-	/* 7540#8.1.2.3: :path must not be empty */
-	if (!phdr[uri_idx].len)
+	if (!(flags & HTX_SL_F_HAS_SCHM)) {
+		/* no scheme, use authority only (CONNECT) */
+		uri = phdr[H2_PHDR_IDX_AUTH];
+		flags |= HTX_SL_F_HAS_AUTHORITY;
+	}
+	else if (fields & H2_PHDR_FND_AUTH) {
+		/* authority is present, let's use the absolute form. We simply
+		 * use the trash to concatenate them since all of them MUST fit
+		 * in a bufsize since it's where they come from.
+		 */
+		uri = ist2bin(trash.area, phdr[H2_PHDR_IDX_SCHM]);
+		istcat(&uri, ist("://"), trash.size);
+		istcat(&uri, phdr[H2_PHDR_IDX_AUTH], trash.size);
+		if (!isteq(phdr[H2_PHDR_IDX_PATH], ist("*")))
+			istcat(&uri, phdr[H2_PHDR_IDX_PATH], trash.size);
+		flags |= HTX_SL_F_HAS_AUTHORITY;
+
+		if (flags & (HTX_SL_F_SCHM_HTTP|HTX_SL_F_SCHM_HTTPS)) {
+			/* we don't know if it was originally an absolute or a
+			 * relative request because newer versions of HTTP use
+			 * the absolute URI format by default, which we call
+			 * the normalized URI format internally. This is the
+			 * strongly recommended way of sending a request for
+			 * a regular client, so we cannot distinguish this
+			 * from a request intended for a proxy. For other
+			 * schemes however there is no doubt.
+			 */
+			flags |= HTX_SL_F_NORMALIZED_URI;
+		}
+	}
+	else {
+		/* usual schemes with or without authority, use origin form */
+		uri = phdr[H2_PHDR_IDX_PATH];
+		if (fields & H2_PHDR_FND_AUTH)
+			flags |= HTX_SL_F_HAS_AUTHORITY;
+	}
+
+	/* make sure the final URI isn't empty. Note that 7540#8.1.2.3 states
+	 * that :path must not be empty.
+	 */
+	if (!uri.len)
 		goto fail;
 
-	/* make sure :path doesn't contain LWS nor CTL characters */
-	for (i = 0; i < phdr[uri_idx].len; i++) {
-		unsigned char c = phdr[uri_idx].ptr[i];
+	/* The final URI must not contain LWS nor CTL characters */
+	for (i = 0; i < uri.len; i++) {
+		unsigned char c = uri.ptr[i];
 		if (HTTP_IS_LWS(c) || HTTP_IS_CTL(c))
 			htx->flags |= HTX_FL_PARSING_ERROR;
 	}
@@ -646,13 +283,11 @@ static struct htx_sl *h2_prepare_htx_reqline(uint32_t fields, struct ist *phdr,
 	flags |= HTX_SL_F_VER_11;    // V2 in fact
 	flags |= HTX_SL_F_XFER_LEN;  // xfer len always known with H2
 
-	sl = htx_add_stline(htx, HTX_BLK_REQ_SL, flags, phdr[H2_PHDR_IDX_METH], phdr[uri_idx], ist("HTTP/2.0"));
+	sl = htx_add_stline(htx, HTX_BLK_REQ_SL, flags, phdr[H2_PHDR_IDX_METH], uri, ist("HTTP/2.0"));
 	if (!sl)
 		goto fail;
 
 	sl->info.req.meth = find_http_meth(phdr[H2_PHDR_IDX_METH].ptr, phdr[H2_PHDR_IDX_METH].len);
-	sl->flags |= HTX_SL_F_HAS_SCHM;
-	sl->flags |= (isteqi(phdr[H2_PHDR_IDX_SCHM], ist("http")) ? HTX_SL_F_SCHM_HTTP : HTX_SL_F_SCHM_HTTPS);
 	return sl;
  fail:
 	return NULL;
@@ -678,12 +313,8 @@ static struct htx_sl *h2_prepare_htx_reqline(uint32_t fields, struct ist *phdr,
  *
  * The Cookie header will be reassembled at the end, and for this, the <list>
  * will be used to create a linked list, so its contents may be destroyed.
- *
- * When <relaxed> is non-nul, some non-dangerous checks will be ignored. This
- * is in order to satisfy "option accept-invalid-http-request" for
- * interoperability purposes.
  */
-int h2_make_htx_request(struct http_hdr *list, struct htx *htx, unsigned int *msgf, unsigned long long *body_len, int relaxed)
+int h2_make_htx_request(struct http_hdr *list, struct htx *htx, unsigned int *msgf, unsigned long long *body_len)
 {
 	struct ist phdr_val[H2_PHDR_NUM_ENTRIES];
 	uint32_t fields; /* bit mask of H2_PHDR_FND_* */
@@ -720,18 +351,11 @@ int h2_make_htx_request(struct http_hdr *list, struct htx *htx, unsigned int *ms
 		}
 
 		/* RFC7540#10.3: intermediaries forwarding to HTTP/1 must take care of
-		 * rejecting NUL, CR and LF characters. For :path we reject all CTL
-		 * chars, spaces, and '#'.
+		 * rejecting NUL, CR and LF characters.
 		 */
-		if (phdr == H2_PHDR_IDX_PATH && !relaxed) {
-			ctl = ist_find_range(list[idx].v, 0, '#');
-			if (unlikely(ctl) && http_path_has_forbidden_char(list[idx].v, ctl))
-				goto fail;
-		} else {
-			ctl = ist_find_ctl(list[idx].v);
-			if (unlikely(ctl) && has_forbidden_char(list[idx].v, ctl))
-				goto fail;
-		}
+		ctl = ist_find_ctl(list[idx].v);
+		if (unlikely(ctl) && has_forbidden_char(list[idx].v, ctl))
+			goto fail;
 
 		if (phdr > 0 && phdr < H2_PHDR_NUM_ENTRIES) {
 			/* insert a pseudo header by its index (in phdr) and value (in value) */
