@@ -14,9 +14,6 @@
  * in an environment able to catch a longjmp, otherwise a
  * critical error can be raised.
  */
-
-#define _GNU_SOURCE
-
 #include <lauxlib.h>
 #include <lua.h>
 #include <lualib.h>
@@ -172,13 +169,12 @@ int hlua_fcn_pushfield(lua_State *L, struct field *field)
 const char *hlua_pushstrippedstring(lua_State *L, const char *str)
 {
 	const char *p;
-	int l;
+	const char *e;
 
 	for (p = str; HTTP_IS_LWS(*p); p++);
+	for (e = p + strlen(p) - 1; e > p && HTTP_IS_LWS(*e); e--);
 
-	for (l = strlen(p); l && HTTP_IS_LWS(p[l-1]); l--);
-
-	return lua_pushlstring(L, p, l);
+	return lua_pushlstring(L, p, e - p);
 }
 
 /* The three following functions are useful for adding entries
@@ -360,7 +356,7 @@ static int hlua_get_info(lua_State *L)
 
 	lua_newtable(L);
 	for (i=0; i<INF_TOTAL_FIELDS; i++) {
-		lua_pushstring(L, info_field_names[i]);
+		lua_pushstring(L, info_fields[i].name);
 		hlua_fcn_pushfield(L, &stats[i]);
 		lua_settable(L, -3);
 	}
@@ -592,12 +588,6 @@ static void hlua_stktable_entry(lua_State *L, struct stktable *t, struct stksess
 			lua_pushinteger(L, read_freq_ctr_period(&stktable_data_cast(ptr, std_t_frqp),
 			                t->data_arg[dt].u));
 			break;
-		case STD_T_DICT: {
-			struct dict_entry *de;
-			de = stktable_data_cast(ptr, std_t_dict);
-			lua_pushstring(L, de ? (char *)de->value.key : "-");
-			break;
-		}
 		}
 
 		lua_settable(L, -3);
@@ -621,7 +611,7 @@ int hlua_stktable_lookup(lua_State *L)
 	t = hlua_check_stktable(L, 1);
 	smp.data.type = SMP_T_STR;
 	smp.flags = SMP_F_CONST;
-	smp.data.u.str.area = (char *)lua_tolstring(L, 2, &smp.data.u.str.data);
+	smp.data.u.str.area = (char *)luaL_checkstring(L, 2);
 
 	skey = smp_to_stkey(&smp, t);
 	if (!skey) {
@@ -868,11 +858,11 @@ int hlua_listener_get_stats(lua_State *L)
 		return 1;
 	}
 
-	stats_fill_li_stats(li->bind_conf->frontend, li, ST_SHLGNDS, stats, STATS_LEN);
+	stats_fill_li_stats(li->bind_conf->frontend, li, STAT_SHLGNDS, stats, STATS_LEN);
 
 	lua_newtable(L);
 	for (i=0; i<ST_F_TOTAL_FIELDS; i++) {
-		lua_pushstring(L, stat_field_names[i]);
+		lua_pushstring(L, stat_fields[i].name);
 		hlua_fcn_pushfield(L, &stats[i]);
 		lua_settable(L, -3);
 	}
@@ -924,11 +914,11 @@ int hlua_server_get_stats(lua_State *L)
 		return 1;
 	}
 
-	stats_fill_sv_stats(srv->proxy, srv, ST_SHLGNDS, stats, STATS_LEN);
+	stats_fill_sv_stats(srv->proxy, srv, STAT_SHLGNDS, stats, STATS_LEN);
 
 	lua_newtable(L);
 	for (i=0; i<ST_F_TOTAL_FIELDS; i++) {
-		lua_pushstring(L, stat_field_names[i]);
+		lua_pushstring(L, stat_fields[i].name);
 		hlua_fcn_pushfield(L, &stats[i]);
 		lua_settable(L, -3);
 	}
@@ -1298,7 +1288,6 @@ int hlua_proxy_pause(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
-	/* safe to call without PROXY_LOCK - pause_proxy takes it */
 	pause_proxy(px);
 	return 0;
 }
@@ -1308,7 +1297,6 @@ int hlua_proxy_resume(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
-	/* safe to call without PROXY_LOCK - resume_proxy takes it */
 	resume_proxy(px);
 	return 0;
 }
@@ -1318,7 +1306,6 @@ int hlua_proxy_stop(lua_State *L)
 	struct proxy *px;
 
 	px = hlua_check_proxy(L, 1);
-	/* safe to call without PROXY_LOCK - stop_proxy takes it */
 	stop_proxy(px);
 	return 0;
 }
@@ -1341,12 +1328,12 @@ int hlua_proxy_get_stats(lua_State *L)
 
 	px = hlua_check_proxy(L, 1);
 	if (px->cap & PR_CAP_BE)
-		stats_fill_be_stats(px, ST_SHLGNDS, stats, STATS_LEN);
+		stats_fill_be_stats(px, STAT_SHLGNDS, stats, STATS_LEN);
 	else
 		stats_fill_fe_stats(px, stats, STATS_LEN);
 	lua_newtable(L);
 	for (i=0; i<ST_F_TOTAL_FIELDS; i++) {
-		lua_pushstring(L, stat_field_names[i]);
+		lua_pushstring(L, stat_fields[i].name);
 		hlua_fcn_pushfield(L, &stats[i]);
 		lua_settable(L, -3);
 	}
