@@ -34,7 +34,7 @@
 #include <proto/http_rules.h>
 #include <proto/http_htx.h>
 #include <proto/log.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/stream_interface.h>
 
 
@@ -62,8 +62,8 @@ static enum act_return http_action_set_req_line(struct act_rule *rule, struct pr
 				       replace->size - replace->data,
 				       &rule->arg.http.logfmt);
 
-	http_replace_req_line(rule->arg.http.action, replace->area,
-			      replace->data, px, s);
+	http_req_replace_stline(rule->arg.http.action, replace->area,
+				replace->data, px, s);
 
 	ret = ACT_RET_CONT;
 
@@ -150,17 +150,10 @@ static enum act_return http_action_replace_uri(struct act_rule *rule, struct pro
 	output  = alloc_trash_chunk();
 	if (!replace || !output)
 		goto leave;
-
-	if (IS_HTX_STRM(s))
-		uri = htx_sl_req_uri(http_get_stline(htxbuf(&s->req.buf)));
-	else
-		uri = ist2(ci_head(&s->req) + s->txn->req.sl.rq.u, s->txn->req.sl.rq.u_l);
+	uri = htx_sl_req_uri(http_get_stline(htxbuf(&s->req.buf)));
 
 	if (rule->arg.act.p[0] == (void *)1)
-		uri = iststop(http_get_path(uri), '?');
-
-	if (!istlen(uri))
-		goto leave;
+		uri = http_get_path(uri); // replace path
 
 	if (!regex_exec_match2(rule->arg.act.p[1], uri.ptr, uri.len, MAX_MATCH, pmatch, 0))
 		goto leave;
@@ -174,7 +167,7 @@ static enum act_return http_action_replace_uri(struct act_rule *rule, struct pro
 	if (len == -1)
 		goto leave;
 
-	http_replace_req_line((long)rule->arg.act.p[0], output->area, len, px, s);
+	http_req_replace_stline((long)rule->arg.act.p[0], output->area, len, px, s);
 
 	ret = ACT_RET_CONT;
 
@@ -231,7 +224,7 @@ static enum act_parse_ret parse_replace_uri(const char **args, int *orig_arg, st
 static enum act_return action_http_set_status(struct act_rule *rule, struct proxy *px,
                                               struct session *sess, struct stream *s, int flags)
 {
-	http_set_status(rule->arg.status.code, rule->arg.status.reason, s);
+	http_res_set_status(rule->arg.status.code, rule->arg.status.reason, s);
 	return ACT_RET_CONT;
 }
 
@@ -288,8 +281,8 @@ static enum act_return http_action_reject(struct act_rule *rule, struct proxy *p
 	si_must_kill_conn(chn_prod(&s->req));
 	channel_abort(&s->req);
 	channel_abort(&s->res);
-	s->req.analysers &= AN_REQ_FLT_END;
-	s->res.analysers &= AN_RES_FLT_END;
+	s->req.analysers = 0;
+	s->res.analysers = 0;
 
 	_HA_ATOMIC_ADD(&s->be->be_counters.denied_req, 1);
 	_HA_ATOMIC_ADD(&sess->fe->fe_counters.denied_req, 1);
@@ -510,11 +503,6 @@ static enum act_parse_ret parse_http_req_capture(const char **args, int *orig_ar
 		cur_arg++;
 
 		hdr = calloc(1, sizeof(*hdr));
-		if (!hdr) {
-			memprintf(err, "out of memory");
-			release_sample_expr(expr);
-			return ACT_RET_PRS_ERR;
-		}
 		hdr->next = px->req_cap;
 		hdr->name = NULL; /* not a header capture */
 		hdr->namelen = 0;
@@ -621,10 +609,7 @@ static int check_http_res_capture(struct act_rule *rule, struct proxy *px, char
 	if (rule->action_ptr != http_action_res_capture_by_id)
 		return 1;
 
-	/* capture slots can only be declared in frontends, so we can't check their
-	 * existence in backends at configuration parsing step
-	 */
-	if (px->cap & PR_CAP_FE && rule->arg.capid.idx >= px->nb_rsp_cap) {
+	if (rule->arg.capid.idx >= px->nb_rsp_cap) {
 		memprintf(err, "unable to find capture id '%d' referenced by http-response capture rule",
 			  rule->arg.capid.idx);
 		return 0;
