@@ -33,7 +33,7 @@
 #include <proto/action.h>
 #include <proto/arg.h>
 #include <proto/http_rules.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/sample.h>
 
 
@@ -50,7 +50,7 @@ struct action_kw_list http_res_keywords = {
 /*
  * Return the struct http_req_action_kw associated to a keyword.
  */
-struct action_kw *action_http_req_custom(const char *kw)
+static struct action_kw *action_http_req_custom(const char *kw)
 {
 	return action_lookup(&http_req_keywords.list, kw);
 }
@@ -58,7 +58,7 @@ struct action_kw *action_http_req_custom(const char *kw)
 /*
  * Return the struct http_res_action_kw associated to a keyword.
  */
-struct action_kw *action_http_res_custom(const char *kw)
+static struct action_kw *action_http_res_custom(const char *kw)
 {
 	return action_lookup(&http_res_keywords.list, kw);
 }
@@ -1037,7 +1037,7 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 	const char *destination = NULL;
 	const char *cookie = NULL;
 	int cookie_set = 0;
-	unsigned int flags = (!dir ? REDIRECT_FLAG_FROM_REQ : REDIRECT_FLAG_NONE);
+	unsigned int flags = REDIRECT_FLAG_NONE;
 	struct acl_cond *cond = NULL;
 
 	cur_arg = 0;
@@ -1129,10 +1129,6 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 	}
 
 	rule = calloc(1, sizeof(*rule));
-	if (!rule) {
-		memprintf(errmsg, "parsing [%s:%d]: out of memory.", file, linenum);
-		return NULL;
-	}
 	rule->cond = cond;
 	LIST_INIT(&rule->rdr_fmt);
 
