@@ -13,18 +13,15 @@
 #include <common/chunk.h>
 #include <common/htx.h>
 
-struct htx htx_empty = { .size = 0, .data = 0, .used = 0 };
+struct htx htx_empty = { .size = 0, .data = 0, .head  = -1, .tail = -1, .first = -1 };
 
 /* Defragments an HTX message. It removes unused blocks and unwraps the payloads
- * part. A temporary buffer is used to do so. This function never fails. Most of
- * time, we need keep a ref on a specific HTX block. Thus is <blk> is set, the
- * pointer on its new position, after defrag, is returned. In addition, if the
- * size of the block must be altered, <blkinfo> info must be provided (!=
- * 0). But in this case, it remains the caller responsibility to update the
- * block content.
+ * part. A temporary buffer is used to do so. This function never fails. if
+ * <blk> is not NULL, we replace it by the new block address, after the
+ * defragmentation. The new <blk> is returned.
  */
 /* TODO: merge data blocks into one */
-struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk, uint32_t blkinfo)
+struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk)
 {
 	struct buffer *chunk = get_trash_chunk();
 	struct htx *tmp = htxbuf(chunk);
@@ -33,7 +30,7 @@ struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk, uint32_t blkinf
 	uint32_t addr, blksz;
 	int32_t first = -1;
 
-	if (!htx->used)
+	if (htx->head == -1)
 		return NULL;
 
 	blkpos = -1;
@@ -41,7 +38,6 @@ struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk, uint32_t blkinf
 	new  = 0;
 	addr = 0;
 	tmp->size = htx->size;
-	tmp->data = 0;
 
 	/* start from the head */
 	for (old = htx_get_head(htx); old != -1; old = htx_get_next(htx, old)) {
@@ -49,38 +45,30 @@ struct htx_blk *htx_defrag(struct htx *htx, struct htx_blk *blk, uint32_t blkinf
 		if (htx_get_blk_type(oldblk) == HTX_BLK_UNUSED)
 			continue;
 
+		newblk = htx_get_blk(tmp, new);
+		newblk->addr = addr;
+		newblk->info = oldblk->info;
 		blksz = htx_get_blksz(oldblk);
-		memcpy((void *)tmp->blocks + addr, htx_get_blk_ptr(htx, oldblk), blksz);
 
 		/* update the start-line position */
 		if (htx->first == old)
 			first = new;
 
-		newblk = htx_get_blk(tmp, new);
-		newblk->addr = addr;
-		newblk->info = oldblk->info;
-
 		/* if <blk> is defined, save its new position */
-		if (blk != NULL && blk == oldblk) {
-			if (blkinfo)
-				newblk->info = blkinfo;
+		if (blk != NULL && blk == oldblk)
 			blkpos = new;
-		}
 
-		blksz = htx_get_blksz(newblk);
-		addr += blksz;
-		tmp->data += blksz;
+		memcpy((void *)tmp->blocks + addr, htx_get_blk_ptr(htx, oldblk), blksz);
 		new++;
+		addr += blksz;
+
 	}
 
-	htx->used = new;
-	htx->data = tmp->data;
 	htx->first = first;
 	htx->head = 0;
 	htx->tail = new - 1;
 	htx->head_addr = htx->end_addr = 0;
 	htx->tail_addr = addr;
-	htx->flags &= ~HTX_FL_FRAGMENTED;
 	memcpy((void *)htx->blocks, (void *)tmp->blocks, htx->size);
 
 	return ((blkpos == -1) ? NULL : htx_get_blk(htx, blkpos));
@@ -127,16 +115,14 @@ static void htx_defrag_blks(struct htx *htx)
 static struct htx_blk *htx_reserve_nxblk(struct htx *htx, uint32_t blksz)
 {
 	struct htx_blk *blk;
-	uint32_t used, tail;
-	uint32_t headroom, tailroom;
+	uint32_t tail, headroom, tailroom;
 
 	if (blksz > htx_free_data_space(htx))
 		return NULL; /* full */
 
-	if (!htx->used) {
+	if (htx->head == -1) {
 		/* Empty message */
 		htx->head = htx->tail = htx->first = 0;
-		htx->used = 1;
 		blk = htx_get_blk(htx, htx->tail);
 		blk->addr = 0;
 		htx->data = blksz;
@@ -151,13 +137,12 @@ static struct htx_blk *htx_reserve_nxblk(struct htx *htx, uint32_t blksz)
 	 * message.
 	 */
 	tail = htx->tail + 1;
-	if (sizeof(htx->blocks[0]) * htx_pos_to_idx(htx, tail) >= htx->tail_addr)
-		used = htx->used + 1;
-	else if (tail > htx->used) {
+	if (htx_pos_to_addr(htx, tail) >= htx->tail_addr)
+		;
+	else if (htx->head > 0) {
 		htx_defrag_blks(htx);
 		tail = htx->tail + 1;
-		used = htx->used + 1;
-		BUG_ON(sizeof(htx->blocks[0]) * htx_pos_to_idx(htx, tail) < htx->tail_addr);
+		BUG_ON(htx_pos_to_addr(htx, tail) < htx->tail_addr);
 	}
 	else
 		goto defrag;
@@ -172,9 +157,7 @@ static struct htx_blk *htx_reserve_nxblk(struct htx *htx, uint32_t blksz)
          *     used, the other one is never used again, until the next defrag.
 	 */
 	headroom = (htx->end_addr - htx->head_addr);
-	tailroom = (!htx->head_addr
-		    ? sizeof(htx->blocks[0]) * htx_pos_to_idx(htx, tail) - htx->tail_addr
-		    : 0);
+	tailroom = (!htx->head_addr ? htx_pos_to_addr(htx, tail) - htx->tail_addr : 0);
 	BUG_ON((int32_t)headroom < 0);
 	BUG_ON((int32_t)tailroom < 0);
 
@@ -191,16 +174,14 @@ static struct htx_blk *htx_reserve_nxblk(struct htx *htx, uint32_t blksz)
 	else {
 	  defrag:
 		/* need to defragment the message before inserting upfront */
-		htx_defrag(htx, NULL, 0);
+		htx_defrag(htx, NULL);
 		tail = htx->tail + 1;
-		used = htx->used + 1;
 		blk = htx_get_blk(htx, tail);
 		blk->addr = htx->tail_addr;
 		htx->tail_addr += blksz;
 	}
 
 	htx->tail  = tail;
-	htx->used  = used;
 	htx->data += blksz;
 	/* Set first position if not already set */
 	if (htx->first == -1)
@@ -237,8 +218,10 @@ static int htx_prepare_blk_expansion(struct htx *htx, struct htx_blk *blk, int32
 	uint32_t sz, tailroom, headroom;
 	int ret = 3;
 
+	BUG_ON(htx->head == -1);
+
 	headroom = (htx->end_addr - htx->head_addr);
-	tailroom = sizeof(htx->blocks[0]) * htx_pos_to_idx(htx, htx->tail) - htx->tail_addr;
+	tailroom = (htx_pos_to_addr(htx, htx->tail) - htx->tail_addr);
 	BUG_ON((int32_t)headroom < 0);
 	BUG_ON((int32_t)tailroom < 0);
 
@@ -323,7 +306,6 @@ struct htx_blk *htx_add_blk(struct htx *htx, enum htx_blk_type type, uint32_t bl
 {
 	struct htx_blk *blk;
 
-	BUG_ON(blksz >= 256 << 20);
 	blk = htx_reserve_nxblk(htx, blksz);
 	if (!blk)
 		return NULL;
@@ -342,12 +324,11 @@ struct htx_blk *htx_remove_blk(struct htx *htx, struct htx_blk *blk)
 	enum htx_blk_type type;
 	uint32_t pos, addr, sz;
 
-	/* This is the last block in use */
-	if (htx->used == 1) {
-		uint32_t flags = (htx->flags & ~HTX_FL_FRAGMENTED); /* Preserve flags except FRAGMENTED */
+	BUG_ON(htx->head == -1);
 
+	/* This is the last block in use */
+	if (htx->head == htx->tail) {
 		htx_reset(htx);
-		htx->flags = flags; /* restore flags */
 		return NULL;
 	}
 
@@ -364,27 +345,22 @@ struct htx_blk *htx_remove_blk(struct htx *htx, struct htx_blk *blk)
 	/* There is at least 2 blocks, so tail is always > 0 */
 	if (pos == htx->head) {
 		/* move the head forward */
-		htx->used--;
 		htx->head++;
 	}
 	else if (pos == htx->tail) {
 		/* remove the tail. this was the last inserted block so
 		 * return NULL. */
 		htx->tail--;
-		htx->used--;
 		blk = NULL;
 		goto end;
 	}
-	else
-		htx->flags |= HTX_FL_FRAGMENTED;
-
 	blk = htx_get_blk(htx, pos+1);
 
   end:
 	if (pos == htx->first)
 		htx->first = (blk ? htx_get_blk_pos(htx, blk) : -1);
 
-	if (htx->used == 1) {
+	if (htx->head == htx->tail) {
 		/* If there is just one block in the HTX message, free space can
 		 * be ajusted. This operation could save some defrags. */
 		struct htx_blk *lastblk = htx_get_blk(htx, htx->tail);
@@ -415,31 +391,6 @@ struct htx_blk *htx_remove_blk(struct htx *htx, struct htx_blk *blk)
 	return blk;
 }
 
-/* Looks for the HTX block containing the offset <offset>, starting at the HTX
- * message's head. The function returns an htx_ret with the found HTX block and
- * the position inside this block where the offset is. If the offset <offset> is
- * ouside of the HTX message, htx_ret.blk is set to NULL.
- */
-struct htx_ret htx_find_offset(struct htx *htx, uint32_t offset)
-{
-	struct htx_blk *blk;
-	struct htx_ret htxret = { .blk = NULL, .ret = 0 };
-
-	if (offset >= htx->data)
-		return htxret;
-
-	for (blk = htx_get_head_blk(htx); blk && offset; blk = htx_get_next_blk(htx, blk)) {
-		uint32_t sz = htx_get_blksz(blk);
-
-		if (offset < sz)
-			break;
-		offset -= sz;
-	}
-	htxret.blk = blk;
-	htxret.ret = offset;
-	return htxret;
-}
-
 /* Removes all blocks after the one containing the offset <offset>. This last
  * one may be truncated if it is a DATA block.
  */
@@ -475,10 +426,7 @@ struct htx_ret htx_drain(struct htx *htx, uint32_t count)
 	struct htx_ret htxret = { .blk = NULL, .ret = 0 };
 
 	if (count == htx->data) {
-		uint32_t flags = (htx->flags & ~HTX_FL_FRAGMENTED); /* Preserve flags except FRAGMENTED */
-
 		htx_reset(htx);
-		htx->flags = flags; /* restore flags */
 		htxret.ret = count;
 		return htxret;
 	}
@@ -522,7 +470,7 @@ struct htx_blk *htx_add_data_atonce(struct htx *htx, struct ist data)
 	void *ptr;
 	uint32_t len, sz, tailroom, headroom;
 
-	if (!htx->used)
+	if (htx->head == -1)
 		goto add_new_block;
 
 	/* Not enough space to store data */
@@ -544,7 +492,7 @@ struct htx_blk *htx_add_data_atonce(struct htx *htx, struct ist data)
 	 * Same type and enough space: append data
 	 */
 	headroom = (htx->end_addr - htx->head_addr);
-	tailroom = sizeof(htx->blocks[0]) * htx_pos_to_idx(htx, htx->tail) - htx->tail_addr;
+	tailroom = (htx_pos_to_addr(htx, htx->tail) - htx->tail_addr);
 	BUG_ON((int32_t)headroom < 0);
 	BUG_ON((int32_t)tailroom < 0);
 
@@ -563,6 +511,7 @@ struct htx_blk *htx_add_data_atonce(struct htx *htx, struct ist data)
 	goto add_new_block;
 
   append_data:
+	/* FIXME: check v.len + data.len < 256MB */
 	/* Append data and update the block itself */
 	ptr = htx_get_blk_ptr(htx, tailblk);
 	memcpy(ptr+sz, data.ptr, len);
@@ -576,6 +525,7 @@ struct htx_blk *htx_add_data_atonce(struct htx *htx, struct ist data)
 	data.len -= len;
 
   add_new_block:
+	/* FIXME: check data.len (< 256MB) */
 	blk = htx_add_blk(htx, HTX_BLK_DATA, data.len);
 	if (!blk)
 		return NULL;
@@ -610,6 +560,11 @@ struct htx_blk *htx_replace_blk_value(struct htx *htx, struct htx_blk *blk,
 	if (!ret)
 		return NULL; /* not enough space */
 
+	/* Before updating the payload, set the new block size and update HTX
+	 * message */
+	htx_set_blk_value_len(blk, v.len + delta);
+	htx->data += delta;
+
 	if (ret == 1) { /* Replace in place */
 		if (delta <= 0) {
 			/* compression: copy new data first then move the end */
@@ -621,10 +576,6 @@ struct htx_blk *htx_replace_blk_value(struct htx *htx, struct htx_blk *blk,
 			memmove(old.ptr + new.len, old.ptr + old.len, (v.ptr + v.len) - (old.ptr + old.len));
 			memcpy(old.ptr, new.ptr, new.len);
 		}
-
-		/* set the new block size and update HTX message */
-		htx_set_blk_value_len(blk, v.len + delta);
-		htx->data += delta;
 	}
 	else if (ret == 2) { /* New address but no defrag */
 		void *ptr = htx_get_blk_ptr(htx, blk);
@@ -643,32 +594,26 @@ struct htx_blk *htx_replace_blk_value(struct htx *htx, struct htx_blk *blk,
 
 		/* Copy value after old part, if any */
 		memcpy(ptr, old.ptr + old.len, (v.ptr + v.len) - (old.ptr + old.len));
-
-		/* set the new block size and update HTX message */
-		htx_set_blk_value_len(blk, v.len + delta);
-		htx->data += delta;
 	}
-	else { /* Do a degrag first (it is always an expansion) */
-		struct htx_blk tmpblk;
-		int32_t offset;
+	else { /* Do a degrag first */
+		struct buffer *tmp = get_trash_chunk();
 
-		/* use tmpblk to set new block size before defrag and to compute
-		 * the offset after defrag
-		 */
-		tmpblk.addr = blk->addr;
-		tmpblk.info = blk->info;
-		htx_set_blk_value_len(&tmpblk, v.len + delta);
+		/* Copy the header name, if any */
+		chunk_memcat(tmp, n.ptr, n.len);
 
-		/* htx_defrag() will take care to update the block size and the htx message */
-		blk = htx_defrag(htx, blk, tmpblk.info);
+		/* Copy value before old part, if any */
+		chunk_memcat(tmp, v.ptr, old.ptr - v.ptr);
 
-		/* newblk is now the new HTX block. Compute the offset to copy/move payload */
-		offset = blk->addr - tmpblk.addr;
+		/* Copy new value */
+		chunk_memcat(tmp, new.ptr, new.len);
 
-		/* move the end first and copy new data
-		 */
-		memmove(old.ptr + offset + new.len, old.ptr + offset + old.len, (v.ptr + v.len) - (old.ptr + old.len));
-		memcpy(old.ptr + offset, new.ptr, new.len);
+		/* Copy value after old part if any */
+                chunk_memcat(tmp, old.ptr + old.len, (v.ptr + v.len) - (old.ptr + old.len));
+
+		blk = htx_defrag(htx, blk);
+
+		/* Finaly, copy data. */
+		memcpy(htx_get_blk_ptr(htx, blk), tmp->area, tmp->data);
 	}
 	return blk;
 }
@@ -778,17 +723,15 @@ struct htx_blk *htx_replace_header(struct htx *htx, struct htx_blk *blk,
 	if (!ret)
 		return NULL; /* not enough space */
 
+	/* Set the new block size and update HTX message */
+	blk->info = (type << 28) + (value.len << 8) + name.len;
+	htx->data += delta;
 
 	/* Replace in place or at a new address is the same. We replace all the
 	 * header (name+value). Only take care to defrag the message if
 	 * necessary. */
 	if (ret == 3)
-		blk = htx_defrag(htx, blk, (type << 28) + (value.len << 8) + name.len);
-	else {
-		/* Set the new block size and update HTX message */
-		blk->info = (type << 28) + (value.len << 8) + name.len;
-		htx->data += delta;
-	}
+		blk = htx_defrag(htx, blk);
 
 	/* Finaly, copy data. */
 	ptr = htx_get_blk_ptr(htx, blk);
@@ -827,16 +770,14 @@ struct htx_sl *htx_replace_stline(struct htx *htx, struct htx_blk *blk, const st
 	if (!ret)
 		return NULL; /* not enough space */
 
+	/* Set the new block size and update HTX message */
+	htx_set_blk_value_len(blk, sz+delta);
+	htx->data += delta;
+
 	/* Replace in place or at a new address is the same. We replace all the
 	 * start-line. Only take care to defrag the message if necessary. */
-	if (ret == 3)  {
-		blk = htx_defrag(htx, blk, (type << 28) + sz + delta);
-	}
-	else {
-		/* Set the new block size and update HTX message */
-		blk->info = (type << 28) + sz + delta;
-		htx->data += delta;
-	}
+	if (ret == 3)
+		blk = htx_defrag(htx, blk);
 
 	/* Restore start-line info and flags and copy parts of the start-line */
 	sl = htx_get_blk_ptr(htx, blk);
@@ -870,6 +811,7 @@ struct htx_sl *htx_add_stline(struct htx *htx, enum htx_blk_type type, unsigned
 
 	size = sizeof(*sl) + p1.len + p2.len + p3.len;
 
+	/* FIXME: check size (< 256MB) */
 	blk = htx_add_blk(htx, type, size);
 	if (!blk)
 		return NULL;
@@ -898,9 +840,7 @@ struct htx_blk *htx_add_header(struct htx *htx, const struct ist name,
 {
 	struct htx_blk *blk;
 
-	if (name.len > 255 || value.len > 1048575)
-		return NULL;
-
+	/* FIXME: check name.len (< 256B) and value.len (< 1MB) */
 	blk = htx_add_blk(htx, HTX_BLK_HDR, name.len + value.len);
 	if (!blk)
 		return NULL;
@@ -919,9 +859,7 @@ struct htx_blk *htx_add_trailer(struct htx *htx, const struct ist name,
 {
 	struct htx_blk *blk;
 
-	if (name.len > 255 || value.len > 1048575)
-		return NULL;
-
+	/* FIXME: check name.len (< 256B) and value.len (< 1MB) */
 	blk = htx_add_blk(htx, HTX_BLK_TLR, name.len + value.len);
 	if (!blk)
 		return NULL;
@@ -932,22 +870,6 @@ struct htx_blk *htx_add_trailer(struct htx *htx, const struct ist name,
 	return blk;
 }
 
-/* Adds an HTX block of type <type> in <htx>, of size <blksz>. It returns the
- * new block on success. Otherwise, it returns NULL. The caller is responsible
- * for filling the block itself.
- */
-struct htx_blk *htx_add_blk_type_size(struct htx *htx, enum htx_blk_type type, uint32_t blksz)
-{
-	struct htx_blk *blk;
-
-	blk = htx_add_blk(htx, type, blksz);
-	if (!blk)
-		return NULL;
-
-	blk->info += blksz;
-	return blk;
-}
-
 /* Add all headers from the list <hdrs> into the HTX message <htx>, followed by
  * the EOH. On sucess, it returns the last block inserted (the EOH), otherwise
  * NULL is returned. */
@@ -1003,6 +925,9 @@ size_t htx_add_data(struct htx *htx, const struct ist data)
 	uint32_t sz, room;
 	int32_t len = data.len;
 
+	if (htx->head == -1)
+		goto add_new_block;
+
 	/* Not enough space to store data */
 	if (len > htx_free_data_space(htx))
 		len = htx_free_data_space(htx);
@@ -1010,9 +935,6 @@ size_t htx_add_data(struct htx *htx, const struct ist data)
 	if (!len)
 		return 0;
 
-	if (!htx->used)
-		goto add_new_block;
-
 	/* get the tail and head block */
 	tailblk = htx_get_tail_blk(htx);
 	if (tailblk == NULL)
@@ -1030,7 +952,7 @@ size_t htx_add_data(struct htx *htx, const struct ist data)
 	if (!htx->head_addr) {
 		if (tailblk->addr+sz != htx->tail_addr)
 			goto add_new_block;
-		room = sizeof(htx->blocks[0]) * htx_pos_to_idx(htx, htx->tail) - htx->tail_addr;
+		room = (htx_pos_to_addr(htx, htx->tail) - htx->tail_addr);
 	}
 	else {
 		if (tailblk->addr+sz != htx->head_addr)
@@ -1042,6 +964,7 @@ size_t htx_add_data(struct htx *htx, const struct ist data)
 		len = room;
 
   append_data:
+	/* FIXME: check v.len + data.len < 256MB */
 	/* Append data and update the block itself */
 	ptr = htx_get_blk_ptr(htx, tailblk);
 	memcpy(ptr + sz, data.ptr, len);
@@ -1054,6 +977,7 @@ size_t htx_add_data(struct htx *htx, const struct ist data)
 	return len;
 
   add_new_block:
+	/* FIXME: check data.len (< 256MB) */
 	blk = htx_add_blk(htx, HTX_BLK_DATA, len);
 	if (!blk)
 		return 0;
@@ -1117,129 +1041,33 @@ void htx_move_blk_before(struct htx *htx, struct htx_blk **blk, struct htx_blk *
 	*ref = pblk;
 }
 
-/* Appends the H1 representation of the request line <sl> to the chunk <chk>. It
- * returns 1 if data are successfully appended, otherwise it returns 0.
+/* Append the HTX message <src> to the HTX message <dst>. It returns 1 on
+ * success and 0 on error.  All the message or nothing is copied. If an error
+ * occurred, all blocks from <src> already appended to <dst> are truncated.
  */
-int htx_reqline_to_h1(const struct htx_sl *sl, struct buffer *chk)
+int htx_append_msg(struct htx *dst, const struct htx *src)
 {
-	size_t sz = chk->data;
+	struct htx_blk *blk, *newblk;
+	enum htx_blk_type type;
+	uint32_t blksz, offset = dst->data;
 
-	if (!chunk_memcat(chk, HTX_SL_REQ_MPTR(sl), HTX_SL_REQ_MLEN(sl)) ||
-	    !chunk_memcat(chk, " ", 1) ||
-	    !chunk_memcat(chk, HTX_SL_REQ_UPTR(sl), HTX_SL_REQ_ULEN(sl)) ||
-	    !chunk_memcat(chk, " ", 1))
-		goto full;
+	for (blk = htx_get_head_blk(src); blk; blk = htx_get_next_blk(src, blk)) {
+		type = htx_get_blk_type(blk);
 
-	if (sl->flags & HTX_SL_F_VER_11) {
-		if (!chunk_memcat(chk, "HTTP/1.1", 8))
-			goto full;
-	}
-	else {
-		if (!chunk_memcat(chk, HTX_SL_REQ_VPTR(sl), HTX_SL_REQ_VLEN(sl)))
-			goto full;
-	}
+		if (type == HTX_BLK_UNUSED)
+			continue;
 
-	if (!chunk_memcat(chk, "\r\n", 2))
-		goto full;
-
-	return 1;
-
-  full:
-	chk->data = sz;
-	return 0;
-}
-
-/* Appends the H1 representation of the status line <sl> to the chunk <chk>. It
- * returns 1 if data are successfully appended, otherwise it returns 0.
- */
-int htx_stline_to_h1(const struct htx_sl *sl, struct buffer *chk)
-{
-	size_t sz = chk->data;
-
-	if (HTX_SL_LEN(sl) + 4 > b_room(chk))
-		return 0;
-
-	if (sl->flags & HTX_SL_F_VER_11) {
-		if (!chunk_memcat(chk, "HTTP/1.1", 8))
-			goto full;
-	}
-	else {
-		if (!chunk_memcat(chk, HTX_SL_RES_VPTR(sl), HTX_SL_RES_VLEN(sl)))
-			goto full;
-	}
-	if (!chunk_memcat(chk, " ", 1) ||
-	    !chunk_memcat(chk, HTX_SL_RES_CPTR(sl), HTX_SL_RES_CLEN(sl)) ||
-	    !chunk_memcat(chk, " ", 1) ||
-	    !chunk_memcat(chk, HTX_SL_RES_RPTR(sl), HTX_SL_RES_RLEN(sl)) ||
-	    !chunk_memcat(chk, "\r\n", 2))
-		goto full;
-
-	return 1;
-
-  full:
-	chk->data = sz;
-	return 0;
-}
-
-/* Appends the H1 representation of the header <n> witht the value <v> to the
- * chunk <chk>. It returns 1 if data are successfully appended, otherwise it
- * returns 0.
- */
-int htx_hdr_to_h1(const struct ist n, const struct ist v, struct buffer *chk)
-{
-	size_t sz = chk->data;
-
-	if (n.len + v.len + 4 > b_room(chk))
-		return 0;
-
-	if (!chunk_memcat(chk, n.ptr, n.len) ||
-	    !chunk_memcat(chk, ": ", 2) ||
-	    !chunk_memcat(chk, v.ptr, v.len) ||
-	    !chunk_memcat(chk, "\r\n", 2))
-		goto full;
-
-	return 1;
-
-  full:
-	chk->data = sz;
-	return 0;
-}
-
-/* Appends the H1 representation of the data <data> to the chunk <chk>. If
- * <chunked> is non-zero, it emits HTTP/1 chunk-encoded data. It returns 1 if
- * data are successfully appended, otherwise it returns 0.
- */
-int htx_data_to_h1(const struct ist data, struct buffer *chk, int chunked)
-{
-	size_t sz = chk->data;
-
-	if (chunked) {
-		uint32_t chksz;
-		char     tmp[10];
-		char    *beg, *end;
-
-		chksz = data.len;
-
-		beg = end = tmp+10;
-		*--beg = '\n';
-		*--beg = '\r';
-		do {
-			*--beg = hextab[chksz & 0xF];
-		} while (chksz >>= 4);
-
-		if (!chunk_memcat(chk, beg, end - beg) ||
-		    !chunk_memcat(chk, data.ptr, data.len) ||
-		    !chunk_memcat(chk, "\r\n", 2))
-			goto full;
-	}
-	else {
-		if (!chunk_memcat(chk, data.ptr, data.len))
-			return 0;
+		blksz = htx_get_blksz(blk);
+		newblk = htx_add_blk(dst, type, blksz);
+		if (!newblk)
+			goto error;
+		newblk->info = blk->info;
+		memcpy(htx_get_blk_ptr(dst, newblk), htx_get_blk_ptr(src, blk), blksz);
 	}
 
 	return 1;
 
-  full:
-	chk->data = sz;
+  error:
+	htx_truncate(dst, offset);
 	return 0;
 }
