@@ -94,11 +94,7 @@ struct pool_head *create_pool(char *name, unsigned int size, unsigned int flags)
 			 * we look for a sharable one or for the next position
 			 * before which we will insert a new one.
 			 */
-			if ((flags & entry->flags & MEM_F_SHARED)
-#ifdef DEBUG_DONT_SHARE_POOLS
-			    && strcmp(name, entry->name) == 0
-#endif
-			    ) {
+			if (flags & entry->flags & MEM_F_SHARED) {
 				/* we can share this one */
 				pool = entry;
 				DPRINTF(stderr, "Sharing %s with %s\n", name, pool->name);
@@ -145,6 +141,8 @@ struct pool_head *create_pool(char *name, unsigned int size, unsigned int flags)
 		}
 #ifndef CONFIG_HAP_LOCKLESS_POOLS
 		HA_SPIN_INIT(&pool->lock);
+#else
+		HA_RWLOCK_INIT(&pool->flush_lock);
 #endif
 	}
 	pool->users++;
@@ -192,16 +190,11 @@ void *__pool_refill_alloc(struct pool_head *pool, unsigned int avail)
 		if (++allocated > avail)
 			break;
 
-		free_list = _HA_ATOMIC_LOAD(&pool->free_list);
+		free_list = pool->free_list;
 		do {
-			while (unlikely(free_list == POOL_BUSY)) {
-				pl_cpu_relax();
-				free_list = _HA_ATOMIC_LOAD(&pool->free_list);
-			}
-			_HA_ATOMIC_STORE(POOL_LINK(pool, ptr), (void *)free_list);
-			__ha_barrier_atomic_store();
-		} while (!_HA_ATOMIC_CAS(&pool->free_list, &free_list, ptr));
-		__ha_barrier_atomic_store();
+			*POOL_LINK(pool, ptr) = free_list;
+			__ha_barrier_store();
+		} while (_HA_ATOMIC_CAS(&pool->free_list, &free_list, ptr) == 0);
 	}
 	__ha_barrier_atomic_store();
 
@@ -226,64 +219,71 @@ void *pool_refill_alloc(struct pool_head *pool, unsigned int avail)
  */
 void pool_flush(struct pool_head *pool)
 {
+	struct pool_free_list cmp, new;
 	void **next, *temp;
 	int removed = 0;
 
 	if (!pool)
 		return;
-
-	/* The loop below atomically detaches the head of the free list and
-	 * replaces it with a NULL. Then the list can be released.
-	 */
-	next = pool->free_list;
+	HA_RWLOCK_WRLOCK(POOL_LOCK, &pool->flush_lock);
 	do {
-		while (unlikely(next == POOL_BUSY)) {
-			pl_cpu_relax();
-			next = _HA_ATOMIC_LOAD(&pool->free_list);
-		}
-		if (next == NULL)
-			return;
-	} while (unlikely((next = _HA_ATOMIC_XCHG(&pool->free_list, POOL_BUSY)) == POOL_BUSY));
-	_HA_ATOMIC_STORE(&pool->free_list, NULL);
+		cmp.free_list = pool->free_list;
+		cmp.seq = pool->seq;
+		new.free_list = NULL;
+		new.seq = cmp.seq + 1;
+	} while (!_HA_ATOMIC_DWCAS(&pool->free_list, &cmp, &new));
 	__ha_barrier_atomic_store();
-
+	HA_RWLOCK_WRUNLOCK(POOL_LOCK, &pool->flush_lock);
+	next = cmp.free_list;
 	while (next) {
 		temp = next;
 		next = *POOL_LINK(pool, temp);
 		removed++;
 		free(temp);
 	}
+	pool->free_list = next;
 	_HA_ATOMIC_SUB(&pool->allocated, removed);
-	/* here, we should have pool->allocated == pool->used */
+	/* here, we should have pool->allocate == pool->used */
 }
 
 /*
  * This function frees whatever can be freed in all pools, but respecting
- * the minimum thresholds imposed by owners. It makes sure to be alone to
- * run by using thread_isolate(). <pool_ctx> is unused.
+ * the minimum thresholds imposed by owners. It takes care of avoiding
+ * recursion because it may be called from a signal handler.
+ *
+ * <pool_ctx> is unused
  */
 void pool_gc(struct pool_head *pool_ctx)
 {
+	static int recurse;
+	int cur_recurse = 0;
 	struct pool_head *entry;
-	int isolated = thread_isolated();
 
-	if (!isolated)
-		thread_isolate();
+	if (recurse || !_HA_ATOMIC_CAS(&recurse, &cur_recurse, 1))
+		return;
 
 	list_for_each_entry(entry, &pools, list) {
-		void *temp;
-		//qfprintf(stderr, "Flushing pool %s\n", entry->name);
-		while (entry->free_list &&
-		       (int)(entry->allocated - entry->used) > (int)entry->minavail) {
-			temp = entry->free_list;
-			entry->free_list = *POOL_LINK(entry, temp);
-			entry->allocated--;
-			free(temp);
+		HA_RWLOCK_WRLOCK(POOL_LOCK, &entry->flush_lock);
+		while ((int)((volatile int)entry->allocated - (volatile int)entry->used) > (int)entry->minavail) {
+			struct pool_free_list cmp, new;
+
+			cmp.seq = entry->seq;
+			__ha_barrier_load();
+			cmp.free_list = entry->free_list;
+			__ha_barrier_load();
+			if (cmp.free_list == NULL)
+				break;
+			new.free_list = *POOL_LINK(entry, cmp.free_list);
+			new.seq = cmp.seq + 1;
+			if (HA_ATOMIC_DWCAS(&entry->free_list, &cmp, &new) == 0)
+				continue;
+			free(cmp.free_list);
+			_HA_ATOMIC_SUB(&entry->allocated, 1);
 		}
+		HA_RWLOCK_WRUNLOCK(POOL_LOCK, &entry->flush_lock);
 	}
 
-	if (!isolated)
-		thread_release();
+	_HA_ATOMIC_STORE(&recurse, 0);
 }
 
 /* frees an object to the local cache, possibly pushing oldest objects to the
@@ -346,7 +346,17 @@ void *__pool_refill_alloc(struct pool_head *pool, unsigned int avail)
 			return NULL;
 		}
 
+		HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
 		ptr = pool_alloc_area(pool->size + POOL_EXTRA);
+#ifdef DEBUG_MEMORY_POOLS
+		/* keep track of where the element was allocated from. This
+		 * is done out of the lock so that the system really allocates
+		 * the data without harming other threads waiting on the lock.
+		 */
+		if (ptr)
+			*POOL_LINK(pool, ptr) = (void *)pool;
+#endif
+		HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
 		if (!ptr) {
 			pool->failed++;
 			if (failed) {
@@ -364,10 +374,6 @@ void *__pool_refill_alloc(struct pool_head *pool, unsigned int avail)
 		pool->free_list = ptr;
 	}
 	pool->used++;
-#ifdef DEBUG_MEMORY_POOLS
-	/* keep track of where the element was allocated from */
-	*POOL_LINK(pool, ptr) = (void *)pool;
-#endif
 	return ptr;
 }
 void *pool_refill_alloc(struct pool_head *pool, unsigned int avail)
@@ -384,58 +390,67 @@ void *pool_refill_alloc(struct pool_head *pool, unsigned int avail)
  */
 void pool_flush(struct pool_head *pool)
 {
-	void *temp, **next;
+	void *temp;
 
 	if (!pool)
 		return;
 
-	HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
-	next = pool->free_list;
-	while (next) {
-		temp = next;
-		next = *POOL_LINK(pool, temp);
+	while (1) {
+		HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
+		temp = pool->free_list;
+		if (!temp) {
+			HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
+			break;
+		}
+		pool->free_list = *POOL_LINK(pool, temp);
 		pool->allocated--;
-	}
-
-	next = pool->free_list;
-	pool->free_list = NULL;
-	HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
-
-	while (next) {
-		temp = next;
-		next = *POOL_LINK(pool, temp);
+		HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
 		pool_free_area(temp, pool->size + POOL_EXTRA);
 	}
-	/* here, we should have pool->allocate == pool->used */
+	/* here, we should have pool->allocated == pool->used */
 }
 
 /*
  * This function frees whatever can be freed in all pools, but respecting
- * the minimum thresholds imposed by owners. It makes sure to be alone to
- * run by using thread_isolate(). <pool_ctx> is unused.
+ * the minimum thresholds imposed by owners. It takes care of avoiding
+ * recursion because it may be called from a signal handler.
+ *
+ * <pool_ctx> is used when pool_gc is called to release resources to allocate
+ * an element in __pool_refill_alloc. It is important because <pool_ctx> is
+ * already locked, so we need to skip the lock here.
  */
 void pool_gc(struct pool_head *pool_ctx)
 {
+	static int recurse;
+	int cur_recurse = 0;
 	struct pool_head *entry;
-	int isolated = thread_isolated();
 
-	if (!isolated)
-		thread_isolate();
+	if (recurse || !_HA_ATOMIC_CAS(&recurse, &cur_recurse, 1))
+		return;
 
 	list_for_each_entry(entry, &pools, list) {
-		void *temp;
+		void *temp, *next;
 		//qfprintf(stderr, "Flushing pool %s\n", entry->name);
-		while (entry->free_list &&
+		if (entry != pool_ctx)
+			HA_SPIN_LOCK(POOL_LOCK, &entry->lock);
+		next = entry->free_list;
+		while (next &&
 		       (int)(entry->allocated - entry->used) > (int)entry->minavail) {
-			temp = entry->free_list;
-			entry->free_list = *POOL_LINK(entry, temp);
+			temp = next;
+			next = *POOL_LINK(entry, temp);
 			entry->allocated--;
+			if (entry != pool_ctx)
+				HA_SPIN_UNLOCK(POOL_LOCK, &entry->lock);
 			pool_free_area(temp, entry->size + POOL_EXTRA);
+			if (entry != pool_ctx)
+				HA_SPIN_LOCK(POOL_LOCK, &entry->lock);
 		}
+		entry->free_list = next;
+		if (entry != pool_ctx)
+			HA_SPIN_UNLOCK(POOL_LOCK, &entry->lock);
 	}
 
-	if (!isolated)
-		thread_release();
+	_HA_ATOMIC_STORE(&recurse, 0);
 }
 #endif
 
@@ -472,14 +487,8 @@ void pool_destroy_all()
 {
 	struct pool_head *entry, *back;
 
-	list_for_each_entry_safe(entry, back, &pools, list) {
-		/* there's only one occurrence of each pool in the list,
-		 * and we're existing instead of looping on the whole
-		 * list just to decrement users, force it to 1 here.
-		 */
-		entry->users = 1;
+	list_for_each_entry_safe(entry, back, &pools, list)
 		pool_destroy(entry);
-	}
 }
 
 /* This function dumps memory usage information into the trash buffer. */
@@ -495,7 +504,7 @@ void dump_pools_to_trash()
 #ifndef CONFIG_HAP_LOCKLESS_POOLS
 		HA_SPIN_LOCK(POOL_LOCK, &entry->lock);
 #endif
-		chunk_appendf(&trash, "  - Pool %s (%u bytes) : %u allocated (%u bytes), %u used, %u failures, %u users, @%p=%02d%s\n",
+		chunk_appendf(&trash, "  - Pool %s (%d bytes) : %d allocated (%u bytes), %d used, %d failures, %d users, @%p=%02d%s\n",
 			 entry->name, entry->size, entry->allocated,
 		         entry->size * entry->allocated, entry->used, entry->failed,
 			 entry->users, entry, (int)pool_get_index(entry),
@@ -531,24 +540,24 @@ int pool_total_failures()
 }
 
 /* This function returns the total amount of memory allocated in pools (in bytes) */
-unsigned long long pool_total_allocated()
+unsigned long pool_total_allocated()
 {
 	struct pool_head *entry;
-	unsigned long long allocated = 0;
+	unsigned long allocated = 0;
 
 	list_for_each_entry(entry, &pools, list)
-		allocated += entry->allocated * (unsigned long long)entry->size;
+		allocated += entry->allocated * entry->size;
 	return allocated;
 }
 
 /* This function returns the total amount of memory used in pools (in bytes) */
-unsigned long long pool_total_used()
+unsigned long pool_total_used()
 {
 	struct pool_head *entry;
-	unsigned long long used = 0;
+	unsigned long used = 0;
 
 	list_for_each_entry(entry, &pools, list)
-		used += entry->used * (unsigned long long)entry->size;
+		used += entry->used * entry->size;
 	return used;
 }
 
@@ -619,7 +628,7 @@ int mem_should_fail(const struct pool_head *pool)
 	int n;
 
 	if (mem_fail_rate > 0 && !(global.mode & MODE_STARTING)) {
-		int randnb = ha_random() % 100;
+		int randnb = random() % 100;
 
 		if (mem_fail_rate > randnb)
 			ret = 1;
