@@ -26,31 +26,23 @@ DECLARE_STATIC_POOL(pool_head_pt_ctx, "mux_pt", sizeof(struct mux_pt_ctx));
 
 static void mux_pt_destroy(struct mux_pt_ctx *ctx)
 {
-	struct connection *conn = NULL;
-
-	if (ctx) {
-		/* The connection must be attached to this mux to be released */
-		if (ctx->conn && ctx->conn->ctx == ctx)
-			conn = ctx->conn;
-
-		tasklet_free(ctx->wait_event.tasklet);
-
-		if (conn && ctx->wait_event.events != 0)
-			conn->xprt->unsubscribe(conn, conn->xprt_ctx, ctx->wait_event.events,
-						&ctx->wait_event);
-		pool_free(pool_head_pt_ctx, ctx);
-	}
-
-	if (conn) {
-		conn->mux = NULL;
-		conn->ctx = NULL;
+	/* The connection must be aattached to this mux to be released */
+	if (ctx && ctx->conn && ctx->conn->ctx == ctx) {
+		struct connection *conn = ctx->conn;
 
 		conn_stop_tracking(conn);
 		conn_full_close(conn);
+		tasklet_free(ctx->wait_event.tasklet);
+		conn->mux = NULL;
+		conn->ctx = NULL;
 		if (conn->destroy_cb)
 			conn->destroy_cb(conn);
+		/* We don't bother unsubscribing here, as we're about to destroy
+		 * both the connection and the mux_pt_ctx
+		 */
 		conn_free(conn);
 	}
+	pool_free(pool_head_pt_ctx, ctx);
 }
 
 /* Callback, used when we get I/Os while in idle mode */
@@ -309,10 +301,6 @@ static size_t mux_pt_snd_buf(struct conn_stream *cs, struct buffer *buf, size_t
 
 	if (ret > 0)
 		b_del(buf, ret);
-
-	if (cs->conn->flags & CO_FL_ERROR)
-		cs->flags |= CS_FL_ERROR;
-
 	return ret;
 }
 
@@ -343,14 +331,7 @@ static int mux_pt_rcv_pipe(struct conn_stream *cs, struct pipe *pipe, unsigned i
 
 static int mux_pt_snd_pipe(struct conn_stream *cs, struct pipe *pipe)
 {
-	int ret;
-
-	ret = cs->conn->xprt->snd_pipe(cs->conn, cs->conn->xprt_ctx, pipe);
-
-	if (cs->conn->flags & CO_FL_ERROR)
-		cs->flags |= CS_FL_ERROR;
-
-	return ret;
+	return (cs->conn->xprt->snd_pipe(cs->conn, cs->conn->xprt_ctx, pipe));
 }
 #endif
 
@@ -394,6 +375,6 @@ const struct mux_ops mux_pt_ops = {
 
 /* PROT selection : default mux has empty name */
 static struct mux_proto_list mux_proto_pt =
-	{ .token = IST(""), .mode = PROTO_MODE_ANY, .side = PROTO_SIDE_BOTH, .mux = &mux_pt_ops };
+	{ .token = IST(""), .mode = PROTO_MODE_TCP, .side = PROTO_SIDE_BOTH, .mux = &mux_pt_ops };
 
 INITCALL1(STG_REGISTER, register_mux_proto, &mux_proto_pt);
